@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,20 +17,18 @@
    along with GNU Radius; if not, write to the Free Software Foundation, 
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#include <sysdep.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <stdio.h>
-#include <radius.h>
-#include <radpaths.h>
 #include <signal.h>
+
+#include <common.h>
 #include <cfg.h>
-#include <list.h>
+#include <radsql.h>
 
 /* Server data structures */
 struct radutmp; /* declared in radutmp.h */
-struct obstack;
 
 enum reload_what {
         reload_config,
@@ -111,20 +111,20 @@ struct queue_stat {
         size_t completed;
 };
 typedef struct queue_stat QUEUE_STAT[R_MAX];
-        
+
 typedef struct client {
-        UINT4                   ipaddr;
+	grad_netdef_t           netdef;
         char                    longname[MAX_LONGNAME+1];
         u_char                  *secret;
         char                    shortname[MAX_SHORTNAME+1];
 } CLIENT;
 
 typedef struct proxy_state {
-	UINT4 ref_ip;             /* Radius server IP */
-        UINT4 proxy_id;           /* Proxy ID assigned by the server */
-        UINT4 remote_ip;          /* Remote radius server IP */
-        UINT4 client_ip;          /* IP of the requesting client */ 
-        UINT4 id;                 /* Radius request ID */
+	grad_uint32_t ref_ip;             /* Radius server IP */
+        grad_uint32_t proxy_id;           /* Proxy ID assigned by the server */
+        grad_uint32_t remote_ip;          /* Remote radius server IP */
+        grad_uint32_t client_ip;          /* IP of the requesting client */ 
+        grad_uint32_t id;                 /* Radius request ID */
 } PROXY_STATE;
 
 typedef struct {
@@ -138,12 +138,14 @@ typedef struct {
  * Internal representation of a user's profile
  */
 typedef struct user_symbol {
-        struct user_symbol *next;
-        char *name;
-        int lineno;
-        int ordnum;
-        VALUE_PAIR *check;
-        VALUE_PAIR *reply;
+        struct user_symbol *next;  /* Link to the next entry */
+        char *name;                /* Label */
+        grad_avp_t *check;         /* LHS */
+        grad_avp_t *reply;         /* RHS */
+        grad_locus_t loc;                 /* Location of the definition of
+				      this entry */
+        int ordnum;                /* Entry ordinal number (used for semantic
+				      checks */
 } User_symbol;
 
 #define SNMP_RO 1
@@ -153,16 +155,10 @@ typedef struct user_symbol {
 
 #include <radsnmp.h>
 
-typedef struct netdef NETDEF;
-struct netdef {
-        UINT4 ipaddr;        /* IP address */
-        UINT4 netmask;
-};
-
 typedef struct netname NETNAME;
 struct netname {
 	char *name;
-	LIST /* of NETDEF */ *netlist;
+	grad_list_t /* of grad_netdef_t */ *netlist;
 };
 
 typedef struct community Community;
@@ -174,13 +170,13 @@ struct community {
 typedef struct access_control_list ACL;
 struct access_control_list {
         Community *community;   /* community or NULL to deny access */
-	LIST /* of NETDEF */ *netlist;
+	grad_list_t /* of grad_netdef_t */ *netlist;
 };
 
 struct radstat {
         struct timeval start_time;
-        counter port_active_count;
-        counter port_idle_count;
+        grad_counter_t port_active_count;
+        grad_counter_t port_idle_count;
 };
 
 typedef enum {
@@ -201,9 +197,9 @@ typedef struct {
 
 #define stat_inc(m,a,c) \
  do if (server_stat) {\
-        NAS *nas;\
+        grad_nas_t *nas;\
         server_stat -> m . c ++;\
-        if ((nas = nas_lookup_ip(a)) != NULL && nas->app_data)\
+        if ((nas = grad_nas_lookup_ip(a)) != NULL && nas->app_data)\
                 ((struct nas_stat*)nas->app_data)-> m . c ++;\
  } while (0)
 
@@ -234,11 +230,17 @@ typedef void (*config_hook_fp)(void *func_data, void *app_data);
 /*
  * Authentication results
  */
-#define AUTH_OK      0 /* OK */
-#define AUTH_FAIL    1 /* Password fail */
-#define AUTH_NOUSER  2 /* No such user  */
-#define AUTH_REJECT  3 /* Rejected */
-#define AUTH_IGNORE  4 /* Silently ignore */
+enum auth_status {
+	auth_ok,              /* Authentication passed */
+	auth_valid,           /* Authentication passed, expiration timeout
+				 is returned */
+	auth_account_expired, /* Account has expired */
+	auth_password_expired,/* Password has expired */ 
+	auth_fail,
+	auth_nouser,
+	auth_reject,
+	auth_ignore,
+};
 
 /* Logging modes */
 #define RLOG_AUTH               0x0001
@@ -263,6 +265,12 @@ typedef void (*config_hook_fp)(void *func_data, void *app_data);
 #define MSG_TIMESPAN_VIOLATION      7
 #define MSG_COUNT                   8
 
+typedef struct radiusd_user {
+	char *username;
+	uid_t uid;
+	gid_t gid;
+} RADIUS_USER;
+
 /*
  *      Global variables.
  */
@@ -270,23 +278,29 @@ extern int radius_mode;
 extern int debug_flag;
 extern int auth_detail;
 extern int acct_detail;
+extern char *auth_detail_template;
+extern char *acct_detail_template;
+extern int acct_system;
+extern int auth_trace_rules;
+extern int acct_trace_rules;
 extern int strip_names;
 extern int checkrad_assume_logged;
+extern int auth_reject_malformed_names;
 extern size_t max_requests;
 extern size_t max_children;
 extern unsigned process_timeout;
 extern unsigned radiusd_write_timeout;
 extern unsigned radiusd_read_timeout;
-extern char *exec_user;
-extern UINT4 expiration_seconds;
-extern UINT4 warning_seconds;
+extern grad_uint32_t expiration_seconds;
+extern grad_uint32_t warning_seconds;
 extern int use_dbm;
-extern UINT4 myip;
-extern UINT4 ref_ip;
+extern grad_uint32_t myip;
+extern grad_uint32_t ref_ip;
 extern int auth_port;
 extern int acct_port;
 extern int suspend_flag;
 extern int log_mode;
+extern char *auth_log_hook;
 extern int use_guile;
 extern char *message_text[MSG_COUNT];
 extern char *username_valid_chars;
@@ -306,6 +320,9 @@ extern struct cfg_stmt snmp_stmt[];
 #endif
 extern int auth_comp_flag; 
 extern int acct_comp_flag; 
+extern RADIUS_USER exec_user;
+extern RADIUS_USER radiusd_user;
+extern grad_symtab_t *user_tab;
 
 /* Input subsystem (input.c) */
 
@@ -357,7 +374,7 @@ int udp_input_handler(int fd, void *data);
 int udp_input_close(int fd, void *data);
 int udp_input_cmp(const void *a, const void *b);
 
-int udp_open(int type, UINT4 ipaddr, int port, int nonblock);
+int udp_open(int type, grad_uint32_t ipaddr, int port, int nonblock);
 
 void radiusd_pidfile_write(char *name);
 pid_t radiusd_pidfile_read(char *name);
@@ -378,22 +395,23 @@ void radiusd_register_input_fd(char *name, int fd, void *data);
 void radiusd_close_channel(int fd);
 
 /* exec.c */
-int radius_exec_program(char *, RADIUS_REQ *, VALUE_PAIR **, int);
+int radius_get_user_ids(RADIUS_USER *usr, const char *name);
+int radius_switch_to_user(RADIUS_USER *usr);
+int radius_exec_command(char *cmd);
+int radius_exec_program(char *, grad_request_t *, grad_avp_t **, int);
 void filter_cleanup(pid_t pid, int status);
-int filter_auth(char *name, RADIUS_REQ *req, VALUE_PAIR **reply_pairs);
-int filter_acct(char *name, RADIUS_REQ *req);
+int filter_auth(char *name, grad_request_t *req, grad_avp_t **reply_pairs);
+int filter_acct(char *name, grad_request_t *req);
 int filters_stmt_term(int finish, void *block_data, void *handler_data);
 extern struct cfg_stmt filters_stmt[];
 
 /* scheme.c */
 void scheme_main();
-void scheme_load(char *filename);
 void scheme_load_path(char *pathname);
-void scheme_debug(int val);
-int scheme_auth(char *procname, RADIUS_REQ *req,
-                VALUE_PAIR *user_check, VALUE_PAIR **user_reply_ptr);
-int scheme_acct(char *procname, RADIUS_REQ *req);
-void scheme_add_load_path(char *path);
+int scheme_auth(char *procname, grad_request_t *req,
+                grad_avp_t *user_check, grad_avp_t **user_reply_ptr);
+int scheme_acct(char *procname, grad_request_t *req);
+int scheme_eval_boolean_expr(char *expr);
 void scheme_read_eval_loop();
 void scheme_redirect_output();
 void start_guile();
@@ -402,12 +420,19 @@ int guile_cfg_handler(int argc, cfg_value_t *argv,
 extern struct cfg_stmt guile_stmt[];
 
 /* log.c */
-void sqllog __PVAR((int status, char *msg, ...));
+void radiusd_logger(int level,
+		    const grad_request_t *req,
+		    const grad_locus_t *loc,
+		    const char *func_name,
+		    int en,
+		    const char *fmt, va_list ap);
+void sqllog(int status, char *msg, ...);
 int logging_stmt_handler(int argc, cfg_value_t *argv, void *block_data,
 			 void *handler_data);
 int logging_stmt_end(void *block_data, void *handler_data);
 int logging_stmt_begin(int finish, void *block_data, void *handler_data);
 void format_exit_status(char *buffer, int buflen, int status);
+int log_change_owner(RADIUS_USER *usr);
 extern struct cfg_stmt logging_stmt[];
 
 /* radius.c */
@@ -416,12 +441,14 @@ extern struct cfg_stmt logging_stmt[];
 #define REQ_AUTH_ZERO 1
 #define REQ_AUTH_BAD  2
 
-void radius_send_reply(int, RADIUS_REQ *, VALUE_PAIR *, char *, int);
-void radius_send_challenge(RADIUS_REQ *radreq, char *msg, char *state, int fd);
+void radius_send_reply(int, grad_request_t *, grad_avp_t *, char *, int);
+void radius_send_challenge(grad_request_t *radreq, char *msg, char *state, int fd);
 int radius_verify_digest(REQUEST *req);
 
-int radius_req_decode(struct sockaddr_in *sa,
-		      void *input, size_t inputsize, void **output);
+int radius_auth_req_decode(struct sockaddr_in *sa,
+			   void *input, size_t inputsize, void **output);
+int radius_acct_req_decode(struct sockaddr_in *sa,
+			   void *input, size_t inputsize, void **output);
 int radius_req_cmp(void *a, void *b);
 void radius_req_free(void *req);
 void radius_req_drop(int type, void *radreq, void *origreq,
@@ -430,6 +457,10 @@ void radius_req_xmit(REQUEST *request);
 int radius_req_failure(int type, struct sockaddr_in *addr);
 void radius_req_update(void *req_ptr, void *data_ptr);
 int radius_respond(REQUEST *req);
+void radius_req_register_locus(grad_request_t *req, grad_locus_t *loc);
+void radius_trace_path(grad_request_t *req);
+grad_avp_t *radius_decrypt_request_pairs(grad_request_t *req, grad_avp_t *pair);
+void radius_destroy_pairs(grad_avp_t **p);
 
 /* shmem.c */
 int shmem_alloc(size_t size);
@@ -442,9 +473,11 @@ int pam_pass(char *name, char *passwd, const char *pamauth, char **reply_msg);
 
 /* proxy.c */
 int proxy_send(REQUEST *req);
-int proxy_receive(RADIUS_REQ *radreq, RADIUS_REQ *oldreq, int activefd);
-void proxy_retry(RADIUS_REQ *radreq, int fd);
-int proxy_cmp(RADIUS_REQ *qr, RADIUS_REQ *r);
+int proxy_receive(grad_request_t *radreq, grad_request_t *oldreq, int activefd);
+void proxy_retry(grad_request_t *radreq, int fd);
+int proxy_cmp(grad_request_t *qr, grad_request_t *r);
+grad_avp_t *proxy_request_recode(grad_request_t *radreq, grad_avp_t *plist,
+				 u_char *secret, u_char *vector);
 
 /* menu.c */
 #define MAX_PATH_LENGTH                 256
@@ -454,47 +487,74 @@ int proxy_cmp(RADIUS_REQ *qr, RADIUS_REQ *r);
 #define MAX_STATE_VALUE                 128
 #define RAD_BUFFER_SIZE                 4096
 
-void menu_reply(RADIUS_REQ *radreq, int fd);
+void menu_reply(grad_request_t *radreq, int fd);
 char *menu_read_text(char *menu_name);
 
 /* acct.c */
-int rad_accounting(RADIUS_REQ *, int, int);
-int radzap(UINT4 nas, int port, char *user, time_t t);
-int rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul, int *pcount);
-int rad_check_realm(REALM *realm);
-int write_detail(RADIUS_REQ *radreq, int authtype, char *f);
+void system_acct_init();
+void acct_init();
+int rad_accounting(grad_request_t *, int, int);
+int radzap(grad_uint32_t nas, int port, char *user, time_t t);
+int write_detail(grad_request_t *radreq, int authtype, int rtype);
+
+int radutmp_mlc_collect_user(char *name, grad_request_t *request,
+			     grad_list_t **sess_list);
+int radutmp_mlc_collect_realm(grad_request_t *request,
+			      grad_list_t **sess_list);
+void radutmp_mlc_close(struct radutmp *up);
+int radutmp_mlc_enabled_p(void);
+
+/* mlc.c */
+typedef int (*mlc_collect_user_t) (char *name, grad_request_t *request,
+				   grad_list_t **sess_list);
+typedef int (*mlc_collect_realm_t) (grad_request_t *request,
+				    grad_list_t **sess_list);
+typedef void (*mlc_close_t) (struct radutmp *up);
+typedef int (*mlc_enabled_t) (void);
+void mlc_register_method(char *name,
+			 mlc_collect_user_t collect_user,
+			 mlc_collect_realm_t collect_realm,
+			 mlc_close_t close,
+			 mlc_enabled_t enabled_p);
+
+extern struct cfg_stmt mlc_stmt[];
+int radius_mlc_user(char *name, grad_request_t *request,
+		    size_t maxsimul, size_t *pcount);
+int radius_mlc_realm(grad_request_t *request);
 
 /* files.c */
-int user_find(char *name, RADIUS_REQ *, VALUE_PAIR **, VALUE_PAIR **);
-int userparse(char *buf, VALUE_PAIR **first_pair, char **errmsg);
-void presuf_setup(VALUE_PAIR *request_pairs);
-int hints_setup(RADIUS_REQ *request);
-int huntgroup_access(RADIUS_REQ *radreq);
-CLIENT *client_lookup_ip(UINT4 ipno);
-char *client_lookup_name(UINT4 ipno, char *buf, size_t size);
+int user_find(char *name, grad_request_t *, grad_avp_t **, grad_avp_t **);
+int userparse(char *buf, grad_avp_t **first_pair, char **errmsg);
+void presuf_setup(grad_avp_t *request_pairs);
+int hints_setup(grad_request_t *request);
+int huntgroup_access(grad_request_t *radreq, grad_locus_t *loc);
+CLIENT *client_lookup_ip(grad_uint32_t ipno);
+char *client_lookup_name(grad_uint32_t ipno, char *buf, size_t size);
 int read_clients_file(char *);
-NAS *nas_find(UINT4 ipno);
-NAS *nas_by_name(char *name);
-char *nas_name(UINT4 ipno);
-char *nas_name2(RADIUS_REQ *r);
-int read_naslist_file(char *);
+grad_nas_t *grad_nas_find(grad_uint32_t ipno);
+grad_nas_t *grad_nas_by_name(char *name);
+char *grad_nas_name(grad_uint32_t ipno);
+char *nas_name2(grad_request_t *r);
+int read_nas_list_file(char *);
 int reload_config_file(enum reload_what);
-int presufcmp(VALUE_PAIR *check, char *name, char *rest);
+int presufcmp(grad_avp_t *check, char *name, char *rest);
 int get_config();
 int get_deny(char *user);
-NAS *findnasbyindex(int);
+grad_nas_t *findnasbyindex(int);
 char *make_server_ident();
 void dump_users_db();
 void strip_username(int do_strip, char *name,
-                    VALUE_PAIR *check_item, char *stripped_name);
+                    grad_avp_t *check_item, char *stripped_name);
 
 /* version.c */
 void version();
 
 /* auth.c */
-int rad_auth_init(RADIUS_REQ *radreq, int activefd);
-int rad_authenticate (RADIUS_REQ *, int);
-void req_decrypt_password(char *password, RADIUS_REQ *req, VALUE_PAIR *pair);
+int rad_auth_init(grad_request_t *radreq, int activefd);
+int rad_auth_check_username(grad_request_t *radreq, int activefd);
+int rad_authenticate (grad_request_t *, int);
+void req_decrypt_password(char *password, grad_request_t *req,
+			  grad_avp_t *pair);
 
 /* timestr.c */
 int timestr_match(char *, time_t);
@@ -504,7 +564,7 @@ int timestr_match(char *, time_t);
 void snmpserv_init(void *arg);
 void snmp_auth_server_reset();
 void snmp_acct_server_reset();
-void snmp_attach_nas_stat(NAS *nas);
+void snmp_attach_nas_stat(grad_nas_t *nas);
 void snmp_init_nas_stat();
 void snmp_sort_nas_stat();
 int snmp_stmt_begin(int finish, void *data, void *up_data);
@@ -517,9 +577,9 @@ void stat_init();
 void stat_done();
 void stat_update(struct radutmp *ut, int status);
 void stat_count_ports();
-struct nas_stat * find_nas_stat(UINT4 ip_addr);
-int stat_get_port_index(NAS *nas, int port_no);
-int stat_get_next_port_no(NAS *nas, int port_no);
+struct nas_stat * find_nas_stat(grad_uint32_t ip_addr);
+int stat_get_port_index(grad_nas_t *nas, int port_no);
+int stat_get_next_port_no(grad_nas_t *nas, int port_no);
 #else
 # define stat_init()
 # define stat_done()
@@ -537,23 +597,75 @@ int snmp_req_respond(REQUEST *request);
         
 /* radutil.c */
 char *radius_xlate(struct obstack *obp, char *str,
-                   RADIUS_REQ *req, VALUE_PAIR *reply_pairs);
+                   grad_request_t *req, grad_avp_t *reply_pairs);
+int radius_eval_avl(grad_request_t *req, grad_avp_t *p);
+char *util_xlate(struct obstack *sp, char *fmt, grad_request_t *radreq);
 
 /* rewrite.y */
 extern struct cfg_stmt rewrite_stmt[];
-int run_rewrite(char *name, RADIUS_REQ *req);
 int parse_rewrite(char *name);
-int va_run_init __PVAR((char *name, RADIUS_REQ *request, char *typestr, ...));
 
 /* radck.c */
-int fix_check_pairs(int sf_file, char *filename, int line, char *name,
-                    VALUE_PAIR **pairs);
-int fix_reply_pairs(int cf_file, char *filename, int line, char *name,
-                    VALUE_PAIR **pairs);
+int fix_check_pairs(int sf_file, grad_locus_t *loc, char *name, grad_avp_t **pairs);
+int fix_reply_pairs(int cf_file, grad_locus_t *loc, char *name, grad_avp_t **pairs);
 void radck();
 
 /* checkrad.c */
-int checkrad(NAS *nas, struct radutmp *up);
+int checkrad(grad_nas_t *nas, struct radutmp *up);
 
+/* forward.c */
+int rad_cfg_forward_auth(int argc, cfg_value_t *argv,
+			 void *block_data, void *handler_data);
+int rad_cfg_forward_acct(int argc, cfg_value_t *argv,
+			 void *block_data, void *handler_data);
+void forward_init();
+void forward_request(int type, grad_request_t *req);
 
+/* Logging */
+
+/* log output modes */
+#define LM_UNKNOWN -1
+#define LM_OFF 0
+#define LM_FILE 1
+#define LM_SYSLOG 2
+
+/* log options */
+#define LO_CONS  0x0001
+#define LO_PID   0x0002
+#define LO_CAT   0x0004
+#define LO_PRI   0x0008
+#define LO_MSEC  0x0010
+#define LO_PERSIST 0x8000
+
+#define RADIUS_DEBUG_BUFFER_SIZE 1024
+
+typedef struct channel Channel;
+
+struct channel {
+        char *name;
+        int  pmask[L_NCAT]; 
+        int mode;   /* LM_ constant */
+        union {
+                int prio;        /* syslog: facility|priority */
+                char *file;      /* file: output file name */
+        } id;
+        int options;
+	char *prefix_hook;       /* prefix hook function */
+	char *suffix_hook;       /* suffix hook function */
+};
+
+Channel *channel_lookup(char *name);
+void channel_free(Channel *chan);
+void channel_free_list(Channel *chan);
+Channel * log_mark();
+void log_release();
+
+void register_channel(Channel *chan);
+void register_category(int cat, int pri, grad_list_t *chanlist);
+
+void log_set_to_console();
+void log_set_default(char *name, int cat, int pri);
+
+void log_open(int cat);
+void log_close();
 
