@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -17,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_RADPDU_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -34,9 +32,8 @@
 #include <pwd.h>
 #include <time.h>
 #include <ctype.h>
-#include <radius.h>
-#include <obstack1.h>
 
+#include <common.h>
 
 /* Structure for building radius PDU. */
 struct radius_pdu {
@@ -51,14 +48,14 @@ struct radius_attr {
         u_char data[AUTH_STRING_LEN];    
 };
 
-void rad_pdu_destroy(struct radius_pdu *pdu);
-int rad_attr_write(struct radius_attr *ap, void *data, size_t size);
-int rad_encode_pair(struct radius_attr *ap, VALUE_PAIR *pair);
+void grad_pdu_destroy(struct radius_pdu *pdu);
+int grad_attr_write(struct radius_attr *ap, void *data, size_t size);
+int grad_encode_pair(struct radius_attr *ap, grad_avp_t *pair);
 
 
 /* Initialize a PDU */
 static void
-rad_pdu_init(struct radius_pdu *pdu)
+grad_pdu_init(struct radius_pdu *pdu)
 {
         pdu->size = 0;
         obstack_init(&pdu->st);
@@ -73,13 +70,13 @@ rad_pdu_init(struct radius_pdu *pdu)
           *ptr   -- Radius reply.
    Return value: length of the data on *ptr. */   
 static size_t
-rad_pdu_finish(void **ptr, struct radius_pdu *pdu,
-	       int code, int id, u_char *vector, u_char *secret)
+grad_pdu_finish(void **ptr, struct radius_pdu *pdu,
+		int code, int id, u_char *vector, u_char *secret)
 {
-        AUTH_HDR *hdr;
+        grad_packet_header_t *hdr;
         void *p;
         size_t secretlen = 0;
-        size_t len = sizeof(AUTH_HDR) + pdu->size;
+        size_t len = sizeof(grad_packet_header_t) + pdu->size;
         u_char digest[AUTH_DIGEST_LEN];
         
 	if (code != RT_ACCESS_REQUEST && code != RT_STATUS_SERVER) {
@@ -88,7 +85,7 @@ rad_pdu_finish(void **ptr, struct radius_pdu *pdu,
         }
         /* Create output array */
         p = obstack_finish(&pdu->st);
-        hdr = emalloc(len + secretlen);
+        hdr = grad_emalloc(len + secretlen);
         hdr->code = code;
         hdr->id = id;
         hdr->length = htons(len);
@@ -108,7 +105,7 @@ rad_pdu_finish(void **ptr, struct radius_pdu *pdu,
 		   the md5 hash over the entire packet and put it in the
 		   vector. */
                 secretlen = strlen(secret);
-                md5_calc(hdr->vector, (u_char *)hdr, len + secretlen);
+                grad_md5_calc(hdr->vector, (u_char *)hdr, len + secretlen);
 		memcpy(vector, hdr->vector, AUTH_VECTOR_LEN);
                 memset((char*)hdr + len, 0, secretlen);
 		break;
@@ -123,7 +120,7 @@ rad_pdu_finish(void **ptr, struct radius_pdu *pdu,
 	default:
 		/* This is a reply message. Calculate the response digest
 		   and store it in the pdu */
-		md5_calc(digest, (u_char *)hdr, len + secretlen);
+		grad_md5_calc(digest, (u_char *)hdr, len + secretlen);
 		memcpy(hdr->vector, digest, AUTH_VECTOR_LEN);
 		memset((char*)hdr + len, 0, secretlen);
 		break;
@@ -135,22 +132,22 @@ rad_pdu_finish(void **ptr, struct radius_pdu *pdu,
 
 /* Destroy the PDU */
 void 
-rad_pdu_destroy(struct radius_pdu *pdu)
+grad_pdu_destroy(struct radius_pdu *pdu)
 {
         obstack_free(&pdu->st, NULL);
 }
 
 /* Append attribute A to the PDU P */
-#define rad_pdu_add(p,a) \
+#define grad_pdu_add(p,a) \
  do { obstack_grow(&(p)->st,&(a),(a).length); \
       (p)->size+=(a).length; } while (0)
 
 /* Initialize the attribute structure. */       
-#define rad_attr_init(a) (a)->length = 2
+#define grad_attr_init(a) (a)->length = 2
 
 /* Append SIZE bytes from DATA to radius_attr AP. */    
 int
-rad_attr_write(struct radius_attr *ap, void *data, size_t size)
+grad_attr_write(struct radius_attr *ap, void *data, size_t size)
 {
         if (sizeof(ap->data) - ap->length + 2 < size)
                 return 0;
@@ -165,9 +162,9 @@ rad_attr_write(struct radius_attr *ap, void *data, size_t size)
    Return value: length of the encoded data or 0 if an error occurred */
    
 int
-rad_encode_pair(struct radius_attr *ap, VALUE_PAIR *pair)
+grad_encode_pair(struct radius_attr *ap, grad_avp_t *pair)
 {
-        UINT4 lval;
+        grad_uint32_t lval;
         size_t len;
         int rc;
 
@@ -180,17 +177,17 @@ rad_encode_pair(struct radius_attr *ap, VALUE_PAIR *pair)
                 len = pair->avp_strlength;
                 if (len > AUTH_STRING_LEN) 
                         len = AUTH_STRING_LEN;
-                rc = rad_attr_write(ap, pair->avp_strvalue, len);
+                rc = grad_attr_write(ap, pair->avp_strvalue, len);
                 break;
                 
         case TYPE_INTEGER:
         case TYPE_IPADDR:
                 lval = htonl(pair->avp_lvalue);
-                rc = rad_attr_write(ap, &lval, sizeof(UINT4));
+                rc = grad_attr_write(ap, &lval, sizeof(grad_uint32_t));
                 break;
 
         default:
-                radlog(L_ERR, "Unknown pair type %d", pair->type);
+                grad_log(L_ERR, "Unknown pair type %d", pair->type);
                 rc = 0;
         }
         return rc;
@@ -204,46 +201,46 @@ rad_encode_pair(struct radius_attr *ap, VALUE_PAIR *pair)
    Return value: lenght of the data in *rptr. 0 on error */
    
 size_t
-rad_create_pdu(void **rptr, int code, int id, u_char *vector,
-	       u_char *secret, VALUE_PAIR *pairlist, char *msg)
+grad_create_pdu(void **rptr, int code, int id, u_char *vector,
+		u_char *secret, grad_avp_t *pairlist, char *msg)
 {
         struct radius_pdu pdu;
         size_t attrlen = 0;
         int status = 0;
         int len;
-        VALUE_PAIR *pair;
+        grad_avp_t *pair;
         
-        rad_pdu_init(&pdu);
+        grad_pdu_init(&pdu);
 
         for (pair = pairlist; pair; pair = pair->next) {
                 struct radius_attr attr;
-                UINT4 lval;
+                grad_uint32_t lval;
                 int vendorcode, vendorpec;
                 
                 if (debug_on(10)) {
                         char *save;
-                        radlog(L_DEBUG,
-                               "send: %s", format_pair(pair, 1, &save));
+                        grad_log(L_DEBUG,
+                                 "send: %s", grad_format_pair(pair, 1, &save));
                         free(save);
                 }
 
-                rad_attr_init(&attr);
-                if ((vendorcode = VENDOR(pair->attribute)) > 0
-                    && (vendorpec  = vendor_id_to_pec(vendorcode)) > 0) {
+                grad_attr_init(&attr);
+                if ((vendorcode = GRAD_VENDOR_CODE(pair->attribute)) > 0
+                    && (vendorpec  = grad_vendor_id_to_pec(vendorcode)) > 0) {
                         attr.attrno = DA_VENDOR_SPECIFIC;
                         lval = htonl(vendorpec);
-                        rad_attr_write(&attr, &lval, 4);
+                        grad_attr_write(&attr, &lval, 4);
 			if (vendorpec == 429) {
 				/* Hack for non-compliant USR VSA */
-				UINT4 atval = htonl(pair->attribute & 0xffff);
-				rad_attr_write(&attr, &atval, 4);
-				attrlen = rad_encode_pair(&attr, pair);
+				grad_uint32_t atval = htonl(pair->attribute & 0xffff);
+				grad_attr_write(&attr, &atval, 4);
+				attrlen = grad_encode_pair(&attr, pair);
 			} else {
 				u_char c = pair->attribute & 0xff;
-				rad_attr_write(&attr, &c, 1);
+				grad_attr_write(&attr, &c, 1);
 				/* Reserve a length byte */
-				rad_attr_write(&attr, &lval, 1); 
-				attrlen = rad_encode_pair(&attr, pair);
+				grad_attr_write(&attr, &lval, 1); 
+				attrlen = grad_encode_pair(&attr, pair);
 				/* Fill in the length */
 				attr.data[5] = 2+attrlen; 
 			}
@@ -251,14 +248,14 @@ rad_create_pdu(void **rptr, int code, int id, u_char *vector,
                         continue;
 		else {
 			attr.attrno = pair->attribute;
-			attrlen = rad_encode_pair(&attr, pair);
+			attrlen = grad_encode_pair(&attr, pair);
 		}
                 if (attrlen < 0) {
-                        radlog(L_ERR, "attrlen = %d", attrlen);
+                        grad_log(L_ERR, "attrlen = %d", attrlen);
                         status = 1;
                         break;
                 }
-                rad_pdu_add(&pdu, attr);
+                grad_pdu_add(&pdu, attr);
         }
 
         /* Append the user message
@@ -275,37 +272,37 @@ rad_create_pdu(void **rptr, int code, int id, u_char *vector,
                         else 
                                 block_len = len;
 
-                        rad_attr_init(&attr);
+                        grad_attr_init(&attr);
                         attr.attrno = DA_REPLY_MESSAGE;
-                        attrlen = rad_attr_write(&attr, msg, block_len);
+                        attrlen = grad_attr_write(&attr, msg, block_len);
                         if (attrlen <= 0) {
                                 status = 1;
                                 break;
                         }
 			debug(10,("send: Reply-Message = %*.*s",
 				  block_len, block_len, attr.data));
-                        rad_pdu_add(&pdu, attr);
+                        grad_pdu_add(&pdu, attr);
                         msg += block_len;
                         len -= block_len;
                 }
         }
 
         if (status == 0) 
-		attrlen = rad_pdu_finish(rptr, &pdu, code, id, vector, secret);
+		attrlen = grad_pdu_finish(rptr, &pdu, code, id, vector, secret);
 	else
                 attrlen = 0;
-        rad_pdu_destroy(&pdu);
+        grad_pdu_destroy(&pdu);
         return attrlen;
 }
 
-static VALUE_PAIR *
-rad_decode_pair(UINT4 attrno, char *ptr, size_t attrlen)
+static grad_avp_t *
+grad_decode_pair(grad_uint32_t attrno, char *ptr, size_t attrlen)
 {
-        DICT_ATTR *attr;
-        VALUE_PAIR *pair;
-        UINT4 lval;
+        grad_dict_attr_t *attr;
+        grad_avp_t *pair;
+        grad_uint32_t lval;
         
-        if ((attr = attr_number_to_dict(attrno)) == NULL) {
+        if ((attr = grad_attr_number_to_dict(attrno)) == NULL) {
                 debug(1, ("Received unknown attribute %d", attrno));
                 return NULL;
         }
@@ -316,7 +313,7 @@ rad_decode_pair(UINT4 attrno, char *ptr, size_t attrlen)
                 return NULL;
         }
 
-        pair = avp_alloc();
+        pair = grad_avp_alloc();
         
         pair->name = attr->name;
         pair->attribute = attr->value;
@@ -329,14 +326,14 @@ rad_decode_pair(UINT4 attrno, char *ptr, size_t attrlen)
         case TYPE_STRING:
                 /* attrlen always <= AUTH_STRING_LEN */
                 pair->avp_strlength = attrlen;
-                pair->avp_strvalue = emalloc(attrlen + 1);
+                pair->avp_strvalue = grad_emalloc(attrlen + 1);
                 memcpy(pair->avp_strvalue, ptr, attrlen);
                 pair->avp_strvalue[attrlen] = 0;
 
                 if (debug_on(10)) {
                         char *save;
-                        radlog(L_DEBUG, "recv: %s",
-                               format_pair(pair, 1, &save));
+                        grad_log(L_DEBUG, "recv: %s",
+                                 grad_format_pair(pair, 1, &save));
                         free(save);
                 }
 
@@ -344,14 +341,14 @@ rad_decode_pair(UINT4 attrno, char *ptr, size_t attrlen)
                         
         case TYPE_INTEGER:
         case TYPE_IPADDR:
-                memcpy(&lval, ptr, sizeof(UINT4));
+                memcpy(&lval, ptr, sizeof(grad_uint32_t));
                 pair->avp_lvalue = ntohl(lval);
 
                 if (debug_on(10)) {
                         char *save;
-                        radlog(L_DEBUG, 
-                               "recv: %s", 
-                               format_pair(pair, 1, &save));
+                        grad_log(L_DEBUG, 
+                                 "recv: %s", 
+                                 grad_format_pair(pair, 1, &save));
                         free(save);
                 }
                 break;
@@ -359,7 +356,7 @@ rad_decode_pair(UINT4 attrno, char *ptr, size_t attrlen)
         default:
                 debug(1, ("    %s (Unknown Type %d)",
                           attr->name,attr->type));
-                avp_free(pair);
+                grad_avp_free(pair);
                 pair = NULL;
                 break;
         }
@@ -368,18 +365,18 @@ rad_decode_pair(UINT4 attrno, char *ptr, size_t attrlen)
 }
 
 static int
-decode_vsa(u_char *ptr, UINT4 attrlen, UINT4 *vendorpec, UINT4 *vendorcode)
+decode_vsa(u_char *ptr, grad_uint32_t attrlen, grad_uint32_t *vendorpec, grad_uint32_t *vendorcode)
 {
-	UINT4 x;
+	grad_uint32_t x;
 	
 	if (attrlen <= 6) { /*FIXME*/
-		radlog(L_NOTICE,
-		       _("Received a vendor-specific attribute with length <= 6"));
+		grad_log(L_NOTICE,
+		    _("Received a vendor-specific attribute with length <= 6"));
 		return 1;
 	}
 	memcpy(&x, ptr, 4);
 	*vendorpec = ntohl(x);
-	*vendorcode = vendor_pec_to_id(*vendorpec);
+	*vendorcode = grad_vendor_pec_to_id(*vendorpec);
 
 	return *vendorcode == 0;
 }
@@ -388,28 +385,28 @@ decode_vsa(u_char *ptr, UINT4 attrlen, UINT4 *vendorpec, UINT4 *vendorcode)
    structure, and attach attribute-value pairs contained in the request
    to the new structure. */
 
-RADIUS_REQ *
-rad_decode_pdu(UINT4 host, u_short udp_port, u_char *buffer, size_t length)
+grad_request_t *
+grad_decode_pdu(grad_uint32_t host, u_short udp_port, u_char *buffer, size_t length)
 {
         u_char          *ptr;
-        AUTH_HDR        *auth;
-        VALUE_PAIR      *first_pair;
-        VALUE_PAIR      *prev;
-        VALUE_PAIR      *pair;
-        RADIUS_REQ      *radreq;
-        UINT4 reported_len;
+        grad_packet_header_t        *auth;
+        grad_avp_t      *first_pair;
+        grad_avp_t      *prev;
+        grad_avp_t      *pair;
+        grad_request_t  *radreq;
+        grad_uint32_t reported_len;
         u_char *endp;
         int stop;
         
-        radreq = radreq_alloc();
+        radreq = grad_request_alloc();
         debug(1,("allocated radreq: %p",radreq));
 	
-        auth = (AUTH_HDR *)buffer;
+        auth = (grad_packet_header_t *)buffer;
         reported_len = ntohs(auth->length);
         if (length > reported_len) { /* FIXME: != ? */
-                radlog(L_WARN,
-                       _("Actual request length does not match reported length (%d, %d)"),
-                       length, reported_len);
+                grad_log(L_WARN,
+             _("Actual request length does not match reported length (%d, %d)"),
+                         length, reported_len);
                 length = reported_len;
         }
                 
@@ -434,7 +431,7 @@ rad_decode_pdu(UINT4 host, u_short udp_port, u_char *buffer, size_t length)
         stop = 0;
         
         while (ptr < endp && !stop) {
-                UINT4 attrno, attrlen, vendorcode, vendorpec;
+                grad_uint32_t attrno, attrlen, vendorcode, vendorpec;
                                 
                 attrno = *ptr++;
                 attrlen = *ptr++;
@@ -457,19 +454,21 @@ rad_decode_pdu(UINT4 host, u_short udp_port, u_char *buffer, size_t length)
 				if (vendorpec == 429) {
 					/* Hack for non-compliant USR VSA */
 					memcpy(&attrno, ptr, 4);
-					attrno = ntohl(attrno)
-						 | (vendorcode << 16);
+					attrno = GRAD_VSA_ATTR_NUMBER(
+						          ntohl(attrno),
+							  vendorcode);
 					ptr += 4;
 					attrlen -= 4;
 					len = attrlen;
 					attrlen = 0;
 				} else {
-					attrno = *ptr++ | (vendorcode << 16);
+					attrno = GRAD_VSA_ATTR_NUMBER(*ptr++,
+								   vendorcode);
 					len = *ptr++ - 2;
 					attrlen -= len + 2;
 				}
 				
-                                pair = rad_decode_pair(attrno, ptr, len);
+                                pair = grad_decode_pair(attrno, ptr, len);
                                 if (pair) {
                                 	if (first_pair == NULL) 
                                         	first_pair = pair;
@@ -480,7 +479,7 @@ rad_decode_pdu(UINT4 host, u_short udp_port, u_char *buffer, size_t length)
                                 ptr += len;
                         }
                 } else {
-                        pair = rad_decode_pair(attrno, ptr, attrlen);
+                        pair = grad_decode_pair(attrno, ptr, attrlen);
                         ptr += attrlen;
                         if (pair) {
                         	if (first_pair == NULL) 
@@ -492,17 +491,15 @@ rad_decode_pdu(UINT4 host, u_short udp_port, u_char *buffer, size_t length)
                 }
         }
 
-	/* Add NAS-IP-Address if the NAS didn't send one */
-	if (!avl_find(first_pair, DA_NAS_IP_ADDRESS)) 
-		avl_add_pair(&first_pair,
-			     avp_create_integer(DA_NAS_IP_ADDRESS, host));
-	
         radreq->request = first_pair;
 #ifdef DEBUG_ONLY
         {
-                VALUE_PAIR *p = avl_find(radreq->request, DA_NAS_IP_ADDRESS);
+                grad_avp_t *p = grad_avl_find(radreq->request,
+					      DA_NAS_IP_ADDRESS);
                 if (p)
                         radreq->ipaddr = p->avp_lvalue;
+		else
+			radreq->ipaddr = host;
         }
 #endif
         return radreq;
