@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -17,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_RADSRV_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -34,22 +32,22 @@
 #include <pwd.h>
 #include <time.h>
 #include <ctype.h>
-#include <radius.h>
-#include <debugmod.h>
+
+#include <common.h>
 
 /* Build and send a reply to the incoming request.
    Input: fd          -- Socket descriptor.
           radreq      -- The request. */
 
 int
-rad_srv_send_reply(int fd, RADIUS_REQ *radreq)
+grad_server_send_reply(int fd, grad_request_t *radreq)
 {
         void *pdu;
         size_t length;
 
-        length = rad_create_pdu(&pdu, radreq->reply_code,
-                                radreq->id, radreq->vector, radreq->secret,
-                                radreq->reply_pairs, radreq->reply_msg);
+        length = grad_create_pdu(&pdu, radreq->reply_code,
+				 radreq->id, radreq->vector, radreq->secret,
+				 radreq->reply_pairs, radreq->reply_msg);
         if (length > 0) {
                 struct sockaddr saremote;
                 struct sockaddr_in *sin;
@@ -58,7 +56,7 @@ rad_srv_send_reply(int fd, RADIUS_REQ *radreq)
                 debug(1, ("Sending %s of id %d to %lx (nas %s)",
                           auth_code_str(radreq->reply_code), radreq->id,
 			  (u_long)radreq->ipaddr,
-                          nas_request_to_name(radreq, buf, sizeof buf)));
+                          grad_nas_request_to_name(radreq, buf, sizeof buf)));
                 
                 sin = (struct sockaddr_in *) &saremote;
                 memset ((char *) sin, '\0', sizeof (saremote));
@@ -69,7 +67,7 @@ rad_srv_send_reply(int fd, RADIUS_REQ *radreq)
                 sendto(fd, pdu, length, 0,
                        &saremote, sizeof(struct sockaddr_in));
 #endif
-                efree(pdu);
+                grad_free(pdu);
         }
 	return length;
 }
@@ -83,19 +81,19 @@ rad_srv_send_reply(int fd, RADIUS_REQ *radreq)
           state       -- Value of the State attribute.
 */
 int
-rad_srv_send_challenge(int fd, RADIUS_REQ *radreq, char *msg, char *state)
+grad_server_send_challenge(int fd, grad_request_t *radreq, char *msg, char *state)
 {
         void *pdu;
         size_t length;
-        VALUE_PAIR *p = avp_create_string(DA_STATE, state);
-	VALUE_PAIR *reply;
+        grad_avp_t *p = grad_avp_create_string(DA_STATE, state);
+	grad_avp_t *reply;
 
-	reply = avl_dup(radreq->reply_pairs);
-	avl_merge(&reply, &p);
-        length = rad_create_pdu(&pdu, RT_ACCESS_CHALLENGE, radreq->id,
-                                radreq->vector, radreq->secret, reply, msg);
-	avl_free(reply);
-	avl_free(p);
+	reply = grad_avl_dup(radreq->reply_pairs);
+	grad_avl_merge(&reply, &p);
+        length = grad_create_pdu(&pdu, RT_ACCESS_CHALLENGE, radreq->id,
+				 radreq->vector, radreq->secret, reply, msg);
+	grad_avl_free(reply);
+	grad_avl_free(p);
 	
         if (length > 0) {
                 struct sockaddr saremote;
@@ -104,7 +102,7 @@ rad_srv_send_challenge(int fd, RADIUS_REQ *radreq, char *msg, char *state)
 
                 debug(1, ("Sending Challenge of id %d to %lx (nas %s)",
                           radreq->id, (u_long)radreq->ipaddr,
-                          nas_request_to_name(radreq, buf, sizeof buf)));
+                          grad_nas_request_to_name(radreq, buf, sizeof buf)));
         
                 sin = (struct sockaddr_in *) &saremote;
                 memset ((char *) sin, '\0', sizeof (saremote));
@@ -116,9 +114,9 @@ rad_srv_send_challenge(int fd, RADIUS_REQ *radreq, char *msg, char *state)
                 sendto(fd, pdu, length, 0,
                        &saremote, sizeof(struct sockaddr_in));
 #endif
-                efree(pdu);
+                grad_free(pdu);
         }
-        avp_free(p);
+        grad_avp_free(p);
 	return length;
 }
 #endif
