@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -25,8 +25,7 @@
 #include <fcntl.h>
 #include <netinet/in.h>
 
-#include <radius.h>
-#include <radutmp.h>
+#include <common.h>
 
 struct _radut_file {
         int fd;
@@ -47,13 +46,13 @@ rut_setent(char *name, int append)
                 fd = open(name, O_RDONLY);
         }
         if (fd == -1) {
-                radlog(L_ERR|L_PERROR, 
-                       _("rut_setent(): cannot open `%s'"), name);
+                grad_log(L_ERR|L_PERROR, 
+                         _("rut_setent(): cannot open `%s'"), name);
                 return NULL;
         }
         if (append)
                 lseek(fd, 0, SEEK_END);
-        fp = emalloc(sizeof(*fp));
+        fp = grad_emalloc(sizeof(*fp));
         fp->fd = fd;
         fp->eof = append;
         fp->readonly = ro;
@@ -73,7 +72,7 @@ rut_endent(radut_file_t file)
         if (!file)
                 return;
         close(file->fd);
-        efree(file);
+        grad_free(file);
 }
 
 struct radutmp *
@@ -94,29 +93,30 @@ int
 rut_putent(radut_file_t file, struct radutmp *ent)
 {
         if (file->readonly) {
-                radlog(L_ERR, "rut_putent(): file opened readonly");
+                grad_log(L_ERR, "rut_putent(): file opened readonly");
                 return -1;
         }
         /* Step back one record unless we have reached eof */
         if (!file->eof &&
             lseek(file->fd, -(off_t)sizeof(file->ut), SEEK_CUR) < 0) {
-                radlog(L_ERR|L_PERROR, 
-                       "rut_putent(): lseek");
+                grad_log(L_ERR|L_PERROR, 
+                         "rut_putent(): lseek");
                 lseek(file->fd, (off_t)0, SEEK_SET);
                 return -1;
         }
         /* Lock the utmp file.  */
-        rad_lock(file->fd, sizeof(*ent), 0, SEEK_CUR);
+        grad_lock_file(file->fd, sizeof(*ent), 0, SEEK_CUR);
 
         if (write(file->fd, ent, sizeof(*ent)) != sizeof(*ent)) {
-                radlog(L_ERR|L_PERROR, 
-                       "rut_putent(): write");
+                grad_log(L_ERR|L_PERROR, 
+                         "rut_putent(): write");
         }
 
         memcpy(&file->ut, ent, sizeof(file->ut));
                
         /* Unlock the file */
-        rad_unlock(file->fd, sizeof(*ent), -(off_t)sizeof(file->fd), SEEK_CUR);
+        grad_unlock_file(file->fd, sizeof(*ent),
+			 -(off_t)sizeof(file->fd), SEEK_CUR);
 
         return 0;
 }
@@ -157,14 +157,16 @@ radutmp_putent(char *filename, struct radutmp *ut, int status)
                         if (ent->time < ut->time)
                                 break;
                         if (ent->type == P_LOGIN) {
-                                radlog(L_INFO,
+                                grad_log(L_INFO,
                 _("login: entry for NAS %s port %d duplicate"),
-                                       ip_iptostr(ntohl(ent->nas_address), ipbuf),
+                                       grad_ip_iptostr(ntohl(ent->nas_address),
+						       ipbuf),
                                        ent->nas_port);
                         } else {
-                                radlog(L_INFO,
+                                grad_log(L_INFO,
                 _("login: entry for NAS %s port %d wrong order"),
-                                       ip_iptostr(ntohl(ent->nas_address), ipbuf),
+                                       grad_ip_iptostr(ntohl(ent->nas_address),
+						       ipbuf),
                                        ent->nas_port);
                         }
                 }
@@ -173,12 +175,13 @@ radutmp_putent(char *filename, struct radutmp *ut, int status)
                         
                 if (status == DV_ACCT_STATUS_TYPE_STOP) {
                         if (ent->type == P_LOGIN) {
-                                radlog(L_ERR,
+                                grad_log(L_ERR,
    _("logout: entry for NAS %s port %d has wrong ID (expected %s found %s)"),
-                                       ip_iptostr(ntohl(ut->nas_address), ipbuf),
-                                       ent->nas_port,
-                                       ut->session_id,
-                                       ent->session_id);
+                                         grad_ip_iptostr(ntohl(ut->nas_address),
+					 	         ipbuf),
+                                         ent->nas_port,
+                                         ut->session_id,
+                                         ent->session_id);
                         }
                 }
                 
@@ -196,9 +199,9 @@ radutmp_putent(char *filename, struct radutmp *ut, int status)
         case DV_ACCT_STATUS_TYPE_STOP:
                 ut->type = P_IDLE;
                 if (!ent) {
-                        radlog(L_ERR,
+                        grad_log(L_ERR,
                          _("logout: login entry for NAS %s port %d not found"),
-                               ip_iptostr(ntohl(ut->nas_address), ipbuf), 
+                               grad_ip_iptostr(ntohl(ut->nas_address), ipbuf), 
                                ut->nas_port);
                 }
                 break;
@@ -215,7 +218,7 @@ radwtmp_putent(char *filename, struct radutmp *ut)
                 
         file = rut_setent(filename, 1);
         if (file == NULL) {
-                radlog(L_ERR|L_PERROR, _("can't open %s"), radwtmp_path);
+                grad_log(L_ERR|L_PERROR, _("can't open %s"), radwtmp_path);
                 return 1;
         }
         rut_putent(file, ut);
