@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -26,8 +26,8 @@
 #include <stdio.h>
 #include <errno.h>
 #include <sysdep.h>
-#include <symtab.h>
-#include <mem.h>
+#include <radius/symtab.h>
+#include <radius/mem.h>
 
 /* Hash sizes. These are prime numbers, the distance between each
    pair of them grows exponentially, starting from 64.
@@ -42,27 +42,17 @@ static size_t hash_size[] = {
 /* Maximum number of re-hashes: */
 static int max_rehash = sizeof (hash_size) / sizeof (hash_size[0]);
 
-
-Symbol * alloc_sym(char *, unsigned);
-static unsigned int hashval(unsigned char *s, unsigned bias);
-static void _sym_add(Symtab *symtab, unsigned h, Symbol *sp);
-
-Symtab *
-symtab_create(unsigned esize, int (*elfree)())
+grad_symbol_t *
+alloc_sym(const char *s, unsigned size)
 {
-        Symtab *symtab;
-        
-        symtab = emalloc(sizeof(*symtab));
-        symtab->elsize = esize;
-        symtab->elcnt = 0;
-        symtab->hash_num = -1;
-        symtab->elfree = elfree;
-        symtab->sym = NULL;
-        return symtab;
+        grad_symbol_t *ptr;
+        ptr = grad_emalloc(size);
+        ptr->name = grad_estrdup(s);
+        return ptr;
 }
 
-unsigned int
-hashval(unsigned char *s, unsigned bias)
+static unsigned int
+hashval(const unsigned char *s, unsigned bias)
 {
         unsigned h = 0;
 
@@ -73,12 +63,12 @@ hashval(unsigned char *s, unsigned bias)
         return h % bias;
 }
 
-void
-_sym_add(Symtab *symtab, unsigned h, Symbol *sp)
+static void
+_sym_add(grad_symtab_t *symtab, unsigned h, grad_symbol_t *sp)
 {
         sp->next = NULL;
         if (symtab->sym[h]) {
-                Symbol *prev;
+                grad_symbol_t *prev;
                 for (prev = symtab->sym[h]; prev->next; prev = prev->next)
                         ;
                 prev->next = sp;
@@ -86,10 +76,24 @@ _sym_add(Symtab *symtab, unsigned h, Symbol *sp)
                 symtab->sym[h] = sp;
 }
 
-int
-symtab_rehash(Symtab *symtab)
+grad_symtab_t *
+grad_symtab_create(unsigned esize, int (*elfree)())
 {
-        Symbol **old_table = symtab->sym;
+        grad_symtab_t *symtab;
+        
+        symtab = grad_emalloc(sizeof(*symtab));
+        symtab->elsize = esize;
+        symtab->elcnt = 0;
+        symtab->hash_num = -1;
+        symtab->elfree = elfree;
+        symtab->sym = NULL;
+        return symtab;
+}
+
+int
+grad_symtab_rehash(grad_symtab_t *symtab)
+{
+        grad_symbol_t **old_table = symtab->sym;
         int i;
   
         if (++symtab->hash_num >= max_rehash) {
@@ -97,13 +101,13 @@ symtab_rehash(Symtab *symtab)
                 abort();
         }
 
-        symtab->sym = emalloc(hash_size[symtab->hash_num] * symtab->elsize);
+        symtab->sym = grad_emalloc(hash_size[symtab->hash_num] * symtab->elsize);
 
         if (old_table) {
                 size_t old_size = hash_size[symtab->hash_num-1];
                 
                 for (i = 0; i < old_size; i++) {
-                        Symbol *sym, *next;
+                        grad_symbol_t *sym, *next;
                         
                         sym = old_table[i];
                         while (sym) {
@@ -117,21 +121,22 @@ symtab_rehash(Symtab *symtab)
                                 sym = next;
                         }
                 }
-                efree (old_table);
+                grad_free (old_table);
         }
         return 0;
 }
 
 void *
-sym_lookup_or_install(Symtab *symtab, char *name, int install)
+grad_sym_lookup_or_install(grad_symtab_t *symtab, const char *name,
+			   int install)
 {
         if (symtab->sym) {
-                Symbol *sp;
+                grad_symbol_t *sp;
                 unsigned h;
 
                 h = hashval((unsigned char *)name,
                             hash_size[symtab->hash_num]);
-
+		
                 for (sp = symtab->sym[h]; sp; sp = sp->next) {
                         if (strcmp(sp->name, name) == 0)
                                 return sp;
@@ -139,20 +144,20 @@ sym_lookup_or_install(Symtab *symtab, char *name, int install)
         }
         
         if (install)
-                return sym_install(symtab, name);
+                return grad_sym_install(symtab, name);
                 
         return NULL;
 }
 
 void *
-sym_install(Symtab *symtab, char *name)
+grad_sym_install(grad_symtab_t *symtab, const char *name)
 {
-        Symbol *sp;
+        grad_symbol_t *sp;
         unsigned int h;
 
         if (!symtab->sym
             || 10 * symtab->elcnt / hash_size[symtab->hash_num] > 20/3)
-                symtab_rehash(symtab);
+                grad_symtab_rehash(symtab);
         
         h = hashval((unsigned char *)name, hash_size[symtab->hash_num]);
 
@@ -163,13 +168,13 @@ sym_install(Symtab *symtab, char *name)
 }
 
 void *
-sym_lookup(Symtab *symtab, char *name)
+grad_sym_lookup(grad_symtab_t *symtab, const char *name)
 {
-        return sym_lookup_or_install(symtab, name, 0);
+        return grad_sym_lookup_or_install(symtab, name, 0);
 }
 
 void *
-sym_next(Symbol *sym)
+grad_sym_next(grad_symbol_t *sym)
 {
         char *name = sym->name;
         for (sym = sym->next; sym; sym = sym->next) {
@@ -179,22 +184,13 @@ sym_next(Symbol *sym)
         return NULL;
 }
 
-Symbol *
-alloc_sym(char *s, unsigned size)
-{
-        Symbol *ptr;
-        ptr = emalloc(size);
-        ptr->name = estrdup(s);
-        return ptr;
-}
-
 /*
  * Delete the symbol `sym' from symtab.
  */
 int
-symtab_delete(Symtab *symtab, Symbol *sym)
+grad_symtab_delete(grad_symtab_t *symtab, grad_symbol_t *sym)
 {
-        Symbol *sp, *prev;
+        grad_symbol_t *sp, *prev;
         unsigned h;
 
         if (!symtab->sym)
@@ -231,23 +227,23 @@ symtab_delete(Symtab *symtab, Symbol *sym)
          */
         if (symtab->elfree)
                 symtab->elfree(sp);
-        sym_free(sp);
+        grad_sym_free(sp);
         symtab->elcnt--;
         return 0;
 }
 
 void
-sym_free(Symbol *sp)
+grad_sym_free(grad_symbol_t *sp)
 {
-        efree(sp->name);
-        efree(sp);
+        grad_free(sp->name);
+        grad_free(sp);
 }
 
 void
-symtab_clear(Symtab *symtab)
+grad_symtab_clear(grad_symtab_t *symtab)
 {
         int i;
-        Symbol *sp, *next;
+        grad_symbol_t *sp, *next;
 
         if (!symtab || !symtab->sym)
                 return;
@@ -257,7 +253,7 @@ symtab_clear(Symtab *symtab)
                         next = sp->next;
                         if (symtab->elfree)
                                 symtab->elfree(sp);
-                        sym_free(sp);
+                        grad_sym_free(sp);
                 }
                 symtab->sym[i] = NULL;
         }
@@ -265,21 +261,21 @@ symtab_clear(Symtab *symtab)
 }
 
 void
-symtab_free(Symtab **symtab)
+grad_symtab_free(grad_symtab_t **symtab)
 {
 	if (!symtab || !*symtab)
 		return;
-        symtab_clear(*symtab);
-        efree((*symtab)->sym);
-        efree(*symtab);
+        grad_symtab_clear(*symtab);
+        grad_free((*symtab)->sym);
+        grad_free(*symtab);
         *symtab = NULL;
 }
 
 void
-symtab_iterate(Symtab *symtab, int (*fn)(), void *closure)
+grad_symtab_iterate(grad_symtab_t *symtab, int (*fn)(), void *closure)
 {
         int i;
-        Symbol *sym, *next;
+        grad_symbol_t *sym, *next;
 
 	if (!symtab->sym)
 		return;
