@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -17,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_ACCT_C 
-
 #define LOG_EMPTY_USERNAME
 
 #ifdef HAVE_CONFIG_H
@@ -42,31 +40,29 @@
 #include <errno.h>
 #include <sys/wait.h>
 #include <fcntl.h>
-#include <sysdep.h>
+
 #include <radiusd.h>
-#include <radutmp.h>
-#ifdef USE_SQL
-# include <radsql.h>
-#endif
+#include <radius/radutmp.h>
+#include <rewrite.h>
 
 int     doradwtmp = 1;
 
 static int write_wtmp(struct radutmp *ut);
-static int write_nas_restart(int status, UINT4 addr);
+static int write_nas_restart(int status, grad_uint32_t addr);
 static int check_ts(struct radutmp *ut);
 
-int rad_acct_system(RADIUS_REQ *radreq, int dowtmp);
-int rad_acct_db(RADIUS_REQ *radreq, int authtype);
-int rad_acct_ext(RADIUS_REQ *radreq);
+int rad_acct_system(grad_request_t *radreq, int dowtmp);
+int rad_acct_db(grad_request_t *radreq, int authtype);
+int rad_acct_ext(grad_request_t *radreq);
 
 
 /* Zap a user, or all users on a NAS, from the radutmp file. */
 int
-radzap(UINT4 nasaddr, int port, char *user, time_t t)
+radzap(grad_uint32_t nasaddr, int port, char *user, time_t t)
 {
         struct radutmp  *up;
         radut_file_t    file;
-        UINT4           netaddr;
+        grad_uint32_t           netaddr;
         
         if (t == 0) time(&t);
         netaddr = htonl(nasaddr);
@@ -154,18 +150,19 @@ end:
 }
 
 int
-check_attribute(VALUE_PAIR *check_pairs, int pair_attr,
+check_attribute(grad_avp_t *check_pairs, int pair_attr,
 		int pair_value, int def)
 {
-        VALUE_PAIR *pair;
+        grad_avp_t *pair;
 
-        if ((pair = avl_find(check_pairs, pair_attr)) == NULL)
+        if ((pair = grad_avl_find(check_pairs, pair_attr)) == NULL)
                 return def;
         do {
                 if (pair->avp_lvalue == pair_value)
                         return 1;
                 check_pairs = pair->next;
-        } while (check_pairs && (pair = avl_find(check_pairs, pair_attr)));
+        } while (check_pairs
+		 && (pair = grad_avl_find(check_pairs, pair_attr)));
         return 0;
 }
 
@@ -178,10 +175,10 @@ check_attribute(VALUE_PAIR *check_pairs, int pair_attr,
 
 /*  Store logins in the RADIUS utmp file. */
 int
-rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
+rad_acct_system(grad_request_t *radreq, int dowtmp)
 {
         struct radutmp  ut;
-        VALUE_PAIR *vp;
+        grad_avp_t *vp;
         int status = -1;
         int nas_address = 0;
         int protocol = -1;
@@ -190,10 +187,15 @@ rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
         int port_seen = 0;
         char buf[MAX_LONGNAME];
 
+	/* Do not do anything if system accounting is not requested */
+	if (!acct_system)
+		return 0;
+	
         /* A packet should have Acct-Status-Type attribute */
-        if ((vp = avl_find(radreq->request, DA_ACCT_STATUS_TYPE)) == NULL) {
-                radlog_req(L_ERR, radreq,
-                           _("no Acct-Status-Type attribute"));
+        if ((vp = grad_avl_find(radreq->request, DA_ACCT_STATUS_TYPE))
+	        == NULL) {
+                grad_log_req(L_ERR, radreq,
+                             _("no Acct-Status-Type attribute"));
                 return -1;
         }
 
@@ -204,8 +206,9 @@ rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
         ut.porttype = -1; /* Unknown so far */
 
         if (radreq->realm) {
-		RADIUS_SERVER *server = list_item(radreq->realm->queue->servers,
-						  radreq->server_no);
+		grad_server_t *server =
+			grad_list_item(radreq->realm->queue->servers,
+				       radreq->server_no);
 		if (server)
 			ut.realm_address = server->addr;
         }
@@ -292,17 +295,17 @@ rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
 
         /* Process Accounting-On/Off records */
         if (status == DV_ACCT_STATUS_TYPE_ACCOUNTING_ON && nas_address) {
-                radlog(L_NOTICE, 
-                        _("NAS %s started (Accounting-On packet seen)"),
-                        nas_ip_to_name(nas_address, buf, sizeof buf));
+                grad_log(L_NOTICE, 
+                         _("NAS %s started (Accounting-On packet seen)"),
+                         grad_nas_ip_to_name(nas_address, buf, sizeof buf));
                 radzap(nas_address, -1, NULL, ut.time);
                 write_nas_restart(status, ut.nas_address);
                 return 0;
         }
         if (status == DV_ACCT_STATUS_TYPE_ACCOUNTING_OFF && nas_address) {
-                radlog(L_NOTICE, 
-                        _("NAS %s shut down (Accounting-Off packet seen)"),
-                        nas_ip_to_name(nas_address, buf, sizeof buf));
+                grad_log(L_NOTICE, 
+                         _("NAS %s shut down (Accounting-Off packet seen)"),
+                         grad_nas_ip_to_name(nas_address, buf, sizeof buf));
                 radzap(nas_address, -1, NULL, ut.time);
                 write_nas_restart(status, ut.nas_address);
                 return 0;
@@ -312,9 +315,9 @@ rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
         if (status != DV_ACCT_STATUS_TYPE_START &&
             status != DV_ACCT_STATUS_TYPE_STOP &&
             status != DV_ACCT_STATUS_TYPE_ALIVE) {
-                radlog_req(L_NOTICE, 
-                           radreq, _("unknown packet type (%d)"),
-                           status);
+                grad_log_req(L_NOTICE, 
+                             radreq, _("unknown packet type (%d)"),
+                             status);
                 return 0;
         } else if (status == DV_ACCT_STATUS_TYPE_START ||
                    status == DV_ACCT_STATUS_TYPE_STOP) {
@@ -322,7 +325,7 @@ rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
                     ("%s: User %s at NAS %s port %d session %-*.*s",
                      status == DV_ACCT_STATUS_TYPE_START ? "start" : "stop",
                      ut.login,
-                     nas_ip_to_name(nas_address, buf, sizeof buf),
+                     grad_nas_ip_to_name(nas_address, buf, sizeof buf),
                      ut.nas_port,
                      sizeof(ut.session_id),
                      sizeof(ut.session_id),
@@ -368,37 +371,150 @@ rad_acct_system(RADIUS_REQ *radreq, int dowtmp)
         } else {
                 ret = -1;
                 stat_inc(acct, radreq->ipaddr, num_norecords);
-                radlog_req(L_NOTICE, radreq, _("NOT writing wtmp record"));
+                grad_log_req(L_NOTICE, radreq, _("NOT writing wtmp record"));
         }
         return ret;
 }
 
+static void
+disable_system_acct()
+{
+        radut_file_t file;
+	struct radutmp *up;
+	int written = 0;
+		
+        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+                return;
+
+        while (up = rut_getent(file)) {
+                switch (up->type) {
+		case P_LOGIN:
+                        up->type = P_IDLE;
+                        time(&up->time);
+                        rut_putent(file, up);
+                        /* Add a logout entry to the wtmp file. */
+                        write_wtmp(up);
+			break;
+
+		case P_ACCT_ENABLED:
+			time(&up->time);
+			up->type = P_ACCT_DISABLED;
+			rut_putent(file, up);
+			write_wtmp(up);
+			written = 1;
+			break;
+
+		case P_ACCT_DISABLED:
+			written = 1;
+			break;
+                }
+	}
+
+	if (!written) {
+		struct radutmp ut;
+		
+		memset(&ut, 0, sizeof ut);
+		time(&ut.time);
+		ut.type = P_ACCT_DISABLED;
+		rut_putent(file, &ut);
+	}
+	rut_endent(file);
+	grad_log(L_INFO, _("System accounting is disabled"));
+}
+
+static void
+enable_system_acct()
+{
+        radut_file_t file;
+	struct radutmp *up, ut;
+		
+        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+                return;
+
+        while (up = rut_getent(file)) {
+                if (up->type == P_ACCT_DISABLED) {
+                        up->type = P_ACCT_ENABLED;
+                        time(&up->time);
+                        rut_putent(file, up);
+                        /* Add an entry to the wtmp file. */
+                        write_wtmp(up);
+                }
+	}
+	rut_endent(file);
+}
+
+void
+system_acct_init()
+{
+	if (acct_system)
+		enable_system_acct();
+	else
+		disable_system_acct();
+}
+
+int
+mkdir_path(char *path, int perms)
+{
+	struct stat st;
+	
+	if (stat(path, &st)) {
+		char *p;
+		
+		if (errno != ENOENT) {
+			grad_log(L_ERR|L_PERROR,
+				 _("Cannot stat path component: %s"),
+				 path);
+			return 1;
+		}
+
+		p = strrchr(path, '/');
+		if (p && p > path) {
+			int rc;
+			*p = 0;
+			rc = mkdir_path(path, perms);
+			*p = '/';
+			if (rc)
+				return 1;
+		}
+
+		if (mkdir(path, perms)) {
+			grad_log(L_ERR|L_PERROR,
+				 _("Cannot create directory %s"),
+				 path);
+			return 1;
+		}
+	} else if (!S_ISDIR(st.st_mode)) {
+		grad_log(L_ERR,
+			 _("Path component is not a directory: %s"),
+			 path);
+		return 1;
+	}
+	return 0;
+}
+
 static int
 check_acct_dir()
 {
         struct stat st;
 
-        if (stat(radacct_dir, &st) == 0) {
-                if (S_ISDIR(st.st_mode)) {
+	if (stat(radacct_dir, &st) == 0) {
+		if (S_ISDIR(st.st_mode)) {
 			if (access(radacct_dir, W_OK)) {
-				radlog(L_ERR,
-				       _("%s: directory not writable"),
-				       radacct_dir);
+				grad_log(L_ERR,
+					 _("%s: directory not writable"),
+					 radacct_dir);
 				return 1;
 			}
-                        return 0;
+			return 0;
 		} else {
-                        radlog(L_ERR, _("%s: not a directory"),
-			       radacct_dir);
-                        return 1;
-                }
-        }
-        if (mkdir(radacct_dir, 0755)) {
-                radlog(L_ERR|L_PERROR,
-                        _("can't create accounting directory `%s'"),
-                        radacct_dir);
+			grad_log(L_ERR, _("%s: not a directory"),
+				 radacct_dir);
+			return 1;
+		}
+	}
+
+	if (mkdir_path(radacct_dir, 0755)) 
                 return 1;
-        }
         return 0;
 }
 
@@ -410,8 +526,8 @@ acct_after_config_hook(void *arg ARG_UNUSED, void *data ARG_UNUSED)
 	if (auth_detail || acct_detail) {
 		acct_dir_status = check_acct_dir();
 		if (acct_dir_status) {
-			radlog(L_NOTICE,
-			       _("Detailed accounting is disabled"));
+			grad_log(L_NOTICE,
+			         _("Detailed accounting is disabled"));
 			auth_detail = acct_detail = 0;
 		}
 	}
@@ -423,40 +539,33 @@ acct_init()
 	radiusd_set_postconfig_hook(acct_after_config_hook, NULL, 0);
 }
 
-int
-write_detail(RADIUS_REQ *radreq, int authtype, char *f)
+static char *
+make_legacy_detail_filename(grad_request_t *radreq, char *default_filename)
 {
-        FILE *outfd;
-        char nasname[MAX_LONGNAME];
-        char *dir, *path;
-        char *save;
-        VALUE_PAIR *pair;
-        UINT4 nas;
-        NAS *cl;
-        time_t curtime;
-        int ret = 0;
-
-	if (acct_dir_status)
-		return;
+	char nasname[MAX_LONGNAME];
+	grad_nas_t *cl;
+	grad_uint32_t ip;
+	grad_avp_t *pair;
 	
-        curtime = time(0);
-        
         /* Find out the name of this terminal server. We try to find
            the DA_NAS_IP_ADDRESS in the naslist file. If that fails,
            we look for the originating address.
            Only if that fails we resort to a name lookup. */
+	
         cl = NULL;
-        nas = radreq->ipaddr;
-        if ((pair = avl_find(radreq->request, DA_NAS_IP_ADDRESS)) != NULL)
-                nas = pair->avp_lvalue;
+        ip = radreq->ipaddr;
+        if ((pair = grad_avl_find(radreq->request, DA_NAS_IP_ADDRESS)) != NULL)
+                ip = pair->avp_lvalue;
+
         if (radreq->realm) {
-		RADIUS_SERVER *server = list_item(radreq->realm->queue->servers,
-						  radreq->server_no);
+		grad_server_t *server =
+			grad_list_item(radreq->realm->queue->servers,
+				       radreq->server_no);
 		if (server)
-			nas = server->addr;
+			ip = server->addr;
 	}
 
-        if ((cl = nas_lookup_ip(nas)) != NULL) {
+        if ((cl = grad_nas_lookup_ip(ip)) != NULL) {
                 if (cl->shortname[0])
                         strcpy(nasname, cl->shortname);
                 else
@@ -464,18 +573,106 @@ write_detail(RADIUS_REQ *radreq, int authtype, char *f)
         }
 
         if (cl == NULL) 
-                ip_gethostname(nas, nasname, sizeof(nasname));
-        
+                grad_ip_gethostname(ip, nasname, sizeof(nasname));
+
+	return grad_mkfilename(nasname, default_filename);
+}
+
+static char *
+make_detail_filename(grad_request_t *req, char *template,
+		     char *default_filename)
+{
+	if (!template) {
+		return make_legacy_detail_filename(req, default_filename);
+	} else if (template[0] == '=') {
+		Datatype type;
+		Datum datum;
+
+		/*FIXME: Should be compiled!*/
+		if (rewrite_interpret(template+1, req, &type, &datum)) 
+			return NULL;
+		if (type != String) {
+			grad_log(L_ERR, "%s: %s",
+			         template+1, _("wrong return type"));
+			return NULL;
+		}
+		return datum.sval;
+	} else {
+		struct obstack stk;
+		char *ptr;
+		
+		obstack_init(&stk);
+		ptr = radius_xlate(&stk, template, req, NULL);
+		if (ptr) 
+			ptr = grad_estrdup(ptr);
+		obstack_free(&stk, NULL);
+		return ptr;
+	}
+}
+
+static int
+make_path_to_file(char *filename, int perms)
+{
+	char *p = strrchr(filename, '/');
+	if (p) {
+		int rc;
+		*p = 0;
+		rc = mkdir_path(filename, perms);
+		*p = '/';
+		return rc;
+	}
+	return 0;
+}
+
+int
+write_detail(grad_request_t *radreq, int authtype, int rtype)
+{
+        FILE *outfd;
+        char *save;
+        grad_avp_t *pair;
+        time_t curtime;
+        int ret = 0;
+	char *filename;
+	char *template;
+	char *deffilename;
+	
+	if (acct_dir_status)
+		return 1;
+
+	curtime = time(0);
+
+	switch (rtype) {
+	case R_ACCT:
+		template = acct_detail_template;
+		deffilename = "detail";
+		break;
+
+	case R_AUTH:
+		template = auth_detail_template;
+		deffilename = "detail.auth";
+		break;
+
+	default:
+		return 1;
+	}
+	
+        filename = make_detail_filename(radreq, template, deffilename);
+	if (!filename)
+		return 1;
+
+	/* Change to the accounting directory */
+	if (chdir(radacct_dir)) {
+		grad_free(filename);
+		return 1;
+	}
+	
         /* Create a directory for this nas. */
-        dir = mkfilename(radacct_dir, nasname);
-        mkdir(dir, 0755);
+        make_path_to_file(filename, 0755);
 
         /* Write Detail file. */
-        path = mkfilename(dir, f);
-        efree(dir);
-        if ((outfd = fopen(path, "a")) == (FILE *)NULL) {
-                radlog(L_ERR|L_PERROR, _("can't open %s"), path);
-                ret = -1;
+        if ((outfd = fopen(filename, "a")) == NULL) {
+                grad_log(L_ERR|L_PERROR, _("can't open %s"), filename);
+                ret = 1;
         } else {
 
                 /* Post a timestamp */
@@ -486,14 +683,16 @@ write_detail(RADIUS_REQ *radreq, int authtype, char *f)
                         /* user wants a full (non-stripped) name to appear
                            in detail */
                         
-                        pair = avl_find(radreq->request, DA_ORIG_USER_NAME);
+                        pair = grad_avl_find(radreq->request,
+					     DA_ORIG_USER_NAME);
                         if (pair) 
                                 pair->name = "User-Name";
                         else
-                                pair = avl_find(radreq->request, DA_USER_NAME);
+                                pair = grad_avl_find(radreq->request,
+						     DA_USER_NAME);
                         if (pair) {
                                 fprintf(outfd, "\t%s\n", 
-                                        format_pair(pair, 0, &save));
+                                        grad_format_pair(pair, 0, &save));
                                 free(save);
                         }
                 }
@@ -511,7 +710,7 @@ write_detail(RADIUS_REQ *radreq, int authtype, char *f)
                                         break;
                         default:
                                 fprintf(outfd, "\t%s\n", 
-                                        format_pair(pair, 0, &save));
+                                        grad_format_pair(pair, 0, &save));
                                 free(save);
                         } 
                 }
@@ -537,39 +736,38 @@ write_detail(RADIUS_REQ *radreq, int authtype, char *f)
                 fclose(outfd);
                 ret = 0;
         }
-        efree(path);
-        
+	
+        grad_free(filename);
+        chdir("/");
         return ret;
 }
 
 int
-rad_acct_db(RADIUS_REQ *radreq, int authtype)
+rad_acct_db(grad_request_t *radreq, int authtype)
 {
         int rc = 0;
 
         if (acct_detail && ACCT_TYPE(radreq->request, DV_ACCT_TYPE_DETAIL))
-                rc = write_detail(radreq, authtype, "detail");
-#ifdef USE_SQL
+                rc = write_detail(radreq, authtype, R_ACCT);
         if (ACCT_TYPE(radreq->request, DV_ACCT_TYPE_SQL))
-                rad_sql_acct(radreq);
-#endif
+                radiusd_sql_acct(radreq);
         return rc;
 }
 
 int
-rad_acct_ext(RADIUS_REQ *radreq)
+rad_acct_ext(grad_request_t *radreq)
 {
-        VALUE_PAIR *p;
+        grad_avp_t *p;
 
 #ifdef USE_SERVER_GUILE
-        for (p = avl_find(radreq->request, DA_SCHEME_ACCT_PROCEDURE);
+        for (p = grad_avl_find(radreq->request, DA_SCHEME_ACCT_PROCEDURE);
 	     p;
-	     p = avl_find(p->next, DA_SCHEME_ACCT_PROCEDURE))
+	     p = grad_avl_find(p->next, DA_SCHEME_ACCT_PROCEDURE))
                 scheme_acct(p->avp_strvalue, radreq);
 #endif
-        for (p = avl_find(radreq->request, DA_ACCT_EXT_PROGRAM);
+        for (p = grad_avl_find(radreq->request, DA_ACCT_EXT_PROGRAM);
 	     p;
-	     p = avl_find(p->next, DA_ACCT_EXT_PROGRAM)) {
+	     p = grad_avl_find(p->next, DA_ACCT_EXT_PROGRAM)) {
     		switch (p->avp_strvalue[0]) {
 		case '/':
                 	radius_exec_program(p->avp_strvalue, radreq, NULL, 0);
@@ -583,7 +781,7 @@ rad_acct_ext(RADIUS_REQ *radreq)
 
 /* run accounting modules */
 int
-rad_accounting(RADIUS_REQ *radreq, int activefd, int verified)
+rad_accounting(grad_request_t *radreq, int activefd, int verified)
 {
 	log_open(L_ACCT);
 
@@ -592,7 +790,7 @@ rad_accounting(RADIUS_REQ *radreq, int activefd, int verified)
 #if defined(RT_ASCEND_EVENT_REQUEST) && defined(RT_ASCEND_EVENT_RESPONSE)
         /* Special handling for Ascend-Event-Request */
         if (radreq->code == RT_ASCEND_EVENT_REQUEST) {
-                write_detail(radreq, verified, "detail");
+                write_detail(radreq, verified, R_ACCT);
                 radius_send_reply(RT_ASCEND_EVENT_RESPONSE,
                                   radreq, NULL, NULL, activefd);
                 stat_inc(acct, radreq->ipaddr, num_resp);
@@ -613,191 +811,12 @@ rad_accounting(RADIUS_REQ *radreq, int activefd, int verified)
         return -1;
 }
 
-/* Check one terminal server to see if a user is logged in.
-   Return value:
-        1 if user is logged in
-        0 if user is not logged in */
-
-static int
-rad_check_ts(struct radutmp *ut)
-{
-        int result;
-        
-        switch (result = check_ts(ut)) {
-        case 0:
-        case 1:
-                return result;
-
-                /* The default return value is a question of policy.
-                   It is defined in /etc/raddb/config */
-        default:
-                if (checkrad_assume_logged) 
-                        radlog(L_NOTICE, _("assuming `%s' is logged in"),
-                               ut->login);
-                else
-                        radlog(L_NOTICE, _("assuming `%s' is NOT logged in"),
-                               ut->login);
-                return checkrad_assume_logged;
-        }
-        /*NOTREACHED*/
-}
-
-/* Return value: that of CHECKRAD process, i.e. 1 means true (user FOUND)
-   0 means false (user NOT found) */
-static int
-check_ts(struct radutmp *ut)
-{
-        NAS     *nas;
-
-        /* Find NAS type. */
-        if ((nas = nas_lookup_ip(ntohl(ut->nas_address))) == NULL) {
-                radlog(L_NOTICE, _("check_ts(): unknown NAS"));
-                return -1; 
-        }
-
-        /* Handle two special types */
-        if (strcmp(nas->nastype, "true") == 0)
-                return 1;
-        else if (strcmp(nas->nastype, "false") == 0)
-                return 0;
-
-        return checkrad(nas, ut);
-}
-
 int
-rad_check_realm(REALM *realm)
-{
-        int count;
-        struct radutmp *up;
-        radut_file_t file;
-        size_t maxlogins;
-		
-        if (!realm || (maxlogins = realm_get_quota(realm)) == 0)
-                return 0;
-
-        if ((file = rut_setent(radutmp_path, 0)) == NULL)
-                return 0;
-
-        /* Pass I: scan radutmp file */
-        count = 0;
-        while (up = rut_getent(file))
-                if (up->type == P_LOGIN
-		    && realm_verify_ip(realm, up->realm_address))
-                        count++;
-
-        if (count < maxlogins) {
-                rut_endent(file);
-                return 0;
-        }
-
-        /* Pass II: verify logins */
-        rut_rewind(file);
-        count = 0;
-
-        while (up = rut_getent(file)) {
-                if (!(up->type == P_LOGIN
-		      && realm_verify_ip(realm, up->realm_address)))
-                        continue;
-
-                if (rad_check_ts(up) == 1) {
-                        count++;
-                } else {
-                        /* Hung record - zap it. */
-                        up->type = P_IDLE;
-                        up->time = time(NULL);
-                                
-                        rut_putent(file, up);
-                        write_wtmp(up);
-                }
-        }
-        rut_endent(file);
-        return !(count < maxlogins);
-}
-
-/* See if a user is already logged in.
-   Check twice. If on the first pass the user exceeds his
-   max. number of logins, do a second pass and validate all
-   logins by querying the terminal server.
-
-   MPP validation is not as reliable as I'd wish it to be.
-
-   Return:
-      0 == OK,
-      1 == user exceeds its simultaneous-use parameter */
-int
-rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul, int *pcount)
-{
-        radut_file_t file;
-        int             count;
-        struct radutmp  *up;
-        VALUE_PAIR      *fra;
-        int             mpp = 1; /* MPP flag. The sense is reversed: 1 if
-                                    the session is NOT an MPP one */
-        UINT4           ipno = 0;
-        
-        if ((file = rut_setent(radutmp_path, 0)) == NULL)
-                return 0;
-
-        /*
-         * Pass I.
-         */
-        count = 0;
-        while (up = rut_getent(file)) 
-                if (strncmp(name, up->login, RUT_NAMESIZE) == 0 &&
-                    up->type == P_LOGIN) {
-                        count++;
-                }
-
-        *pcount = count;
-        
-        if (count < maxsimul) {
-                rut_endent(file);
-                return 0;
-        }
-
-
-        if ((fra = avl_find(request, DA_FRAMED_IP_ADDRESS)) != NULL)
-                ipno = htonl(fra->avp_lvalue);
-
-        /* Pass II. Check all registered logins. */
-
-        count = 0;
-
-        rut_rewind(file);
-        while (up = rut_getent(file)) {
-                if (strncmp(name, up->login, RUT_NAMESIZE) == 0
-                    && up->type == P_LOGIN) {
-                        if (rad_check_ts(up) == 1) {
-                                count++;
-                                /* Does it look like a MPP attempt? */
-                                if (ipno && up->framed_address == ipno)
-                                        switch (up->proto) {
-                                        case DV_FRAMED_PROTOCOL_PPP:
-                                        case DV_FRAMED_PROTOCOL_SLIP:
-                                        case 256: /* Ascend MPP */
-                                        	mpp = 0;
-                                        }
-                        } else {
-                                /* Hung record */
-                                up->type = P_IDLE;
-                                up->time = time(NULL);
-                                rut_putent(file, up);
-                                write_wtmp(up);
-                        }
-                }
-        }
-        rut_endent(file);
-
-        *pcount = count;
-        return (count < maxsimul) ? 0 : mpp;
-}
-
-int
-write_nas_restart(int status, UINT4 addr)
+write_nas_restart(int status, grad_uint32_t addr)
 {
         struct radutmp ut;
 
-        bzero(&ut, sizeof(ut));
+        memset(&ut, 0, sizeof(ut));
         if (status == DV_ACCT_STATUS_TYPE_ACCOUNTING_ON) 
                 ut.type = P_NAS_START;
         else
@@ -807,7 +826,74 @@ write_nas_restart(int status, UINT4 addr)
         return write_wtmp(&ut);
 }
 
+
+/* Multiple login checking */
 
+int
+radutmp_mlc_collect_user(char *name, grad_request_t *request,
+			 grad_list_t **sess_list)
+{
+        radut_file_t file;
+	struct radutmp *up;
+		
+        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+                return 1;
+
+        while (up = rut_getent(file)) {
+                if (strncmp(name, up->login, RUT_NAMESIZE) == 0 &&
+                    up->type == P_LOGIN) {
+			struct radutmp *tmp;
+			
+			if (*sess_list == NULL) 
+				*sess_list = grad_list_create();
+			tmp = grad_emalloc(sizeof(*tmp));
+			memcpy(tmp, up, sizeof(*tmp));
+			grad_list_append(*sess_list, tmp);
+                }
+	}
+	rut_endent(file);
+	return 0;
+}
+
+int
+radutmp_mlc_collect_realm(grad_request_t *request, grad_list_t **sess_list)
+{
+        radut_file_t file;
+	struct radutmp *up;
+		
+        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+                return 1;
+
+        while (up = rut_getent(file)) {
+                if (up->type == P_LOGIN
+		    && grad_realm_verify_ip(request->realm, up->realm_address)) {
+			struct radutmp *tmp;
+			
+			if (*sess_list == NULL) 
+				*sess_list = grad_list_create();
+			tmp = grad_emalloc(sizeof(*tmp));
+			memcpy(tmp, up, sizeof(*tmp));
+			grad_list_append(*sess_list, tmp);
+                }
+	}
+	rut_endent(file);
+	return 0;
+}
+
+void
+radutmp_mlc_close(struct radutmp *up)
+{
+	up->type = P_IDLE;
+	up->time = time(NULL);
+	radutmp_putent(radutmp_path, up, DV_ACCT_STATUS_TYPE_STOP);
+}
+
+int
+radutmp_mlc_enabled_p()
+{
+	return acct_system;
+}
+	
 
 
 
