@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from /home/gray/radius/radiusd/./config.y
+/*  A Bison parser, made from /home/gray/gnu/radius/radiusd/./config.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -15,7 +15,7 @@
 #line 1 "config.y"
 
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -50,7 +50,6 @@
 #include <ctype.h>
 	
 #include <radiusd.h>
-#include <obstack1.h>
 #include <cfg.h>
  
 #define YYMAXDEPTH 16
@@ -60,10 +59,10 @@ struct cfg_memblock {
 	int line_num;
 };
 
-static RAD_LIST /* of struct cfg_memblock */ *cfg_memory_pool;
+static grad_list_t /* of struct cfg_memblock */ *cfg_memory_pool;
  
-static RAD_LIST *_cfg_vlist_create(cfg_value_t *val);
-static void _cfg_vlist_append(RAD_LIST *vlist, cfg_value_t *val);
+static grad_list_t *_cfg_vlist_create(cfg_value_t *val);
+static void _cfg_vlist_append(grad_list_t *vlist, cfg_value_t *val);
 
 void *cfg_malloc(size_t size, void (*destructor)(void *));
  
@@ -96,7 +95,7 @@ static struct syntax_block *block;
 static void _cfg_push_block(struct cfg_stmt *stmt, cfg_end_fp end, void *data);
 static struct syntax_block *_cfg_pop_block();
 
-int _cfg_make_argv(cfg_value_t **argv, char *keyword, RAD_LIST *vlist);
+int _cfg_make_argv(cfg_value_t **argv, char *keyword, grad_list_t *vlist);
 void _cfg_free_argv(int argc, cfg_value_t *argv);
 
 struct cfg_stmt *_cfg_find_keyword(struct cfg_stmt *stmt, char *str);
@@ -109,15 +108,15 @@ static char *buffer;
 static char *curp;
 
 
-#line 98 "config.y"
+#line 97 "config.y"
 typedef union {
         size_t number;
         int bool;
-        UINT4 ipaddr;
+        grad_uint32_t ipaddr;
         char *string;
         cfg_value_t value;
         cfg_network_t network;
-	RAD_LIST *vlist;
+	grad_list_t *vlist;
 	struct cfg_stmt *stmt;
 } YYSTYPE;
 #ifndef YYDEBUG
@@ -193,10 +192,10 @@ static const short ccrhs[] = {    15,
 
 #if YYDEBUG != 0
 static const short ccrline[] = { 0,
-   123,   126,   127,   130,   131,   134,   135,   136,   142,   143,
-   146,   149,   178,   185,   189,   192,   216,   220,   225,   232,
-   238,   243,   248,   253,   258,   263,   268,   276,   281,   288,
-   295,   296
+   122,   125,   126,   129,   130,   133,   134,   135,   141,   142,
+   145,   148,   177,   184,   188,   191,   215,   219,   224,   231,
+   237,   242,   247,   252,   257,   262,   267,   275,   280,   287,
+   294,   295
 };
 #endif
 
@@ -813,13 +812,13 @@ ccreduce:
   switch (ccn) {
 
 case 8:
-#line 137 "config.y"
+#line 136 "config.y"
 {
 		      ccclearin; ccerrok;
 	      ;
     break;}
 case 12:
-#line 150 "config.y"
+#line 149 "config.y"
 {
 		      if (ccvsp[-2].stmt && ccvsp[-2].stmt->type == CS_BLOCK) {
 			      if (ccvsp[-2].stmt->handler) {
@@ -838,30 +837,30 @@ case 12:
 			      _cfg_push_block(ccvsp[-2].stmt->block, ccvsp[-2].stmt->end, ccvsp[-2].stmt->data);
 		      } else {
 			      if (block->stmt) {
-				      radlog(L_ERR,
-					     "%s:%d: %s",
-					     cfg_filename, cfg_line_num,
-					     _("unknown block statement"));
+				      grad_log(L_ERR,
+					       "%s:%d: %s",
+					       cfg_filename, cfg_line_num,
+					       _("unknown block statement"));
 			      }
 			      _cfg_push_block(NULL, NULL, NULL);
 		      }
 	      ;
     break;}
 case 13:
-#line 179 "config.y"
+#line 178 "config.y"
 {
 		      if (block->prev)
 			      _cfg_pop_block();
 	      ;
     break;}
 case 14:
-#line 186 "config.y"
+#line 185 "config.y"
 {
 		      ccval.vlist = NULL;
 	      ;
     break;}
 case 16:
-#line 193 "config.y"
+#line 192 "config.y"
 {
 		      if (ccvsp[-2].stmt) {
 			      if (ccvsp[-2].stmt->handler) {
@@ -878,82 +877,82 @@ case 16:
 					      ccerror("syntax error");
 			      }
 		      } else if (block->stmt)
-			      radlog(L_ERR,
-				     "%s:%d: %s",
-				     cfg_filename, cfg_line_num,
-				     _("unknown keyword"));
+			      grad_log(L_ERR,
+				       "%s:%d: %s",
+				       cfg_filename, cfg_line_num,
+				       _("unknown keyword"));
 	      ;
     break;}
 case 17:
-#line 217 "config.y"
+#line 216 "config.y"
 {
 		      ccval.vlist = _cfg_vlist_create(&ccvsp[0].value);
 	      ;
     break;}
 case 18:
-#line 221 "config.y"
+#line 220 "config.y"
 {
 		      _cfg_vlist_append(ccvsp[-1].vlist, &ccvsp[0].value);
 		      ccval.vlist = ccvsp[-1].vlist;
 	      ;
     break;}
 case 19:
-#line 226 "config.y"
+#line 225 "config.y"
 {
 		      _cfg_vlist_append(ccvsp[-2].vlist, &ccvsp[0].value);
 		      ccval.vlist = ccvsp[-2].vlist;
 	      ;
     break;}
 case 20:
-#line 233 "config.y"
+#line 232 "config.y"
 {
 		      ccval.stmt = _cfg_find_keyword(block->stmt, ccvsp[0].string);
 	      ;
     break;}
 case 21:
-#line 239 "config.y"
+#line 238 "config.y"
 {
 		      ccval.value.type = CFG_STRING;
 		      ccval.value.v.string = ccvsp[0].string;
 	      ;
     break;}
 case 22:
-#line 244 "config.y"
+#line 243 "config.y"
 {
 		      ccval.value.type = CFG_STRING;
 		      ccval.value.v.string = ccvsp[0].string;
 	      ;
     break;}
 case 23:
-#line 249 "config.y"
+#line 248 "config.y"
 {
 		      ccval.value.type = CFG_INTEGER;
 		      ccval.value.v.number = ccvsp[0].number;
 	      ;
     break;}
 case 24:
-#line 254 "config.y"
+#line 253 "config.y"
 {
 		      ccval.value.type = CFG_BOOLEAN;
 		      ccval.value.v.bool = ccvsp[0].bool;
 	      ;
     break;}
 case 25:
-#line 259 "config.y"
+#line 258 "config.y"
 {
 		      ccval.value.type = CFG_CHAR;
 		      ccval.value.v.ch = ccvsp[0].number;
 	      ;
     break;}
 case 26:
-#line 264 "config.y"
+#line 263 "config.y"
 {
 		      ccval.value.type = CFG_NETWORK;
 		      ccval.value.v.network = ccvsp[0].network;
 	      ;
     break;}
 case 27:
-#line 269 "config.y"
+#line 268 "config.y"
 {
 		      ccval.value.type = CFG_HOST;
 		      ccval.value.v.host.ipaddr = ccvsp[-2].ipaddr;
@@ -961,32 +960,32 @@ case 27:
 	      ;
     break;}
 case 28:
-#line 277 "config.y"
+#line 276 "config.y"
 {
 		      ccval.network.ipaddr = ccvsp[0].ipaddr;
 		      ccval.network.netmask = 0xffffffffL;
 	      ;
     break;}
 case 29:
-#line 282 "config.y"
+#line 281 "config.y"
 {
 		      ccval.network.ipaddr = ccvsp[-2].ipaddr;
 		      ccval.network.netmask = ccvsp[0].ipaddr;
 	      ;
     break;}
 case 30:
-#line 289 "config.y"
+#line 288 "config.y"
 {
 		      if (ccvsp[0].number != '/')
 			      YYERROR;
 	      ;
     break;}
 case 32:
-#line 297 "config.y"
+#line 296 "config.y"
 {
 		      if (ccvsp[0].number > 32) {
-			      radlog(L_ERR,
-				     _("invalid netmask length: %d"), ccvsp[0].number);
+			      grad_log(L_ERR,
+				       _("invalid netmask length: %d"), ccvsp[0].number);
 			      YYERROR;
 		      }
 		      ccval.ipaddr = (0xfffffffful >> (32-ccvsp[0].number)) << (32-ccvsp[0].number);
@@ -1214,7 +1213,7 @@ ccerrhandle:
     }
   return 1;
 }
-#line 307 "config.y"
+#line 306 "config.y"
 
 
 static void
@@ -1258,7 +1257,7 @@ copy_string()
 
         while (*curp) {
 		if (*curp == '\\') {
-			obstack_grow_backslash(&cfg_obstack, curp, &curp);
+			grad_obstack_grow_backslash(&cfg_obstack, curp, &curp);
 		} else if (*curp == quote) {
                         curp++;
                         break;
@@ -1308,7 +1307,7 @@ keyword()
 {
 	int tok;
 	
-	if ((tok = xlat_keyword(booleans, cclval.string, -1)) != -1) {
+	if ((tok = grad_xlat_keyword(booleans, cclval.string, -1)) != -1) {
 		cclval.bool = tok;
 		return T_BOOL;
 	}
@@ -1323,7 +1322,7 @@ cclex()
 again:
         skipws();
 
-        if (*curp == '#') { 
+        if (*curp == '#') {
                 skipline();
                 goto again;
         } 
@@ -1334,9 +1333,11 @@ again:
                 do {
                         while (*curp != '*') {
                                 if (*curp == 0) {
-                                        radlog(L_ERR, 
-                                               _("%s:%d: unexpected EOF in comment started at line %d"),
-                                                cfg_filename, cfg_line_num, keep_line);
+                                        grad_log(L_ERR, 
+                       _("%s:%d: unexpected EOF in comment started at line %d"),
+                                                 cfg_filename, 
+                                                 cfg_line_num, 
+                                                 keep_line);
                                         return 0;
                                 } else if (*curp == '\n')
                                         cfg_line_num++;
@@ -1359,11 +1360,18 @@ again:
                 cclval.string = copy_string();
                 return T_STRING;
         }
-        
+
+	if (*curp == '-' && !isspace(curp[1])) {
+		/* For the sake of passing keyword arguments
+		   to scheme */
+		cclval.string = copy_alpha();
+		return T_STRING;
+	}
+	
         if (isdigit(*curp)) {
                 if (copy_digit()) {
                         /* IP address */
-                        cclval.ipaddr = ip_strtoip(cclval.string);
+                        cclval.ipaddr = grad_ip_strtoip(cclval.string);
                         return T_IPADDR;
                 }
                 cclval.number = strtol(cclval.string, NULL, 0);
@@ -1385,7 +1393,7 @@ again:
 static int
 ccerror(char *s)
 {
-        radlog(L_ERR, "%s:%d: %s", cfg_filename, cfg_line_num, s);
+        grad_log(L_ERR, "%s:%d: %s", cfg_filename, cfg_line_num, s);
 }
                 
 /* ************************************************************************* */
@@ -1419,38 +1427,38 @@ _cfg_free_item(void *item, void *data)
 	struct cfg_memblock *p = item;
 	if (p->destructor)
 		p->destructor(p+1);
-	efree(p);
+	grad_free(p);
 	return 0;
 }
 
 void
 _cfg_free_memory_pool()
 {
-	list_destroy(&cfg_memory_pool, _cfg_free_item, NULL);
+	grad_list_destroy(&cfg_memory_pool, _cfg_free_item, NULL);
 }
 
 int
-_cfg_make_argv(cfg_value_t **argv, char *keyword, RAD_LIST *vlist)
+_cfg_make_argv(cfg_value_t **argv, char *keyword, grad_list_t *vlist)
 {
 	int argc;
 
 	if (vlist)
-		argc = list_count(vlist) + 1;
+		argc = grad_list_count(vlist) + 1;
 	else
 		argc = 1;
-	*argv = emalloc(sizeof(**argv)*argc);
+	*argv = grad_emalloc(sizeof(**argv)*argc);
 	(*argv)[0].type = CFG_STRING;
 	(*argv)[0].v.string = keyword;
 	if (vlist) {
 		int i;
 		cfg_value_t *val;
-		ITERATOR *itr = iterator_create(vlist);
+		grad_iterator_t *itr = grad_iterator_create(vlist);
 
 		if (itr) {
-			for (i = 1, val = iterator_first(itr); val;
-		     	     i++, val = iterator_next(itr))
+			for (i = 1, val = grad_iterator_first(itr); val;
+		     	     i++, val = grad_iterator_next(itr))
 				(*argv)[i] = *val;
-			iterator_destroy(&itr);
+			grad_iterator_destroy(&itr);
 		}
 	}
 	return argc;
@@ -1459,29 +1467,29 @@ _cfg_make_argv(cfg_value_t **argv, char *keyword, RAD_LIST *vlist)
 void
 _cfg_free_argv(int argc, cfg_value_t *argv)
 {
-	efree(argv);
+	grad_free(argv);
 }
 
 static void
 _cfg_vlist_destroy(void *arg)
 {
-	RAD_LIST **pl = arg;
-	list_destroy(pl, NULL, NULL);
+	grad_list_t **pl = arg;
+	grad_list_destroy(pl, NULL, NULL);
 }
 
 void
-_cfg_vlist_append(RAD_LIST *vlist, cfg_value_t *val)
+_cfg_vlist_append(grad_list_t *vlist, cfg_value_t *val)
 {
 	cfg_value_t *vp = cfg_malloc(sizeof(*vp), NULL);
 	*vp = *val;
-	list_append(vlist, vp);
+	grad_list_append(vlist, vp);
 }
 
-RAD_LIST *
+grad_list_t *
 _cfg_vlist_create(cfg_value_t *val)
 {
-	RAD_LIST *vlist = list_create();
-	RAD_LIST **lp = cfg_malloc(sizeof(*lp), _cfg_vlist_destroy);
+	grad_list_t *vlist = grad_list_create();
+	grad_list_t **lp = cfg_malloc(sizeof(*lp), _cfg_vlist_destroy);
 	*lp = vlist;
 	_cfg_vlist_append(vlist, val);
 	return vlist;
@@ -1490,7 +1498,7 @@ _cfg_vlist_create(cfg_value_t *val)
 void
 _cfg_push_block(struct cfg_stmt *stmt, cfg_end_fp end, void *block_data)
 {
-	struct syntax_block *p = emalloc(sizeof(*p));
+	struct syntax_block *p = grad_emalloc(sizeof(*p));
 	p->stmt = stmt;
 	p->end  = end;
 	p->data = block_data;
@@ -1507,7 +1515,7 @@ _cfg_pop_block()
 		block = p->prev;
 		if (p->end)
 			p->end(block ? block->data : NULL, p->data);
-		efree(p);
+		grad_free(p);
 	}
 	return block;
 }
@@ -1527,7 +1535,7 @@ int
 _get_value(cfg_value_t *arg, int type, void *base)
 {
         struct servent *s;
-        UINT4 ipaddr;
+        grad_uint32_t ipaddr;
         cfg_value_t value;
 
 	value = *arg;
@@ -1543,10 +1551,10 @@ _get_value(cfg_value_t *arg, int type, void *base)
                           if (s) 
                                   value.v.number = ntohs(s->s_port);
                           else {
-                                  radlog(L_ERR, 
-                                         _("%s:%d: no such service: %s"),
-                                         cfg_filename, cfg_line_num,
-                                         value.v.string);
+                                  grad_log(L_ERR, 
+                                           _("%s:%d: no such service: %s"),
+                                           cfg_filename, cfg_line_num,
+                                           value.v.string);
                                   return 0;
                           }
                           type = value.type = CFG_INTEGER;
@@ -1567,12 +1575,12 @@ _get_value(cfg_value_t *arg, int type, void *base)
                         break;
 			
                 case CFG_STRING:
-                        ipaddr = ip_gethostaddr(value.v.string);
+                        ipaddr = grad_ip_gethostaddr(value.v.string);
                         if (ipaddr == 0) {
-                                radlog(L_ERR, 
-                                       _("%s:%d: unknown host: %s"),
-                                       cfg_filename, cfg_line_num,
-                                       value.v.string);
+                                grad_log(L_ERR, 
+                                         _("%s:%d: unknown host: %s"),
+                                         cfg_filename, cfg_line_num,
+                                         value.v.string);
                         }
                         value.v.ipaddr = ipaddr;
                         value.type = CFG_IPADDR;
@@ -1603,11 +1611,11 @@ _get_value(cfg_value_t *arg, int type, void *base)
                 break;
 		
         case CFG_STRING:
-                string_replace((char**)base, value.v.string);
+                grad_string_replace((char**)base, value.v.string);
                 break;
 		
         case CFG_IPADDR:
-                *(UINT4*) base = value.v.ipaddr;
+                *(grad_uint32_t*) base = value.v.ipaddr;
                 break;
 		
         case CFG_BOOLEAN:
@@ -1619,9 +1627,9 @@ _get_value(cfg_value_t *arg, int type, void *base)
 		break;
 		
         default:
-                radlog(L_CRIT,
-                       _("INTERNAL ERROR at %s:%d: unknown datatype %d"),
-                       __FILE__, __LINE__, type);
+                grad_log(L_CRIT,
+                         _("INTERNAL ERROR at %s:%d: unknown datatype %d"),
+                         __FILE__, __LINE__, type);
         }
 	return 0;
 }
@@ -1633,35 +1641,36 @@ _get_value(cfg_value_t *arg, int type, void *base)
 void *
 cfg_malloc(size_t size,	void (*destructor)(void *))
 {
-	struct cfg_memblock *p = emalloc(size + sizeof(*p));
+	struct cfg_memblock *p = grad_emalloc(size + sizeof(*p));
 	p->destructor = destructor;
 	p->line_num = cfg_line_num;
 	if (!cfg_memory_pool)
-		cfg_memory_pool = list_create();
-	list_append(cfg_memory_pool, p);
+		cfg_memory_pool = grad_list_create();
+	grad_list_append(cfg_memory_pool, p);
 	return p+1;
 }
 
 void
 cfg_type_error(int type)
 {
-	radlog(L_ERR, 
-	       _("%s:%d: wrong datatype (should be %s)"),
-	       cfg_filename, cfg_line_num, typestr[type]);
+	grad_log(L_ERR, 
+	         _("%s:%d: wrong datatype (should be %s)"),
+	         cfg_filename, cfg_line_num, typestr[type]);
 }
 
 void
 cfg_argc_error(int few)
 {
-	radlog(L_ERR,
-	       "%s:%d: %s",
-	       cfg_filename, cfg_line_num,
-	       few ? _("too few arguments") : _("too many arguments"));
+	grad_log(L_ERR,
+	         "%s:%d: %s",
+	         cfg_filename, cfg_line_num,
+	         few ? _("too few arguments") : _("too many arguments"));
 }
 
 #define _check_argc(argc, max) \
  if (argc-1 > max) {\
-     radlog(L_ERR, "%s:%d: %s", cfg_filename, cfg_line_num, _("too many arguments"));\
+     grad_log(L_ERR, "%s:%d: %s", \
+              cfg_filename, cfg_line_num, _("too many arguments"));\
      return 0;\
  }		
 
@@ -1676,7 +1685,7 @@ int
 cfg_obsolete(int argc ARG_UNUSED, cfg_value_t *argv ARG_UNUSED,
 	     void *block_data ARG_UNUSED, void *handler_data ARG_UNUSED)
 {
-	radlog(L_WARN, _("%s:%d: obsolete statement"),
+	grad_log(L_WARN, _("%s:%d: obsolete statement"),
 	       cfg_filename, cfg_line_num);
 	return 0;
 }
@@ -1755,13 +1764,13 @@ cfg_read(char *fname, struct cfg_stmt *syntax, void *data)
         cfg_filename = fname;
 	_cfg_push_block(syntax, NULL, data);
         if (stat(cfg_filename, &st)) {
-                radlog(L_ERR|L_PERROR, _("can't stat `%s'"), cfg_filename);
+                grad_log(L_ERR|L_PERROR, _("can't stat `%s'"), cfg_filename);
                 return -1;
         }
         fd = open(cfg_filename, O_RDONLY);
         if (fd == -1) {
                 if (errno != ENOENT)
-                        radlog(L_ERR|L_PERROR, 
+                        grad_log(L_ERR|L_PERROR, 
                                 _("can't open config file `%s'"), cfg_filename);
                 return -1;
         }
@@ -1772,7 +1781,7 @@ cfg_read(char *fname, struct cfg_stmt *syntax, void *data)
         close(fd);
         curp = buffer;
 
-        radlog(L_INFO, _("reading %s"), cfg_filename);
+        grad_log(L_INFO, _("reading %s"), cfg_filename);
         cfg_line_num = 1;
 
         if (strncmp(curp, "#debug", 6) == 0) {
