@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,15 +28,14 @@
 #include <unistd.h>
 #include <errno.h>
 #include <syslog.h>
+
 #include <radiusd.h>
+#include <rewrite.h>
 
 static int logging_category = L_CAT(L_MAIN);
-static LIST /* of Channel*/ *chanlist;     /* List of defined channels */
-
-static void log_to_channel(Channel *chan, int cat, int pri,
-                           char *buf1, char *buf2, char *buf3);
-static FILE *channel_open_file(Channel *chan);
-static void channel_close_file(Channel *chan, FILE *fp);
+static grad_list_t /* of Channel*/ *chanlist;     /* List of defined channels */
+static char *log_prefix_hook;             /* Name of the global prefix hook */
+static char *log_suffix_hook;             /* Name of the global suffix hook */
 
 #define SP(p) ((p)?(p):"")
 
@@ -62,46 +63,6 @@ log_close()
         log_set_category(L_MAIN);
 }
 
-void
-vlog(int level,
-     const char *file, int line,
-     const char *func_name, int en,
-     const char *fmt, va_list ap)
-{
-        Channel *chan;
-        int cat, pri;
-        char *buf1 = NULL;
-        char *buf2 = NULL;
-        char *buf3 = NULL;
-	ITERATOR *itr = iterator_create(chanlist);
-
-        cat = L_CAT(level);
-        if (cat == 0)
-                cat = log_get_category();
-        pri = L_PRI(level);
-        
-        if (file) 
-                asprintf(&buf1, "%s:%d:%s: ", file, line, SP(func_name));
-        if (en)
-                asprintf(&buf3, ": %s", strerror(en));
-
-        vasprintf(&buf2, fmt, ap);
-        
-        for (chan = iterator_first(itr); chan; chan = iterator_next(itr)) {
-                /* Skip channels whith incorrect priority */
-                if (chan->pmask[cat] & L_MASK(pri))
-                        log_to_channel(chan, cat, pri, buf1, buf2, buf3);
-        }
-        iterator_destroy(&itr);
-
-        if (buf1)
-                free(buf1);
-        if (buf2)
-                free(buf2);
-        if (buf3)
-                free(buf3);     
-}
-
 static char *catname[] = { /* category names */
         N_("none"),
         N_("Main"),
@@ -122,12 +83,85 @@ static char *priname[] = { /* priority names */
         N_("debug")
 };
 
+static char *
+run_log_hook(const grad_request_t *req, const char *hook_name)
+{
+	char *result;
+	char nasbuf[MAX_LONGNAME];
+	
+	if (rewrite_invoke(String,
+			   &result,
+			   hook_name,
+			   req,
+			   "isi",
+			   req->code,
+			   grad_nas_request_to_name(req,
+						    nasbuf, sizeof nasbuf),
+			   req->id))
+		return NULL;
+	return result;
+}
+
+static void
+channel_format_prefix(char **bufp, Channel *chan, const grad_request_t *req)
+{
+	char **hook_name_ptr = &(chan->prefix_hook ? 
+		                   chan->prefix_hook : log_prefix_hook);
+	char *hook_res = NULL;
+
+	if (*hook_name_ptr) {
+		hook_res = run_log_hook(req, *hook_name_ptr);
+		if (!hook_res) 
+			*hook_name_ptr = NULL;
+	}
+
+	if (hook_res)  
+		asprintf(bufp, "%s", hook_res);
+}
+
+static void
+channel_format_suffix(char **bufp, Channel *chan, const grad_request_t *req)
+{
+	char **hook_name_ptr = &(chan->suffix_hook ? 
+		                   chan->suffix_hook : log_suffix_hook);
+	char *hook_res = NULL;
+	
+	if (*hook_name_ptr) {
+		hook_res = run_log_hook(req, *hook_name_ptr);
+		if (!hook_res) 
+			*hook_name_ptr = NULL;
+	}
+
+	if (hook_res)  
+		asprintf(bufp, "%s", hook_res);
+}
+
+static FILE *
+channel_open_file(Channel *chan)
+{
+        FILE *fp = NULL;
+
+        if (strcmp(chan->id.file, "stdout"))
+                fp = fopen(chan->id.file, "a");
+        return fp ? fp : stderr;
+}
+
+/*ARGSUSED*/
+static void
+channel_close_file(Channel *chan, FILE *fp)
+{
+        if (fp != stderr)
+                fclose(fp);
+}
+
 void
 log_to_channel(Channel *chan, int cat, int pri,
+	       const grad_request_t *req,
 	       char *buf1, char *buf2, char *buf3)
 {
         char *cat_pref = NULL;
         char *prefix = NULL;
+	char *req_prefix_buf = NULL, *req_suffix_buf = NULL;
         time_t  timeval;
         char buffer[256];
         struct tm *tm, tms;
@@ -151,6 +185,11 @@ log_to_channel(Channel *chan, int cat, int pri,
         if (cat_pref)
                 free(cat_pref);
 
+  	if (req) {
+		channel_format_prefix(&req_prefix_buf, chan, req);
+		channel_format_suffix(&req_suffix_buf, chan, req);
+	}
+	
         switch (chan->mode) {
         case LM_FILE:
                 if (chan->options & LO_MSEC) {
@@ -178,10 +217,14 @@ log_to_channel(Channel *chan, int cat, int pri,
                         fprintf(fp, "%s: ", prefix);
                 if (buf1)
                         fprintf(fp, "%s", buf1);
+		if (req_prefix_buf)
+			fprintf(fp, "%s", req_prefix_buf);
                 if (buf2)
                         fprintf(fp, "%s", buf2);
                 if (buf3)
                         fprintf(fp, "%s", buf3);
+		if (req_suffix_buf)
+			fprintf(fp, "%s", req_suffix_buf);
                 fprintf(fp, "\n");
                 channel_close_file(chan, fp);
                 break;
@@ -191,34 +234,77 @@ log_to_channel(Channel *chan, int cat, int pri,
                 if (chan->options & LO_PID)
                         spri |= LOG_PID;
                 if (prefix)
-                        syslog(spri, "%s: %s%s%s",
-			       prefix, SP(buf1), SP(buf2), SP(buf3));
+                        syslog(spri, "%s: %s%s%s%s%s",
+			       prefix,
+			       SP(buf1),
+			       SP(req_prefix_buf),
+			       SP(buf2), SP(buf3),
+			       SP(req_suffix_buf));
                 else
-                        syslog(spri, "%s%s%s",
-                               SP(buf1), SP(buf2), SP(buf3));
+                        syslog(spri, "%s%s%s%s%s",
+                               SP(buf1),
+			       SP(req_prefix_buf),
+			       SP(buf2), SP(buf3),
+			       SP(req_suffix_buf));
                 break;
         }
         
         if (prefix)
                 free(prefix);
+	if (req_prefix_buf)
+		free(req_prefix_buf);
+	if (req_suffix_buf)
+		free(req_suffix_buf);
 }
 
-FILE *
-channel_open_file(Channel *chan)
-{
-        FILE *fp = NULL;
-
-        if (strcmp(chan->id.file, "stdout"))
-                fp = fopen(chan->id.file, "a");
-        return fp ? fp : stderr;
-}
-
-/*ARGSUSED*/
 void
-channel_close_file(Channel *chan, FILE *fp)
+radiusd_logger(int level,
+	       const grad_request_t *req,
+	       const grad_locus_t *loc,
+	       const char *func_name,
+	       int en,
+	       const char *fmt, va_list ap)
 {
-        if (fp != stderr)
-                fclose(fp);
+        Channel *chan;
+        int cat, pri;
+        char *buf1 = NULL;
+        char *buf2 = NULL;
+        char *buf3 = NULL;
+        grad_iterator_t *itr = grad_iterator_create(chanlist);
+
+        cat = L_CAT(level);
+        if (cat == 0)
+                cat = log_get_category();
+        pri = L_PRI(level);
+        
+        if (loc) {
+		if (func_name)
+			asprintf(&buf1,
+				 "%s:%d:%s: ", loc->file, loc->line,
+				 func_name);
+		else
+			asprintf(&buf1,
+				 "%s:%d: ", loc->file, loc->line);
+	}
+	
+        if (en)
+                asprintf(&buf3, ": %s", strerror(en));
+
+        vasprintf(&buf2, fmt, ap);
+        
+        for (chan = grad_iterator_first(itr); chan; chan = grad_iterator_next(itr)) {
+                /* Skip channels whith incorrect priority */
+                if (chan->pmask[cat] & L_MASK(pri))
+                        log_to_channel(chan, cat, pri, req, buf1, buf2, buf3);
+        }
+        grad_iterator_destroy(&itr);
+
+        if (buf1)
+                free(buf1);
+        if (buf2)
+                free(buf2);
+        if (buf3)
+                free(buf3);     
 }
 
 /* Interface */
@@ -234,13 +320,14 @@ sqllog(int status, char *msg, ...)
         char *filename;
 
         filename = status ? "sql-lost" : "sql.log";
-        path = mkfilename(radacct_dir, filename);
+        path = grad_mkfilename(radacct_dir, filename);
         if ((fp = fopen(path, "a")) == NULL) {
-                radlog(L_ERR|L_PERROR, _("could not append to file %s"), path);
-                efree(path);
+                grad_log(L_ERR|L_PERROR,  
+                         _("could not append to file %s"), path);
+                grad_free(path);
                 return;
         }
-        efree(path);
+        grad_free(path);
 	va_start(ap, msg);
         vfprintf(fp, msg, ap);
         fprintf(fp, ";\n");
@@ -254,16 +341,18 @@ sqllog(int status, char *msg, ...)
 void
 channel_free(Channel *chan)
 {
-        efree(chan->name);
+        grad_free(chan->name);
         if (chan->mode == LM_FILE)
-                efree(chan->id.file);
-        efree(chan);
+                grad_free(chan->id.file);
+	grad_free(chan->prefix_hook);
+	grad_free(chan->suffix_hook);
+        grad_free(chan);
 }
 
 Channel *
 log_mark()
 {
-        return list_item(chanlist, 0);
+        return grad_list_item(chanlist, 0);
 }
 
 void
@@ -271,14 +360,14 @@ log_release(Channel *chan)
 {
         Channel *cp;
         int emerg, alert, crit;
-	ITERATOR *itr = iterator_create(chanlist);
+	grad_iterator_t *itr = grad_iterator_create(chanlist);
 
-	for (cp = iterator_first(itr); cp; cp = iterator_next(itr))
+	for (cp = grad_iterator_first(itr); cp; cp = grad_iterator_next(itr))
 		if (cp == chan)
 			break;
-        for (; cp; cp = iterator_next(itr)) {
+        for (; cp; cp = grad_iterator_next(itr)) {
                 if (!(cp->options & LO_PERSIST)) {
-			list_remove(chanlist, cp, NULL);
+			grad_list_remove(chanlist, cp, NULL);
                         channel_free(cp);
                 }
         }
@@ -288,7 +377,7 @@ log_release(Channel *chan)
         emerg = L_EMERG;
         alert = L_ALERT;
         crit  = L_CRIT;
-	for (cp = iterator_first(itr); cp; cp = iterator_next(itr)) {
+	for (cp = grad_iterator_first(itr); cp; cp = grad_iterator_next(itr)) {
                 int i;
                 for (i = 1; i < L_NCAT; i++) {
                         if (emerg && (cp->pmask[i] & L_MASK(emerg)))
@@ -299,11 +388,30 @@ log_release(Channel *chan)
                                 crit = 0;
                 }
         }
-	iterator_destroy(&itr);
+	grad_iterator_destroy(&itr);
         if (emerg || alert || crit)
                 log_set_default("##emerg##", -1, emerg|alert|crit);
 }
 
+int
+log_change_owner(RADIUS_USER *usr)
+{
+        Channel *cp;
+	int errcnt = 0;
+	grad_iterator_t *itr = grad_iterator_create(chanlist);
+	for (cp = grad_iterator_first(itr); cp; cp = grad_iterator_next(itr)) {
+		if (cp->mode == LM_FILE
+		    && chown(cp->id.file, usr->uid, usr->gid)) {
+			grad_log(L_ERR,
+			         _("%s: cannot change owner to %d:%d"),
+			         cp->id.file, usr->uid, usr->gid);
+			errcnt++;
+		}
+	}
+	grad_iterator_destroy(&itr);
+	return errcnt;
+}
+
 static int
 _chancmp(const void *item, const void *data)
 {
@@ -315,7 +423,7 @@ _chancmp(const void *item, const void *data)
 Channel *
 channel_lookup(char *name)
 {
-        return list_locate(chanlist, name, _chancmp);
+        return grad_list_locate(chanlist, name, _chancmp);
 }
 
 void
@@ -327,36 +435,39 @@ register_channel(Channel *chan)
 
         if (chan->mode == LM_FILE) {
                 if (strcmp(chan->id.file, "stdout")) {
-                        filename = mkfilename(radlog_dir ?
-                                              radlog_dir : RADLOG_DIR,
-                                              chan->id.file);
+                        filename = grad_mkfilename(radlog_dir ?
+						   radlog_dir : RADLOG_DIR,
+						   chan->id.file);
                         
                         /* check the accessibility of the file */
                         fp = fopen(filename, "a");
                         if (!fp) {
-                                radlog(L_CRIT|L_PERROR,
-                                       _("can't access `%s'"), filename);
-                                efree(filename);
-                                filename = estrdup("stdout");
+                                grad_log(L_CRIT|L_PERROR,
+                                         _("can't access log file `%s'"),
+                                         filename);
+                                grad_free(filename);
+                                filename = grad_estrdup("stdout");
                         } else
                                 fclose(fp);
                 } else
-                        filename = estrdup("stdout");
+                        filename = grad_estrdup("stdout");
         } else if (chan->mode == LM_SYSLOG) {
         } 
 
-        channel = emalloc(sizeof(*channel));
-        channel->name = estrdup(chan->name);
+        channel = grad_emalloc(sizeof(*channel));
+        channel->name = grad_estrdup(chan->name);
         channel->mode = chan->mode;
         if (chan->mode == LM_FILE)
                 channel->id.file = filename;
         else if (chan->mode == LM_SYSLOG)
                 channel->id.prio = chan->id.prio;
         channel->options = chan->options;
-
+	channel->prefix_hook = chan->prefix_hook;
+	channel->suffix_hook = chan->suffix_hook;
+	
 	if (!chanlist)
-		chanlist = list_create();
-	list_prepend(chanlist, channel);
+		chanlist = grad_list_create();
+	grad_list_prepend(chanlist, channel);
 }
 
 void
@@ -385,7 +496,7 @@ _regcat(void *item, void *data)
 }
 
 void
-register_category(int cat, int pri, LIST *clist)
+register_category(int cat, int pri, grad_list_t *clist)
 {
 	struct category_closure clos;
 
@@ -394,7 +505,7 @@ register_category(int cat, int pri, LIST *clist)
 
 	clos.cat = cat;
 	clos.pri = pri;
-	list_iterate(clist, _regcat, &clos);
+	grad_list_iterate(clist, _regcat, &clos);
 }
 
 /* Auxiliary calls */
@@ -421,7 +532,8 @@ log_set_default(char *name, int cat, int pri)
         chan.name = name;
         chan.id.file = "radius.log";
         chan.options = LO_CAT|LO_PRI;
-
+	chan.prefix_hook = chan.suffix_hook = NULL;
+	
         if (!channel_lookup(name))
                 register_channel(&chan);
         register_category0(cat, pri, channel_lookup(name));
@@ -433,16 +545,22 @@ format_exit_status(char *buffer, int buflen, int status)
 {
 	if (WIFEXITED(status)) {
 		snprintf(buffer, buflen,
+/* TRANSLATORS: the subject will always be printed before this msgid.
+   for example: "child 222 exited with status 0"
+*/
 			 _("exited with status %d"),
 			 WEXITSTATUS(status));
 	} else if (WIFSIGNALED(status)) {
 		snprintf(buffer, buflen,
+/* TRANSLATORS: the subject will always be printed before this msgid. */
 			 _("terminated on signal %d"),
 			 WTERMSIG(status));
 	} else
+/* TRANSLATORS: the subject will always be printed before this msgid. */
 		snprintf(buffer, buflen, _("terminated"));
 }
 
+
 /* ************************************************************************* */
 /* Configuration issues */
 
@@ -451,7 +569,7 @@ static struct category_def {
 	int init;
 	int cat;
 	int pri;
-        LIST /* of Channel */ *clist;
+        grad_list_t /* of Channel */ *clist;
         int level;
 } cat_def;
 
@@ -508,6 +626,10 @@ logging_stmt_handler(int argc, cfg_value_t *argv, void *block_data,
 		     void *handler_data)
 {
 	mark = log_mark();
+	grad_free(log_prefix_hook);
+	log_prefix_hook = NULL;
+	grad_free(log_suffix_hook);
+	log_suffix_hook = NULL;
 	return 0;
 }
 
@@ -549,9 +671,9 @@ static int
 channel_stmt_end(void *block_data, void *handler_data)
 {
 	if (channel.mode == LM_UNKNOWN) {
-		radlog(L_ERR,
-		       _("%s:%d: no channel mode for `%s'"), 
-		       cfg_filename, cfg_line_num, channel.name);
+		grad_log(L_ERR,
+		         _("%s:%d: no channel mode for `%s'"), 
+		         cfg_filename, cfg_line_num, channel.name);
 	} else 
 		register_channel(&channel);
 	return 0;
@@ -562,9 +684,9 @@ get_priority(cfg_value_t *argv)
 {
 	if (argv[0].type != CFG_CHAR || argv[1].type != CFG_STRING)
 		return 1;
-	cat_def.pri = xlat_keyword(log_priorities,
-				   argv[1].v.string,
-				   -1);
+	cat_def.pri = grad_xlat_keyword(log_priorities,
+					argv[1].v.string,
+					-1);
 	if (cat_def.pri == -1)
 		return 1;
 
@@ -602,12 +724,12 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.cat = xlat_keyword(log_categories,
-						   argv[1].v.string, -1);
+			cat_def.cat = grad_xlat_keyword(log_categories,
+							argv[1].v.string, -1);
 			if (cat_def.cat == -1) {
-				cat_def.pri = xlat_keyword(log_priorities,
-							   argv[1].v.string,
-							   -1);
+				cat_def.pri = grad_xlat_keyword(log_priorities,
+							      argv[1].v.string,
+								-1);
 				if (cat_def.pri == -1)
 					return 1;
 				cat_def.pri = L_UPTO(cat_def.pri);
@@ -633,8 +755,8 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.cat = xlat_keyword(log_categories,
-						   argv[1].v.string, -1);
+			cat_def.cat = grad_xlat_keyword(log_categories,
+							argv[1].v.string, -1);
 			if (cat_def.cat == -1) 
 				return 1;
 			break;
@@ -652,8 +774,8 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.pri = xlat_keyword(log_priorities,
-						   argv[3].v.string, -1);
+			cat_def.pri = grad_xlat_keyword(log_priorities,
+							argv[3].v.string, -1);
 			if (cat_def.pri == -1) 
 				return 1;
 			cat_def.pri = L_UPTO(cat_def.pri);
@@ -677,8 +799,8 @@ category_stmt_handler(int argc, cfg_value_t *argv,
 			break;
 			
 		case CFG_STRING:
-			cat_def.cat = xlat_keyword(log_categories,
-						   argv[1].v.string, -1);
+			cat_def.cat = grad_xlat_keyword(log_categories,
+							argv[1].v.string, -1);
 			if (cat_def.cat == -1) 
 				return 1;
 			break;
@@ -710,13 +832,13 @@ category_stmt_end(void *block_data, void *handler_data)
 			break;
 		default:
 			if (cat_def.level)
-				radlog(L_WARN,
-				       "%s:%d: %s",
-				       cfg_filename, cfg_line_num,
+				grad_log(L_WARN,
+				         "%s:%d: %s",
+				         cfg_filename, cfg_line_num,
 				_("no levels applicable for this category"));
 		}
 		register_category(cat_def.cat, cat_def.pri, cat_def.clist);
-		list_destroy(&cat_def.clist, NULL, NULL);
+		grad_list_destroy(&cat_def.clist, NULL, NULL);
 	}
 	return 0;
 }
@@ -738,13 +860,13 @@ category_set_channel(int argc, cfg_value_t *argv,
 	channel = channel_lookup(argv[1].v.string);
 
 	if (!channel) {
-		radlog(L_ERR,
-		       _("%s:%d: channel `%s' not defined"),
-		       cfg_filename, cfg_line_num, argv[1].v.string);
+		grad_log(L_ERR,
+		         _("%s:%d: channel `%s' not defined"),
+		         cfg_filename, cfg_line_num, argv[1].v.string);
 	} else {
 		if (!cat_def.clist)
-			cat_def.clist = list_create();
-		list_append(cat_def.clist, channel);
+			cat_def.clist = grad_list_create();
+		grad_list_append(cat_def.clist, channel);
 	}
 	
 	return 0;
@@ -782,10 +904,10 @@ category_set_level(int argc, cfg_value_t *argv,
 		int level;
 		
 		if (argv[i].type != CFG_STRING) {
-			radlog(L_ERR,
-			       _("%s:%d: list item %d has wrong datatype"),
-			       cfg_filename, cfg_line_num,
-			       i);
+			grad_log(L_ERR,
+			         _("%s:%d: list item %d has wrong datatype"),
+			         cfg_filename, cfg_line_num,
+			         i);
 			return 1;
 		}
 		modname = argv[i++].v.string;
@@ -798,9 +920,9 @@ category_set_level(int argc, cfg_value_t *argv,
 			level = argv[i++].v.number;
 		}
 		if (set_module_debug_level(modname, level)) {
-			radlog(L_WARN,
-			       _("%s:%d: no such module name: %s"),
-			       cfg_filename, cfg_line_num, modname);
+			grad_log(L_WARN,
+			         _("%s:%d: no such module name: %s"),
+			         cfg_filename, cfg_line_num, modname);
 		}
 	}
 	return 0;
@@ -841,7 +963,8 @@ channel_syslog_handler(int argc, cfg_value_t *argv, void *block_data,
 		break;
 
 	case CFG_STRING:
-		facility = xlat_keyword(syslog_facility, argv[1].v.string, -1);
+		facility = grad_xlat_keyword(syslog_facility,
+					     argv[1].v.string, -1);
 		break;
 
 	default:
@@ -860,7 +983,8 @@ channel_syslog_handler(int argc, cfg_value_t *argv, void *block_data,
 		break;
 
 	case CFG_STRING:
-		prio = xlat_keyword(syslog_priority, argv[3].v.string, -1);
+		prio = grad_xlat_keyword(syslog_priority,
+					 argv[3].v.string, -1);
 		break;
 
 	default:
@@ -911,6 +1035,10 @@ static struct cfg_stmt channel_stmt[] = {
 	  NULL, NULL },
 	{ "print-milliseconds", CS_STMT, NULL, channel_set_flag,
 	  (void*)LO_MSEC, NULL, NULL },
+	{ "prefix-hook", CS_STMT, NULL, cfg_get_string, &channel.prefix_hook,
+	  NULL, NULL },
+	{ "suffix-hook", CS_STMT, NULL, cfg_get_string, &channel.suffix_hook,
+	  NULL, NULL },
 	{ NULL }
 };
 
@@ -932,6 +1060,10 @@ struct cfg_stmt logging_stmt[] = {
 	  channel_stmt_handler, NULL, channel_stmt, channel_stmt_end },
 	{ "category", CS_BLOCK, NULL,
 	  category_stmt_handler, NULL, category_stmt, category_stmt_end }, 
+	{ "prefix-hook", CS_STMT, NULL, cfg_get_string, &log_prefix_hook,
+	  NULL, NULL },
+	{ "suffix-hook", CS_STMT, NULL, cfg_get_string, &log_suffix_hook,
+	  NULL, NULL },
 	{ NULL },
 };
 
