@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -17,27 +17,24 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_SQL_C
-
 #if defined(HAVE_CONFIG_H)
 # include <config.h>
 #endif
 
-#if defined(USE_SQL)
-
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <radiusd.h>
-#include <radsql.h>
-#include <obstack1.h>
 #include <ctype.h>
+
+#include <radiusd.h>
 #ifdef USE_SERVER_GUILE
 # include <libguile.h>
 #endif
 
+#if defined(USE_SQL)
+
 static void sql_check_config(const char *, SQL_cfg *);
 static struct sql_connection *attach_sql_connection(int type);
 static void detach_sql_connection(int type);
@@ -64,20 +61,18 @@ static struct sql_connection *sql_conn[SQL_NSERVICE];
 #define STMT_DOACCT                 7
 #define STMT_AUTH_DB                8
 #define STMT_ACCT_DB                9
-#define STMT_AUTH_QUERY            10
-#define STMT_ACCT_START_QUERY      11
-#define STMT_ACCT_STOP_QUERY       12
-#define STMT_ACCT_NASUP_QUERY      13
-#define STMT_ACCT_NASDOWN_QUERY    14
-#define STMT_ACCT_KEEPALIVE_QUERY  15
-#define STMT_QUERY_BUFFER_SIZE     16
-#define STMT_IDLE_TIMEOUT          17
-#define STMT_MAX_AUTH_CONNECTIONS  18
-#define STMT_MAX_ACCT_CONNECTIONS  19
-#define STMT_GROUP_QUERY           20
-#define STMT_REPLY_ATTR_QUERY      21
-#define STMT_INTERFACE             22
-#define STMT_CHECK_ATTR_QUERY      23
+#define STMT_ACCT_KEEPALIVE_QUERY  10
+#define STMT_QUERY_BUFFER_SIZE     11
+#define STMT_IDLE_TIMEOUT          12
+#define STMT_MAX_AUTH_CONNECTIONS  13
+#define STMT_MAX_ACCT_CONNECTIONS  14
+#define STMT_INTERFACE             15
+#define STMT_QUERY_BASE           100
+
+#define STMT_QUERY(c) (STMT_QUERY_BASE+c)
+#define STMT_QUERY_CODE(c) ((c)-STMT_QUERY_BASE)
+#define STMT_QUERY_P(t) ((t)>=STMT_QUERY_BASE \
+                         && STMT_QUERY_CODE(t)<num_radius_sql_query)
 
 
 /* *********************** Configuration File Parser *********************** */
@@ -101,21 +96,39 @@ struct keyword sql_keyword[] = {
         { "doacct",             STMT_DOACCT },
         { "auth_db",            STMT_AUTH_DB },
         { "acct_db",            STMT_ACCT_DB },
-        { "auth_query",         STMT_AUTH_QUERY },
-        { "group_query",        STMT_GROUP_QUERY },
-        { "attr_query",         STMT_REPLY_ATTR_QUERY },
-        { "reply_attr_query",   STMT_REPLY_ATTR_QUERY },
-        { "check_attr_query",   STMT_CHECK_ATTR_QUERY },
-        { "acct_start_query",   STMT_ACCT_START_QUERY },
-        { "acct_stop_query",    STMT_ACCT_STOP_QUERY },
-        { "acct_alive_query",   STMT_ACCT_KEEPALIVE_QUERY },
-        { "acct_keepalive_query", STMT_ACCT_KEEPALIVE_QUERY },
-        { "acct_nasup_query",   STMT_ACCT_NASUP_QUERY },
-        { "acct_nasdown_query", STMT_ACCT_NASDOWN_QUERY },
         { "interface",          STMT_INTERFACE },
+        { "auth_query",         STMT_QUERY(auth_query) },
+        { "group_query",        STMT_QUERY(group_query) },
+        { "attr_query",         STMT_QUERY(reply_attr_query) },
+        { "reply_attr_query",   STMT_QUERY(reply_attr_query) },
+        { "check_attr_query",   STMT_QUERY(check_attr_query) },
+        { "acct_start_query",   STMT_QUERY(acct_start_query) },
+        { "acct_stop_query",    STMT_QUERY(acct_stop_query) },
+        { "acct_alive_query",   STMT_QUERY(acct_keepalive_query) },
+        { "acct_keepalive_query", STMT_QUERY(acct_keepalive_query) },
+        { "acct_nasup_query",   STMT_QUERY(acct_nasup_query) },
+        { "acct_nasdown_query", STMT_QUERY(acct_nasdown_query) },
+	{ "mlc_user_query",     STMT_QUERY(mlc_user_query) },
+	{ "mlc_realm_query",    STMT_QUERY(mlc_realm_query) },
+	{ "mlc_stop_query",     STMT_QUERY(mlc_stop_query) },
+	{ "auth_success_query", STMT_QUERY(auth_success_query) },
+	{ "auth_failure_query", STMT_QUERY(auth_failure_query) },
         { NULL }
 };
 
+static char *
+sql_keyword_name(int kw)
+{
+	struct keyword *p;
+
+	for (p = sql_keyword; p->name; p++)
+		if (p->tok == kw)
+			return p->name;
+	return NULL;
+}
+
+#define sql_query_name(c) sql_keyword_name(STMT_QUERY(c)) 
+
 /*
  * Chop off trailing whitespace. Return length of the resulting string
  */
@@ -180,7 +193,7 @@ getline()
                 while (*cur_ptr && isspace(*cur_ptr))
                         cur_ptr++;
         }
-        stmt_type = xlat_keyword(sql_keyword, cur_line, -1);
+        stmt_type = grad_xlat_keyword(sql_keyword, cur_line, -1);
         return cur_ptr;
 }
 
@@ -214,7 +227,7 @@ sql_digest(SQL_cfg *cfg)
                 + STRLEN(cfg->acct_db)
                 + 1;
 
-        digest = emalloc(length);
+        digest = grad_emalloc(length);
         p = digest;
         
         *(int*)p = length;
@@ -275,32 +288,33 @@ sql_cfg_comp(SQL_cfg *a, SQL_cfg *b)
         dig1 = sql_digest(a);
         dig2 = sql_digest(b);
         rc = sql_digest_comp(dig1, dig2);
-        efree(dig1);
-        efree(dig2);
+        grad_free(dig1);
+        grad_free(dig2);
         return rc;
 }
 
 int 
-rad_sql_init()
+radiusd_sql_config()
 {
         char *sqlfile;
         char *ptr;
         time_t timeout;
         SQL_cfg new_cfg;
+	int i;
+	
+#define FREE(a) if (a) grad_free(a); a = NULL
 
-#define FREE(a) if (a) efree(a); a = NULL
-
-        bzero(&new_cfg, sizeof(new_cfg));
+        memset(&new_cfg, 0, sizeof(new_cfg));
         new_cfg.keepopen = 0;
         new_cfg.idle_timeout = 4*3600; /* four hours */
         new_cfg.active[SQL_ACCT] = 0;
         new_cfg.active[SQL_AUTH] = 0;
 
-        sqlfile = mkfilename(radius_dir, "sqlserver");
+        sqlfile = grad_mkfilename(radius_dir, "sqlserver");
         /* Open source file */
         if ((sqlfd = fopen(sqlfile, "r")) == (FILE *) NULL) {
-                radlog(L_ERR, _("could not read sqlserver file %s"), sqlfile);
-                efree(sqlfile);
+                grad_log(L_ERR, _("could not read sqlserver file %s"), sqlfile);
+                grad_free(sqlfile);
                 return -1;
         }
         line_no = 0;
@@ -308,162 +322,133 @@ rad_sql_init()
         obstack_init(&parse_stack);
         while (getline()) {
                 if (stmt_type == -1) {
-                        radlog(L_ERR,
-                               "%s:%d: %s",
-                               sqlfile, line_no,
-			       _("unrecognized keyword"));
+                        grad_log(L_ERR,
+                                 "%s:%d: %s",
+                                 sqlfile, line_no,
+			         _("unrecognized keyword"));
                         continue;
                 }
                 /* Each keyword should have an argument */
                 if (cur_ptr == NULL) {
-                        radlog(L_ERR,
-                               "%s:%d: %s",
-                               sqlfile, line_no,
-			       _("required argument missing"));
+                        grad_log(L_ERR,
+                                 "%s:%d: %s",
+                                 sqlfile, line_no,
+			         _("required argument missing"));
                         continue;
                 }
                 
                 switch (stmt_type) {
                 case STMT_SERVER:
-                       	if (cur_ptr[0] != '/' && !ip_gethostaddr(cur_ptr)) {
-                                radlog(L_ERR,
-                                       _("%s:%d: unknown host: %s"),
-                                       sqlfile, line_no,
-                                       cur_ptr);
+                       	if (cur_ptr[0] != '/'
+			    && !grad_ip_gethostaddr(cur_ptr)) {
+                                grad_log(L_ERR,
+                                         _("%s:%d: unknown host: %s"),
+                                         sqlfile, line_no,
+                                         cur_ptr);
                                 new_cfg.active[SQL_ACCT] = 0;
                                 new_cfg.active[SQL_AUTH] = 0;
                         } else {
-                                new_cfg.server = estrdup(cur_ptr);
+                                new_cfg.server = grad_estrdup(cur_ptr);
                         }
                         break;
                         
                 case STMT_PORT:
                         new_cfg.port = strtol(cur_ptr, &ptr, 0);
                         if (*ptr != 0 && !isspace(*ptr)) {
-                                radlog(L_ERR,
-				       "%s:%d: %s",
-                                       sqlfile, line_no,
-				       _("number parse error"));
+                                grad_log(L_ERR,
+				         "%s:%d: %s",
+                                         sqlfile, line_no,
+				         _("number parse error"));
                                 new_cfg.active[SQL_ACCT] = 0;
                                 new_cfg.active[SQL_AUTH] = 0;
                         }
                         break;
                         
                 case STMT_LOGIN:
-                        new_cfg.login = estrdup(cur_ptr);
+                        new_cfg.login = grad_estrdup(cur_ptr);
                         break;
                         
                 case STMT_PASSWORD:
-                        new_cfg.password = estrdup(cur_ptr);
+                        new_cfg.password = grad_estrdup(cur_ptr);
                         break;
                         
                 case STMT_KEEPOPEN:
                         if (get_boolean(cur_ptr, &new_cfg.keepopen))
-                                radlog(L_ERR,
-                                       "%s:%d: %s",
-                                       sqlfile, line_no,
-				       _("expected boolean value"));
+                                grad_log(L_ERR,
+                                         "%s:%d: %s",
+                                         sqlfile, line_no,
+				         _("expected boolean value"));
                         break;
 
                 case STMT_IDLE_TIMEOUT:
                         timeout = strtol(cur_ptr, &ptr, 0);
                         if ((*ptr != 0 && !isspace(*ptr)) || timeout <= 0) {
-                                radlog(L_ERR, "%s:%d: %s",
-                                       sqlfile, line_no,
-				       _("number parse error"));
+                                grad_log(L_ERR, "%s:%d: %s",
+                                         sqlfile, line_no,
+				         _("number parse error"));
                         } else 
                                 new_cfg.idle_timeout = timeout;
                         break;
                         
                 case STMT_DOAUTH:       
                         if (get_boolean(cur_ptr, &new_cfg.active[SQL_AUTH]))
-                                radlog(L_ERR,
-                                       "%s:%d: %s",
-                                       sqlfile, line_no,
-				       _("expected boolean value"));
+                                grad_log(L_ERR,
+                                         "%s:%d: %s",
+                                         sqlfile, line_no,
+				         _("expected boolean value"));
                         break;
                         
                 case STMT_DOACCT:
                         if (get_boolean(cur_ptr, &new_cfg.active[SQL_ACCT]))
-                                radlog(L_ERR,
-                                       "%s:%d: %s", 
-                                       sqlfile, line_no,
-				       _("expected boolean value"));
+                                grad_log(L_ERR,
+                                         "%s:%d: %s", 
+                                         sqlfile, line_no,
+				         _("expected boolean value"));
                         break;
                         
                 case STMT_AUTH_DB:
-                        new_cfg.auth_db = estrdup(cur_ptr);
+                        new_cfg.auth_db = grad_estrdup(cur_ptr);
                         break;
 
                 case STMT_ACCT_DB:
-                        new_cfg.acct_db = estrdup(cur_ptr);
-                        break;
-                        
-                case STMT_AUTH_QUERY:
-                        new_cfg.auth_query = estrdup(cur_ptr);
-                        break;
-
-                case STMT_GROUP_QUERY:
-                        new_cfg.group_query = estrdup(cur_ptr);
-                        break;
-                        
-                case STMT_ACCT_START_QUERY:
-                        new_cfg.acct_start_query = estrdup(cur_ptr);
-                        break;
-                        
-                case STMT_ACCT_STOP_QUERY:
-                        new_cfg.acct_stop_query = estrdup(cur_ptr);
-                        break;
-                        
-                case STMT_ACCT_KEEPALIVE_QUERY:
-                        new_cfg.acct_keepalive_query = estrdup(cur_ptr);
-                        break;
-                        
-                case STMT_ACCT_NASUP_QUERY:
-                        new_cfg.acct_nasup_query = estrdup(cur_ptr);
-                        break;
-
-                case STMT_ACCT_NASDOWN_QUERY:
-                        new_cfg.acct_nasdown_query = estrdup(cur_ptr);
-                        break;
-                        
-                case STMT_REPLY_ATTR_QUERY:
-                        new_cfg.reply_attr_query = estrdup(cur_ptr);
-                        break;
-
-                case STMT_CHECK_ATTR_QUERY:
-                        new_cfg.check_attr_query = estrdup(cur_ptr);
+                        new_cfg.acct_db = grad_estrdup(cur_ptr);
                         break;
                         
                 case STMT_MAX_AUTH_CONNECTIONS:
-                        radlog(L_WARN,
-                               "%s:%d: %s",
-                               sqlfile, line_no,
-			       _("auth_max_connections is obsolete"));
+                        grad_log(L_WARN,
+                                 "%s:%d: %s",
+                                 sqlfile, line_no,
+			         _("auth_max_connections is obsolete"));
                         break;
                         
                 case STMT_MAX_ACCT_CONNECTIONS:
-                        radlog(L_WARN,
-                               "%s:%d: %s",
-                               sqlfile, line_no,
-			       _("acct_max_connections is obsolete"));
+                        grad_log(L_WARN,
+                                 "%s:%d: %s",
+                                 sqlfile, line_no,
+			         _("acct_max_connections is obsolete"));
                         break;
                         
                 case STMT_QUERY_BUFFER_SIZE:
-                        radlog(L_WARN,
-			       "%s:%d: %s",
-			       sqlfile, line_no,
-			       _("query_buffer_size is obsolete"));
+                        grad_log(L_WARN,
+			         "%s:%d: %s",
+			         sqlfile, line_no,
+			         _("query_buffer_size is obsolete"));
                         break;
                         
                 case STMT_INTERFACE:
                         new_cfg.interface = disp_sql_interface_index(cur_ptr);
                         if (!new_cfg.interface) {
-                                radlog(L_WARN, "%s:%d: %s",
-                                       sqlfile, line_no,
-				       _("Unsupported SQL interface"));
+                                grad_log(L_WARN, "%s:%d: %s",
+                                         sqlfile, line_no,
+				         _("Unsupported SQL interface"));
                         }
                         break;
+
+		default:
+			if (STMT_QUERY_P(stmt_type)) {
+				new_cfg.query[STMT_QUERY_CODE(stmt_type)]
+					       = grad_estrdup(cur_ptr);
+			} 			
                 }
                 
         }
@@ -473,7 +458,7 @@ rad_sql_init()
         fclose(sqlfd);
 
         sql_check_config(sqlfile, &new_cfg);
-        efree(sqlfile);
+        grad_free(sqlfile);
 
         if (!sql_cfg_empty_p(&sql_cfg)
 	    && sql_cfg_comp(&new_cfg, &sql_cfg)) 
@@ -485,16 +470,9 @@ rad_sql_init()
         FREE(sql_cfg.password);
         FREE(sql_cfg.acct_db) ;
         FREE(sql_cfg.auth_db);
-        FREE(sql_cfg.group_query);
-        FREE(sql_cfg.auth_query);
-        FREE(sql_cfg.acct_start_query);
-        FREE(sql_cfg.acct_stop_query);
-        FREE(sql_cfg.acct_nasup_query);
-        FREE(sql_cfg.acct_nasdown_query);
-        FREE(sql_cfg.acct_keepalive_query);
-        FREE(sql_cfg.reply_attr_query);
-        FREE(sql_cfg.check_attr_query);
-        
+	for (i = 0; i < num_radius_sql_query; i++)
+		FREE(sql_cfg.query[i]);
+	
         /* copy new config */
         sql_cfg = new_cfg;
                 
@@ -518,8 +496,8 @@ radiusd_sql_shutdown()
 static void
 missing_statement(int prio, const char *filename, char *stmt)
 {
-	radlog(prio, _("%s: missing `%s' statement"),
-	       filename, stmt);
+	grad_log(prio, _("%s: missing `%s' statement"),
+	         filename, stmt);
 }
 
 void
@@ -529,7 +507,7 @@ sql_check_config(const char *filename, SQL_cfg *cfg)
 	int doacct = cfg->active[SQL_ACCT];
 
 #define FREE_IF_EMPTY(s) if (s && strcmp(s, "none") == 0) {\
-                                efree(s);\
+                                grad_free(s);\
                                 s = NULL;\
                          }
 
@@ -558,15 +536,15 @@ sql_check_config(const char *filename, SQL_cfg *cfg)
 			doauth = 0;
 		}
 
-		FREE_IF_EMPTY(cfg->auth_query);
-		if (!cfg->auth_query) {
+		FREE_IF_EMPTY(cfg->query[auth_query]);
+		if (!cfg->query[auth_query]) {
 			missing_statement(L_ERR, filename, "auth_query");
 			doauth = 0;
 		}
-			
-                if (!cfg->group_query) 
+			 
+		if (!cfg->query[group_query]) 
 			missing_statement(L_WARN, filename, "group_query");
-                FREE_IF_EMPTY(cfg->group_query);
+                FREE_IF_EMPTY(cfg->query[group_query]);
         }
 
 	/* Check SQL accounting setup */
@@ -575,40 +553,40 @@ sql_check_config(const char *filename, SQL_cfg *cfg)
 			missing_statement(L_ERR, filename, "acct_db");
 			doacct = 0;
 		}
-                if (!cfg->acct_start_query) 
+                if (!cfg->query[acct_start_query]) 
 			missing_statement(L_WARN, filename,
 					  "acct_start_query");
-                FREE_IF_EMPTY(cfg->acct_start_query);
+                FREE_IF_EMPTY(cfg->query[acct_start_query]);
 
-		if (!cfg->acct_stop_query) {
+		if (!cfg->query[acct_stop_query]) {
 			missing_statement(L_ERR, filename, "acct_stop_query");
 			doacct = 0;
 		}
-		FREE_IF_EMPTY(cfg->acct_stop_query);
+		FREE_IF_EMPTY(cfg->query[acct_stop_query]);
 
-                if (!cfg->acct_nasdown_query) 
+                if (!cfg->query[acct_nasdown_query]) 
 			missing_statement(L_WARN, filename,
 					  "acct_nasdown_query");
-                FREE_IF_EMPTY(cfg->acct_nasdown_query);
+                FREE_IF_EMPTY(cfg->query[acct_nasdown_query]);
 
-                if (!cfg->acct_nasup_query)
+                if (!cfg->query[acct_nasup_query])
 			missing_statement(L_WARN, filename,
 					  "acct_nasup_query");
-                FREE_IF_EMPTY(cfg->acct_nasup_query);
+                FREE_IF_EMPTY(cfg->query[acct_nasup_query]);
         }
 
 	/* Inform the user */
 	if (cfg->active[SQL_AUTH] != doauth)
-		radlog(L_NOTICE, _("disabling SQL authentication"));
+		grad_log(L_NOTICE, _("disabling SQL authentication"));
 	if (cfg->active[SQL_ACCT] != doacct)
-		radlog(L_NOTICE, _("disabling SQL accounting"));
+		grad_log(L_NOTICE, _("disabling SQL accounting"));
 }
 
 void
 sql_flush()
 {
-        radlog(L_NOTICE,
-               _("SQL configuration changed: closing existing connections"));
+        grad_log(L_NOTICE,
+                 _("SQL configuration changed: closing existing connections"));
         radiusd_flush_queue();
 	radiusd_sql_shutdown();
 }
@@ -622,15 +600,15 @@ sql_result_destroy(SQL_RESULT *res)
 	
 	if (!res)
 		return;
-	efree(res->query);
+	grad_free(res->query);
 	for (i = 0; i < res->ntuples; i++) {
 		size_t j;
 		for (j = 0; j < res->nfields; j++) 
-			efree(res->tuple[i][j]);
-		efree(res->tuple[i]);
+			grad_free(res->tuple[i][j]);
+		grad_free(res->tuple[i]);
 	}
-	efree(res->tuple);
-	efree(res);
+	grad_free(res->tuple);
+	grad_free(res);
 }
 
 static size_t
@@ -679,8 +657,8 @@ sql_cache_retrieve(struct sql_connection *conn, char *query)
 	size_t i;
 	
 	debug(20,("query: %s",query));
-	res = emalloc(sizeof(*res));
-	res->query = estrdup(query);
+	res = grad_emalloc(sizeof(*res));
+	res->query = grad_estrdup(query);
 	res->nfields = res->ntuples = 0;
 	res->tuple = NULL;
         data = disp_sql_exec(conn, query);
@@ -697,17 +675,17 @@ sql_cache_retrieve(struct sql_connection *conn, char *query)
 		return res;
 	}
 
-	res->tuple = emalloc(sizeof(res->tuple[0]) * res->ntuples);
+	res->tuple = grad_emalloc(sizeof(res->tuple[0]) * res->ntuples);
         for (i = 0; i < res->ntuples
 		     && disp_sql_next_tuple(conn, data) == 0; i++) {
 		int j;
 
-		res->tuple[i] = emalloc(sizeof(res->tuple[0][0]) * res->nfields);
+		res->tuple[i] = grad_emalloc(sizeof(res->tuple[0][0]) * res->nfields);
 			
 		for (j = 0; j < res->nfields; j++) {
 			char *p = disp_sql_column(conn, data, j);
 			chop(p);
-			res->tuple[i][j] = estrdup(p);
+			res->tuple[i][j] = grad_estrdup(p);
 		}
 	}
 		
@@ -749,7 +727,7 @@ attach_sql_connection(int type)
         if (!conn) {
                 debug(1, ("allocating new %d sql connection", type));
 
-                conn = emalloc(sizeof(struct sql_connection));
+                conn = grad_emalloc(sizeof(struct sql_connection));
                 conn->connected = 0;
                 conn->last_used = now;
                 conn->type = type;
@@ -781,10 +759,7 @@ detach_sql_connection(int type)
         if (conn->destroy_on_close) {
                 debug(1, ("destructing sql connection %p",
                           conn));
-                if (conn->connected)
-                        disp_sql_disconnect(conn);
-                efree(conn);
-                sql_conn[type] = NULL;
+		sql_conn_destroy(&sql_conn[type]);
         }
 }
 
@@ -792,9 +767,10 @@ static void
 sql_conn_destroy(struct sql_connection **conn)
 {
 	if (*conn) {
-		disp_sql_disconnect(*conn);
+	        if ((*conn)->connected)
+		        disp_sql_disconnect(*conn);
 		sql_cache_destroy(*conn);
-		efree(*conn);
+		grad_free(*conn);
 		*conn = NULL;
 	}
 }
@@ -804,7 +780,7 @@ sql_conn_destroy(struct sql_connection **conn)
 
 /*ARGSUSED*/
 void
-rad_sql_cleanup(int type, void *req ARG_UNUSED)
+radiusd_sql_cleanup(int type, void *req ARG_UNUSED)
 {
         if (sql_cfg.active[type])
                 detach_sql_connection(type);
@@ -814,11 +790,11 @@ rad_sql_cleanup(int type, void *req ARG_UNUSED)
  * Perform normal accounting
  */ 
 void
-rad_sql_acct(RADIUS_REQ *radreq)
+radiusd_sql_acct(grad_request_t *radreq)
 {
         int rc, count;
         int status;
-        VALUE_PAIR *pair;
+        grad_avp_t *pair;
         char *query;
         struct sql_connection *conn;
         struct obstack stack;
@@ -828,10 +804,10 @@ rad_sql_acct(RADIUS_REQ *radreq)
         if (!sql_cfg.active[SQL_ACCT])
                 return;
 
-        if ((pair = avl_find(radreq->request, DA_ACCT_STATUS_TYPE)) == NULL) {
+        if (!(pair = grad_avl_find(radreq->request, DA_ACCT_STATUS_TYPE))) {
                 /* should never happen!! */
-                radlog_req(L_ERR, radreq,
-                           _("no Acct-Status-Type attribute in rad_sql_acct()"));
+                grad_log_req(L_ERR, radreq,
+                             _("no Acct-Status-Type attribute in rad_sql_acct()"));
                 return;
         }
         status = pair->avp_lvalue;
@@ -842,10 +818,10 @@ rad_sql_acct(RADIUS_REQ *radreq)
         switch (status) {
         case DV_ACCT_STATUS_TYPE_START:
 		query_name = "acct_start_query";
-                if (!sql_cfg.acct_start_query)
+                if (!sql_cfg.query[acct_start_query])
                         break;
                 query = radius_xlate(&stack,
-                                     sql_cfg.acct_start_query,
+                                     sql_cfg.query[acct_start_query],
                                      radreq, NULL);
                 rc = disp_sql_query(conn, query, NULL);
                 sqllog(rc, query);
@@ -854,10 +830,10 @@ rad_sql_acct(RADIUS_REQ *radreq)
                 
         case DV_ACCT_STATUS_TYPE_STOP:
 		query_name = "acct_stop_query";
-                if (!sql_cfg.acct_stop_query)
+                if (!sql_cfg.query[acct_stop_query])
                         break;
                 query = radius_xlate(&stack,
-                                     sql_cfg.acct_stop_query,
+                                     sql_cfg.query[acct_stop_query],
                                      radreq, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
@@ -867,10 +843,10 @@ rad_sql_acct(RADIUS_REQ *radreq)
 
         case DV_ACCT_STATUS_TYPE_ACCOUNTING_ON:
 		query_name = "acct_nasup_query";
-                if (!sql_cfg.acct_nasup_query)
+                if (!sql_cfg.query[acct_nasup_query])
                         break;
                 query = radius_xlate(&stack,
-                                     sql_cfg.acct_nasup_query,
+                                     sql_cfg.query[acct_nasup_query],
                                      radreq, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
@@ -880,10 +856,10 @@ rad_sql_acct(RADIUS_REQ *radreq)
 
         case DV_ACCT_STATUS_TYPE_ACCOUNTING_OFF:
 		query_name = "acct_nasdown_query";
-                if (!sql_cfg.acct_nasdown_query)
+                if (!sql_cfg.query[acct_nasdown_query])
                         break;
                 query = radius_xlate(&stack,
-                                     sql_cfg.acct_nasdown_query,
+                                     sql_cfg.query[acct_nasdown_query],
                                      radreq, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
@@ -893,26 +869,26 @@ rad_sql_acct(RADIUS_REQ *radreq)
 
         case DV_ACCT_STATUS_TYPE_ALIVE:
 		query_name = "acct_keepalive_query";
-                if (!sql_cfg.acct_keepalive_query)
+                if (!sql_cfg.query[acct_keepalive_query])
                         break;
                 query = radius_xlate(&stack,
-                                     sql_cfg.acct_keepalive_query,
+                                     sql_cfg.query[acct_keepalive_query],
                                      radreq, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
-                if (rc != 0) 
-			log_facility = L_INFO;
+                if (rc == 0 && count != 1) 
+			log_facility = L_WARN;
                 break;
                 
         }
 
 	if (log_facility) {
-		radlog_req(log_facility, radreq,
-			   ngettext("%s updated %d record",
-				    "%s updated %d records",
-				    count),
-			   query_name,
-			   count);
+		grad_log_req(log_facility, radreq,
+			     ngettext("%s updated %d record",
+				      "%s updated %d records",
+				      count),
+			     query_name,
+			     count);
 	}
 
         obstack_free(&stack, NULL);
@@ -920,7 +896,7 @@ rad_sql_acct(RADIUS_REQ *radreq)
 
 
 char *
-rad_sql_pass(RADIUS_REQ *req, char *authdata)
+radiusd_sql_pass(grad_request_t *req, char *authdata)
 {
         char *mysql_passwd;
         struct sql_connection *conn;
@@ -928,19 +904,20 @@ rad_sql_pass(RADIUS_REQ *req, char *authdata)
         struct obstack stack;
         
         if (sql_cfg.active[SQL_AUTH] == 0) {
-                radlog(L_ERR,
-                       _("SQL Auth specified in users file, but not in sqlserver file"));
+                grad_log(L_ERR,
+                         _("SQL Auth specified in users file, but not in sqlserver file"));
                 return NULL;
         }
         
         if (authdata) {
-                avl_add_pair(&req->request,
-                             avp_create_string(DA_AUTH_DATA, authdata));
+                grad_avl_add_pair(&req->request,
+				  grad_avp_create_string(DA_AUTH_DATA,
+							 authdata));
         }
         
         obstack_init(&stack);
-        query = radius_xlate(&stack, sql_cfg.auth_query, req, NULL);
-        avl_delete(&req->request, DA_AUTH_DATA);
+        query = radius_xlate(&stack, sql_cfg.query[auth_query], req, NULL);
+        grad_avl_delete(&req->request, DA_AUTH_DATA);
         
         conn = attach_sql_connection(SQL_AUTH);
         mysql_passwd = disp_sql_getpwd(conn, query);
@@ -954,9 +931,7 @@ rad_sql_pass(RADIUS_REQ *req, char *authdata)
 }
 
 int
-rad_sql_checkgroup(req, groupname)
-        RADIUS_REQ *req;
-        char *groupname;
+radiusd_sql_checkgroup(grad_request_t *req, char *groupname)
 {
         int   rc = -1;
         struct sql_connection *conn;
@@ -965,7 +940,7 @@ rad_sql_checkgroup(req, groupname)
 	SQL_RESULT *res;
 	size_t i;
 	
-        if (sql_cfg.active[SQL_AUTH] == 0 || sql_cfg.group_query == NULL) 
+        if (sql_cfg.active[SQL_AUTH] == 0 || !sql_cfg.query[group_query]) 
                 return -1;
 
         conn = attach_sql_connection(SQL_AUTH);
@@ -974,12 +949,14 @@ rad_sql_checkgroup(req, groupname)
 
         obstack_init(&stack);
 	
-        query = radius_xlate(&stack, sql_cfg.group_query, req, NULL);
+        query = radius_xlate(&stack, sql_cfg.query[group_query], req, NULL);
 	res = sql_cache_lookup(conn, query);
 	if (!res) {
 		res = sql_cache_retrieve(conn, query);
-		if (!res)
-			return rc;
+		if (!res) {
+			obstack_free(&stack, NULL);
+			return -1;
+		}
 	}
         obstack_free(&stack, NULL);
 
@@ -997,12 +974,12 @@ rad_sql_checkgroup(req, groupname)
 static int
 rad_sql_retrieve_pairs(struct sql_connection *conn,
 		       char *query,
-		       VALUE_PAIR **return_pairs,
+		       grad_avp_t **return_pairs,
 		       int op_too)
 {
 	SQL_RESULT *res;
 	size_t i;
-	LOCUS loc;
+	grad_locus_t loc;
 	
 	res = sql_cache_lookup(conn, query);
 	if (!res) {
@@ -1017,30 +994,30 @@ rad_sql_retrieve_pairs(struct sql_connection *conn,
 		return 0;
 	
         for (i = 0; i < res->ntuples; i++) {
-		int op;
-		VALUE_PAIR *pair;
+		enum grad_operator op;
+		grad_avp_t *pair;
 		
 		if (op_too) {
 			char *opstr = res->tuple[i][2];
-                        op = str_to_op(opstr);
+                        op = grad_str_to_op(opstr);
                         if (op == NUM_OPERATORS) {
-                                radlog(L_NOTICE,
-                                       _("SQL: invalid operator: %s"), opstr);
+                                grad_log(L_NOTICE,
+                                         _("SQL: invalid operator: %s"), opstr);
                                 continue;
                         }
 		} else
-			op = OPERATOR_EQUAL;
+			op = grad_operator_equal;
 
 		loc.file = __FILE__;
 		loc.line = __LINE__;
-                pair = install_pair(&loc,
-				    res->tuple[i][0],
-				    op,
-				    res->tuple[i][1]);
+                pair = grad_create_pair(&loc,
+					res->tuple[i][0],
+					op,
+					res->tuple[i][1]);
                 
                 if (pair) {
-                        avl_merge(return_pairs, &pair);
-                        avl_free(pair);
+                        grad_avl_merge(return_pairs, &pair);
+                        grad_avl_free(pair);
                 }
 	}		
 
@@ -1048,20 +1025,21 @@ rad_sql_retrieve_pairs(struct sql_connection *conn,
 }
 			
 int
-rad_sql_reply_attr_query(RADIUS_REQ *req, VALUE_PAIR **reply_pairs)
+radiusd_sql_reply_attr_query(grad_request_t *req, grad_avp_t **reply_pairs)
 {
         struct sql_connection *conn;
         char *query;
         int rc;
         struct obstack stack;
         
-        if (sql_cfg.active[SQL_AUTH] == 0 || !sql_cfg.reply_attr_query)
+        if (sql_cfg.active[SQL_AUTH] == 0 || !sql_cfg.query[reply_attr_query])
                 return 0;
         
         conn = attach_sql_connection(SQL_AUTH);
         obstack_init(&stack);
 
-        query = radius_xlate(&stack, sql_cfg.reply_attr_query, req, NULL);
+        query = radius_xlate(&stack, sql_cfg.query[reply_attr_query],
+			     req, NULL);
         rc = rad_sql_retrieve_pairs(conn, query, reply_pairs, 0);
 
         obstack_free(&stack, NULL);
@@ -1069,26 +1047,198 @@ rad_sql_reply_attr_query(RADIUS_REQ *req, VALUE_PAIR **reply_pairs)
 }
 
 int
-rad_sql_check_attr_query(RADIUS_REQ *req, VALUE_PAIR **return_pairs)
+radiusd_sql_check_attr_query(grad_request_t *req, grad_avp_t **return_pairs)
 {
         struct sql_connection *conn;
         char *query;
         int rc;
         struct obstack stack;
         
-        if (sql_cfg.active[SQL_AUTH] == 0 || !sql_cfg.check_attr_query)
+        if (sql_cfg.active[SQL_AUTH] == 0 || !sql_cfg.query[check_attr_query])
                 return 0;
         
         conn = attach_sql_connection(SQL_AUTH);
         obstack_init(&stack);
 	
-        query = radius_xlate(&stack, sql_cfg.check_attr_query, req, NULL);
+        query = radius_xlate(&stack, sql_cfg.query[check_attr_query],
+			     req, NULL);
         rc = rad_sql_retrieve_pairs(conn, query, return_pairs, 1);
         
         obstack_free(&stack, NULL);
         return rc == 0;
 }
 
+int
+sql_auth_avail_p(const char **msg)
+{
+	if (sql_cfg.active[SQL_AUTH] && sql_cfg.query[auth_query])
+		return 1;
+	*msg = _("SQL authentication is not enabled in raddb/sqlserver");
+	return 0;
+}
+
+
+/* ****************************************************************************
+ * Auth failure trigger
+ */
+void
+radiusd_sql_auth_result_query(grad_request_t *req, int fail)
+{
+        struct sql_connection *conn;
+        char *query;
+        int rc;
+        struct obstack stack;
+	enum radius_sql_query q = fail ? auth_failure_query : auth_success_query;
+	
+	if (!sql_cfg.active[SQL_AUTH] || !sql_cfg.query[q])
+		return;
+
+        conn = attach_sql_connection(SQL_AUTH);
+        obstack_init(&stack);
+	
+        query = radius_xlate(&stack, sql_cfg.query[q],
+			     req, NULL);
+	rc = disp_sql_query(conn, query, NULL);
+	sqllog(rc, query);
+        obstack_free(&stack, NULL);
+}
+
+
+/* ****************************************************************************
+ * Multiple login checking
+ */
+
+static int
+rad_sql_retrieve_sessions(struct sql_connection *conn,
+			  char *query,
+			  grad_list_t **sess_list)
+{
+	SQL_RESULT *res;
+	size_t i;
+	grad_locus_t loc;
+	struct radutmp utmp;
+	
+	res = sql_cache_lookup(conn, query);
+	if (!res) {
+		res = sql_cache_retrieve(conn, query);
+		if (!res)
+			return 0;
+	}
+
+	if (res->ntuples == 0 || res->nfields != 4)
+		return 0;
+	
+        for (i = 0; i < res->ntuples; i++) {
+		struct radutmp *up = grad_emalloc(sizeof(*up));
+		GRAD_STRING_COPY(up->login, res->tuple[i][0]);
+		up->nas_address = htonl(grad_ip_strtoip(res->tuple[i][1]));
+		up->nas_port = strtoul(res->tuple[i][2], NULL, 0);
+		GRAD_STRING_COPY(up->session_id, res->tuple[i][3]);
+		if (*sess_list == NULL) 
+			*sess_list = grad_list_create();
+		grad_list_append(*sess_list, up);
+	}		
+
+	return i;
+}
+
+int
+sql_mlc_collect(const char *query_template,
+		grad_request_t *request,
+		grad_list_t **sess_list)
+{
+        struct sql_connection *conn;
+        char *query;
+        int rc;
+        struct obstack stack;
+        
+        conn = attach_sql_connection(SQL_ACCT);
+        obstack_init(&stack);
+	
+        query = radius_xlate(&stack, query_template, request, NULL);
+        rc = rad_sql_retrieve_sessions(conn, query, sess_list);
+	obstack_free(&stack, NULL);
+        return rc == 0;
+}
+
+int
+rad_sql_mlc_enabled_p()
+{
+	return sql_cfg.active[SQL_ACCT]
+		&& sql_cfg.query[mlc_user_query];
+}
+
+int
+rad_sql_mlc_collect_user(char *name, grad_request_t *request,
+			 grad_list_t **sess_list)
+{
+        if (sql_cfg.active[SQL_ACCT] == 0 || !sql_cfg.query[mlc_user_query])
+                return 1;
+	return sql_mlc_collect(sql_cfg.query[mlc_user_query],
+			       request, sess_list);
+}
+
+int
+rad_sql_mlc_collect_realm(grad_request_t *request, grad_list_t **sess_list)
+{
+        if (sql_cfg.active[SQL_ACCT] == 0 || !sql_cfg.query[mlc_realm_query])
+                return 1;
+	return sql_mlc_collect(sql_cfg.query[mlc_realm_query],
+			       request, sess_list);
+}
+
+void
+rad_sql_mlc_close(struct radutmp *up)
+{
+	int query_index = sql_cfg.query[mlc_stop_query] ?
+		                  mlc_stop_query : acct_stop_query;
+	char *query;
+	struct obstack stack;
+	grad_request_t *req;
+	int rc;
+	size_t count;
+	struct sql_connection *conn;
+	
+	if (sql_cfg.active[SQL_ACCT] == 0 || !sql_cfg.query[query_index])
+		return;
+	conn = attach_sql_connection(SQL_ACCT);
+	obstack_init(&stack);
+	/* Create a temporary request */
+	req = grad_request_alloc();
+	grad_avl_add_pair(&req->request,
+			  grad_avp_create_string(DA_USER_NAME, up->login));
+	grad_avl_add_pair(&req->request,
+			  grad_avp_create_integer(DA_NAS_PORT_ID,
+						  up->nas_port));
+	grad_avl_add_pair(&req->request,
+			  grad_avp_create_string(DA_ACCT_SESSION_ID,
+						 up->session_id));
+	query = radius_xlate(&stack, sql_cfg.query[query_index], req, NULL);
+	grad_request_free(req);
+	rc = disp_sql_query(conn, query, &count);
+	sqllog(rc, query);
+	if (rc == 0 && count != 1) {
+		grad_log(L_WARN,
+			 ngettext("%s updated %d record",
+				  "%s updated %d records",
+				  count),
+			 sql_query_name(query_index),
+			 count);
+	}
+	obstack_free(&stack, NULL);
+}
+
+
+void
+sql_init()
+{
+	mlc_register_method("sql",
+			    rad_sql_mlc_collect_user,
+			    rad_sql_mlc_collect_realm,
+			    rad_sql_mlc_close,
+			    rad_sql_mlc_enabled_p);
+}
+
 
 /* ****************************************************************************
  * Guile interface
@@ -1145,4 +1295,13 @@ sql_exec_query(int type, char *query)
 }
 #endif
 
+#else
+
+int
+sql_auth_avail_p(const char **msg)
+{
+	*msg = _("Radiusd is compiled without SQL support");
+	return 0;
+}
+
 #endif
