@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2003 Free Software Foundation
+   Copyright (C) 2003,2004 Free Software Foundation
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,19 +34,15 @@
 #endif
 
 #include <radiusd.h>
-#include <radargp.h>
-#include <radutmp.h>
+#include <radius/radargp.h>
+#include <radius/radutmp.h>
 #include <rewrite.h>
-#include <argcv.h>
 #include <snmp/asn1.h>
 #include <snmp/snmp.h>
-#ifdef USE_SQL
-# include <radsql.h>
-#endif
 #include <timestr.h>
 
 static int interactive;
-static RADIUS_REQ test_req;
+static grad_request_t test_req;
 static char *tsh_ps1 = "(radiusd) ";
 static char *tsh_ps2 = "[radiusd] ";
 
@@ -158,7 +154,7 @@ tsh_help(int argc, char **argv, char *cmd ARG_UNUSED)
 static void
 tsh_query_nas(int argc, char **argv, char *cmd ARG_UNUSED)
 {
-	NAS *nas;
+	grad_nas_t *nas;
 	struct radutmp ut;
 
 	if (argc < 5 || argc > 6) {
@@ -166,15 +162,15 @@ tsh_query_nas(int argc, char **argv, char *cmd ARG_UNUSED)
 			_("%s: wrong number of arguments\n"), argv[0]);
 		return;
 	}
-	nas = nas_lookup_name(argv[1]);
+	nas = grad_nas_lookup_name(argv[1]);
 	if (nas) {
-		if (nas->netdef.netmask != 32) {
-			fprintf(stderr, _("%s: is a network name\n"), argv[0]);
+		if (nas->netdef.netmask != 0xffffffffL) {
+			fprintf(stderr, _("%s is a network name\n"), argv[1]);
 			return;
 		}
 		ut.nas_address = nas->netdef.ipaddr;
 	} else {
-		ut.nas_address = ip_gethostaddr(argv[0]);
+		ut.nas_address = grad_ip_gethostaddr(argv[0]);
 		if (!ut.nas_address) {
 			fprintf(stderr, _("%s: unknown nas\n"), argv[0]);
 			return;
@@ -185,7 +181,7 @@ tsh_query_nas(int argc, char **argv, char *cmd ARG_UNUSED)
 	strncpy(ut.session_id, argv[3], sizeof(ut.session_id));
 	ut.nas_port = atoi(argv[4]);
 	if (argc == 6)
-		ut.framed_address = ip_strtoip(argv[5]);
+		ut.framed_address = grad_ip_strtoip(argv[5]);
 	printf("%d\n", checkrad(nas, &ut));
 }
 
@@ -340,7 +336,7 @@ static void
 tsh_req_define(int argc, char **argv, char *cmd)
 {
 	char *errp;
-	VALUE_PAIR *vp = NULL;
+	grad_avp_t *vp = NULL;
 	
 	if (argc > 1) {
 		while (*cmd && isspace(*cmd))
@@ -349,7 +345,7 @@ tsh_req_define(int argc, char **argv, char *cmd)
 			cmd++;
 
 		if (userparse(cmd, &vp, &errp)) {
-                        radlog(L_ERR, "%s", errp);
+                        grad_log(L_ERR, "%s", errp);
 			return;
 		}
 	} else {
@@ -358,22 +354,22 @@ tsh_req_define(int argc, char **argv, char *cmd)
 		while ((cmd = tsh_readline(tsh_ps2)) != NULL
 		       && cmd[0]) {
 			if (userparse(cmd, &vp, &errp)) {
-				radlog(L_ERR, "%s", errp);
+				grad_log(L_ERR, "%s", errp);
 				free(cmd);
-				avl_free(vp);
+				grad_avl_free(vp);
 				return;
 			}
 			free(cmd);
 		}
 	}
-	avl_free(test_req.request);
+	grad_avl_free(test_req.request);
 	test_req.request = vp;
 }
 
 static void
 tsh_req_print(int argc, char **argv, char *cmd)
 {
-	avl_fprint(stdout, "    ", 1, test_req.request);
+	grad_avl_fprint(stdout, "    ", 1, test_req.request);
 }
 
 
@@ -583,7 +579,11 @@ tsh_find_function(char *name)
 static void
 tsh_run_function(int argc, char **argv, char *cmd)
 {
-	tsh_command fp = tsh_find_function(argv[0]);
+	tsh_command fp;
+
+	if (argc == 0)
+		return;
+	fp = tsh_find_function(argv[0]);
 	if (fp)
 		fp(argc, argv, cmd);
 	else
