@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -35,10 +35,9 @@
 #include <signal.h>
 #include <netinet/in.h>
 
-#include <radius.h>
-#include <radpaths.h>
-#include <radutmp.h>
-#include <radargp.h>
+#include <common.h>
+#include <radius/radutmp.h>
+#include <radius/radargp.h>
 
 #define IP_ADDR_LEN 15
 
@@ -65,6 +64,7 @@ WTMP *find_logout(struct radutmp *bp);
 WTMP *find_restart(struct radutmp *bp);
 void print_entry(WTMP *pp, struct radutmp *bp, int mark);
 void print_reboot_entry(struct radutmp *bp);
+void print_acct_toggle(struct radutmp *bp);
 void delete_logout(WTMP *pp, struct radutmp *utp);
 WTMP *add_wtmp_entry(WTMP **first, WTMP *pp);
 WTMP *delete_wtmp_entry(WTMP **first, WTMP *pp);
@@ -72,8 +72,8 @@ WTMP *find_wtmp_nas(WTMP *first, struct radutmp *bp);
 WTMP *find_wtmp_nas_port(WTMP *first, struct radutmp *bp);
 WTMP *find_wtmp_nas_port_sid(WTMP *first, struct radutmp *bp);
 
-UINT4 host_ip = 0;
-NETDEF nas_ip = { 0, 0 } ;
+grad_uint32_t host_ip = 0;
+grad_netdef_t nas_ip = { 0, 0 } ;
 int port = 0;
 int width = 5;
 int show_seconds = 0;
@@ -148,12 +148,17 @@ parse_opt (int key, char *arg, struct argp_state *state)
         case '5': case '6': case '7': case '8': case '9':
                 if (maxrec == -1) {
                         char *p;
-                        
-                        maxrec = strtoul(state->argv[state->next-1]+1, &p, 0);
+			int i;
+			
+			if (state->argv[state->next-1][1] == key)
+				i = state->next-1;
+			else
+				i = state->next;
+                        maxrec = strtoul(state->argv[i]+1, &p, 0);
                         if (!maxrec) {
-                                radlog(L_ERR,
-                                       "invalid number (near %s)",
-                                       p);
+                                grad_log(L_ERR,
+                                         "invalid number (near %s)",
+                                         p);
                                 exit(1);
                         }
                 }
@@ -161,7 +166,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
         case 'c':
                 maxrec = atol(arg);
                 if (!maxrec) {
-                        radlog(L_ERR, "invalid number of records");
+                        grad_log(L_ERR, "invalid number of records");
                         exit(1);
                 }
                 break;
@@ -169,7 +174,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
                 file = arg;
                 break;
         case 'h':
-                host_ip = htonl(ip_gethostaddr(arg));
+                host_ip = htonl(grad_ip_gethostaddr(arg));
                 break;
         case 'm':
                 mark_missing_stops++;
@@ -199,8 +204,8 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
         case ARGP_KEY_FINI:
                 if (show_seconds && width == 8) {
-                        radlog(L_ERR,
-                             _("--width is incompatible with --show-seconds"));
+                        grad_log(L_ERR,
+                                 _("--width is incompatible with --show-seconds"));
                         exit (1);
                 }
                 break;
@@ -225,9 +230,8 @@ main(int argc, char **argv)
 {
         int index;
 
-        app_setup();
-        initlog(argv[0]);
-        if (rad_argp_parse(&argp, &argc, &argv, 0, &index, NULL))
+        grad_app_setup();
+        if (grad_argp_parse(&argp, &argc, &argv, 0, &index, NULL))
                 return 1;
 
         argv += index;
@@ -239,14 +243,14 @@ main(int argc, char **argv)
                         adduser(*argv);
         }
 
-        dict_init();
+        grad_dict_init();
         read_naslist();
 
         if (nas_name) {
-		NAS *nas = nas_lookup_name(nas_name);
+		grad_nas_t *nas = grad_nas_lookup_name(nas_name);
 		if (!nas) {
-			if (ip_getnetaddr(nas_name, &nas_ip)) {
-                                radlog(L_ERR, "unknown nas: %s", nas_name);
+			if (grad_ip_getnetaddr(nas_name, &nas_ip)) {
+                                grad_log(L_ERR, "unknown nas: %s", nas_name);
                                 return 1;
                         }
                 }       
@@ -259,9 +263,9 @@ int
 read_naslist()
 {
         int rc;
-        char *path = mkfilename(radius_dir, RADIUS_NASLIST);
-        rc = nas_read_file(path);
-        efree(path);
+        char *path = grad_mkfilename(radius_dir, RADIUS_NASLIST);
+        rc = grad_nas_read_file(path);
+        grad_free(path);
         return rc;
 }
 
@@ -273,11 +277,11 @@ rawread()
         struct radutmp ut;
         struct tm *tm;
         char ct[256];
-        UINT4 ipaddr;
+        grad_uint32_t ipaddr;
         char ip_str[DOTTED_QUAD_LEN];
         
         if ((wfd = open(file, O_RDONLY, 0)) < 0) {
-                radlog(L_ERR|L_PERROR, "can't open %s", file);
+                grad_log(L_ERR|L_PERROR, "can't open %s", file);
                 exit(1);
         }
         while (read(wfd, &ut, sizeof ut) == sizeof ut) {
@@ -285,7 +289,7 @@ rawread()
                 strftime(ct, sizeof(ct), "%c", tm);
 
                 ipaddr = ut.framed_address;
-                ip_iptostr(ntohl(ipaddr), ip_str);
+                grad_ip_iptostr(ntohl(ipaddr), ip_str);
                 
                 printf("%d %-*.*s %-*.*s %3.3d %-4.4s %2s %-*.*s %-*.*s %-*.*s %10.10s %5.5s\n",
                        ut.type,
@@ -294,7 +298,7 @@ rawread()
                        ut.login,
                        
                        nas_name_len, nas_name_len,
-                       nas_ip_to_name(ntohl(ut.nas_address)),
+                       grad_nas_ip_to_name(ntohl(ut.nas_address)),
 
                        ut.nas_port,
 
@@ -337,14 +341,15 @@ radwtmp()
         struct tm *tm;
         char ct[256];
         WTMP *pp;
-
+	int acct_enabled = 0;
+		
         if ((wfd = open(file, O_RDONLY, 0)) < 0 || fstat(wfd, &stb) == -1) {
-                radlog(L_ERR|L_PERROR, "can't open %s", file);
+                grad_log(L_ERR|L_PERROR, "can't open %s", file);
                 exit(1);
         }
         bl = (stb.st_size + sizeof(buf) - 1) / sizeof(buf);
         
-        rad_set_signal(SIGINT, sig_int);
+        grad_set_signal(SIGINT, sig_int);
         stop = 0;
 
         /*time(&buf[0].ut_time);*/
@@ -352,8 +357,9 @@ radwtmp()
         while (!stop && --bl >= 0) {
                 if (lseek(wfd, (off_t)(bl * sizeof(buf)), L_SET) == -1 ||
                     (bytes = read(wfd, buf, sizeof(buf))) == -1)
-                        radlog(L_ERR, "%s", file);
-                for (bp = &buf[bytes / sizeof(buf[0]) - 1]; bp >= buf; --bp) {
+                        grad_log(L_ERR, "%s", file);
+                for (bp = &buf[bytes / sizeof(buf[0]) - 1]; !stop && bp >= buf;
+		     --bp) {
                         switch (bp->type) {
                         case P_LOGIN:
                                 if (pp = find_logout(bp)) {
@@ -407,6 +413,17 @@ radwtmp()
                                                 return;
                                 }
                                 break;
+			case P_ACCT_DISABLED:
+				if (!acct_enabled) {
+					printf(_("System accounting is disabled\n"));
+					stop = 1;
+				} else
+					print_acct_toggle(bp);
+				break;
+			case P_ACCT_ENABLED:
+				acct_enabled = 1;
+				print_acct_toggle(bp);
+				break;
                         default:
                                 break;
                         }
@@ -426,11 +443,11 @@ want(struct radutmp *ut)
          */  
         if (ut->type == P_NAS_START) {
                 if (show_reboot_rec) 
-                        return ip_addr_in_net_p(&nas_ip,
+                        return grad_ip_in_net_p(&nas_ip,
 						ntohl(ut->nas_address));
         } else if (ut->type == P_NAS_SHUTDOWN) {
                 if (show_shutdown_rec)
-			return ip_addr_in_net_p(&nas_ip,
+			return grad_ip_in_net_p(&nas_ip,
 						ntohl(ut->nas_address));
         } else {
                 /* Process ususal login/logout entry */
@@ -443,7 +460,7 @@ want(struct radutmp *ut)
                                         if (host_ip != 0)
                                                 return ut->framed_address == host_ip ;
                                         if (nas_ip.ipaddr != 0)
-                                                return ip_addr_in_net_p(
+                                                return grad_ip_in_net_p(
 						       &nas_ip,
 						       ntohl(ut->nas_address));
 
@@ -460,7 +477,7 @@ want(struct radutmp *ut)
                         return 1;
 
                 if (nas_ip.ipaddr != 0 &&
-		    ip_addr_in_net_p(&nas_ip,  ntohl(ut->nas_address)))
+		    grad_ip_in_net_p(&nas_ip,  ntohl(ut->nas_address)))
                         return 1;
         
                 if (port != 0 && ut->nas_port == port)
@@ -484,13 +501,13 @@ adduser(char *s)
                         show_shutdown_rec = 1;
         }
 
-        uc = emalloc(sizeof(*uc));
+        uc = grad_emalloc(sizeof(*uc));
         uc->next = NULL;
         if (user_last) 
                 user_last->next = uc;
         else
                 user_chain = uc;
-        uc->name = estrdup(s);
+        uc->name = grad_estrdup(s);
         user_last = uc;
 }
 
@@ -573,13 +590,13 @@ add_logout(struct radutmp *bp)
 {
         WTMP *wp;
 
-        wp = emalloc(sizeof(*wp));
+        wp = grad_emalloc(sizeof(*wp));
         wp->ut = *bp;
         add_wtmp_entry(&logout_list, wp);
         /* purge deleted queue */
         if (wp = find_wtmp_nas_port(login_list, bp)) {
                 delete_wtmp_entry(&login_list, wp);
-                efree(wp);
+                grad_free(wp);
         }
 }
 
@@ -590,10 +607,10 @@ add_nas_restart(struct radutmp *bp)
 
         if (wp = find_wtmp_nas(nas_updown_list, bp)) {
                 delete_wtmp_entry(&nas_updown_list, wp);
-                efree(wp);
+                grad_free(wp);
         }
                     
-        wp = emalloc(sizeof(*wp));
+        wp = grad_emalloc(sizeof(*wp));
         wp->ut = *bp;
         add_wtmp_entry(&nas_updown_list, wp);
 }
@@ -638,7 +655,7 @@ delete_logout(WTMP *pp, struct radutmp *utp)
 char *
 proto_str(int id)
 {
-        DICT_VALUE *dval = value_lookup(id, "Framed-Protocol");
+        grad_dict_value_t *dval = grad_value_lookup(id, "Framed-Protocol");
         static char buf[64];
         
         if (!dval) {
@@ -651,7 +668,7 @@ proto_str(int id)
 char *
 port_type_str(int porttype)
 {
-	DICT_VALUE *dval = value_lookup(porttype, "NAS-Port-Type");
+	grad_dict_value_t *dval = grad_value_lookup(porttype, "NAS-Port-Type");
 	static char buf[80];
 	char *s;
 	
@@ -677,7 +694,7 @@ print_entry(WTMP *pp, struct radutmp *bp, int mark)
         char ct[256];
         char ip_str[IP_ADDR_LEN+1];
         time_t delta;
-        UINT4 ipaddr;
+        grad_uint32_t ipaddr;
         char buf[MAX_LONGNAME];
         
         tm = localtime(&bp->time);
@@ -686,7 +703,7 @@ print_entry(WTMP *pp, struct radutmp *bp, int mark)
         ipaddr = bp->framed_address;
         if (ipaddr == 0 && pp)
                 ipaddr = pp->ut.framed_address;
-        ip_iptostr(ntohl(ipaddr), ip_str);
+        grad_ip_iptostr(ntohl(ipaddr), ip_str);
 
         if (long_fmt) {                                   
                 printf("%-*.*s %-*.*s %3.3d %-4.4s %2s %-*.*s %-*.*s %-*.*s %16.16s ",
@@ -694,7 +711,8 @@ print_entry(WTMP *pp, struct radutmp *bp, int mark)
                        bp->login,
                        
                        nas_name_len, nas_name_len,
-                       nas_ip_to_name(ntohl(bp->nas_address), buf, sizeof buf),
+                       grad_nas_ip_to_name(ntohl(bp->nas_address),
+					   buf, sizeof buf),
 
                        bp->nas_port,
 
@@ -718,7 +736,8 @@ print_entry(WTMP *pp, struct radutmp *bp, int mark)
                        bp->login,
                        
                        nas_name_len, nas_name_len,
-                       nas_ip_to_name(ntohl(bp->nas_address), buf, sizeof buf),
+                       grad_nas_ip_to_name(ntohl(bp->nas_address),
+					   buf, sizeof buf),
 
                        bp->nas_port,
 
@@ -768,14 +787,37 @@ print_reboot_entry(struct radutmp *bp)
         strftime(ct, sizeof(ct), "%a %b %d %H:%M", tm);
 
         if (bp->type == P_NAS_SHUTDOWN)
-                s = "shutdown";
+                s = _("shutdown");
         else
-                s = "reboot";
+                s = _("reboot");
         printf("%-*.*s %s      ~                   %16.16s\n",
                namesize, namesize,
                s,
                        
-               nas_ip_to_name(ntohl(bp->nas_address), buf, sizeof buf),
+               grad_nas_ip_to_name(ntohl(bp->nas_address), buf, sizeof buf),
+               ct);
+}
+
+void
+print_acct_toggle(struct radutmp *bp)
+{
+        char *s;
+        struct tm *tm;
+        char ct[256];
+        char buf[MAX_LONGNAME];
+        
+        tm = localtime(&bp->time);
+        strftime(ct, sizeof(ct), "%a %b %d %H:%M", tm);
+
+        if (bp->type == P_ACCT_DISABLED)
+                s = _("acct_off");
+        else
+                s = _("acct_on");
+        printf("%-*.*s %s      ~                   %16.16s\n",
+               namesize, namesize,
+               s,
+                       
+               grad_nas_ip_to_name(ntohl(bp->nas_address), buf, sizeof buf),
                ct);
 }
 
