@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -22,8 +22,8 @@
 #endif
 
 #include <libguile.h>
-#include <radius.h>
-#include <radscm.h>
+#include <radius/radius.h>
+#include <radius/radscm.h>
 
 static struct keyword radlog_kw[] = {
 #define D(c) { #c, c }
@@ -57,9 +57,10 @@ parse_facility(SCM list)
                 if (SCM_IMP(car) && SCM_INUMP(car)) 
                         val = SCM_INUM(car);
                 else if (SCM_NIMP(car) && SCM_STRINGP(car))
-                        val = xlat_keyword(radlog_kw, SCM_STRING_CHARS(car), 0);
+                        val = grad_xlat_keyword(radlog_kw,
+						SCM_STRING_CHARS(car), 0);
                 else if (SCM_BIGP(car)) 
-		  val = (UINT4) scm_i_big2dbl(car);
+		  val = (grad_uint32_t) scm_i_big2dbl(car);
                 else
                         continue;
                 accval |= val;
@@ -77,7 +78,7 @@ SCM_DEFINE(rad_log_open, "rad-log-open", 1, 0, 0,
         if (SCM_IMP(PRIO) && SCM_INUMP(PRIO)) {
                 prio = SCM_INUM(PRIO);
         } else if (SCM_BIGP(PRIO)) {
-                prio = (UINT4) scm_i_big2dbl(PRIO);
+                prio = (grad_uint32_t) scm_i_big2dbl(PRIO);
         } else {
                 SCM_ASSERT(SCM_NIMP(PRIO) && SCM_CONSP(PRIO),
                            PRIO, SCM_ARG1, FUNC_NAME);
@@ -101,7 +102,7 @@ SCM_DEFINE(rad_log, "rad-log", 2, 0, 0,
         } else if (SCM_IMP(PRIO) && SCM_INUMP(PRIO)) {
                 prio = SCM_INUM(PRIO);
         } else if (SCM_BIGP(PRIO)) {
-                prio = (UINT4) scm_i_big2dbl(PRIO);
+                prio = (grad_uint32_t) scm_i_big2dbl(PRIO);
         } else {
                 SCM_ASSERT(SCM_NIMP(PRIO) && SCM_CONSP(PRIO),
                            PRIO, SCM_ARG1, FUNC_NAME);
@@ -110,7 +111,7 @@ SCM_DEFINE(rad_log, "rad-log", 2, 0, 0,
 
         SCM_ASSERT(SCM_NIMP(TEXT) && SCM_STRINGP(TEXT),
                    TEXT, SCM_ARG1, FUNC_NAME);
-        radlog(prio, "%s", SCM_STRING_CHARS(TEXT));
+        grad_log(prio, "%s", SCM_STRING_CHARS(TEXT));
         return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
