@@ -28,7 +28,7 @@
 #line 1 "gram.y"
 
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -46,8 +46,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_GRAM
-        
 #if defined(HAVE_CONFIG_H)        
 # include <config.h>
 #endif
@@ -67,12 +65,11 @@
 #include <signal.h>
 #include <errno.h>
 #include <sys/wait.h>
-#include <sysdep.h>
-#include <radius.h>
-#include <radtest.h>
-#include <debugmod.h>
 
-extern LOCUS source_locus;
+#include <common.h>
+#include <radtest.h>
+
+extern grad_locus_t source_locus;
 
 char *print_ident(Variable *var);
 int subscript(Variable *var, char *attr_name, int all, Variable *ret_var);
@@ -81,19 +78,20 @@ extern int yylex();
 int yyerror(char *s);
 
 
-#line 77 "gram.y"
+#line 75 "gram.y"
 typedef union {
         char *string;
         int number;
-        UINT4 ipaddr;
+        grad_uint32_t ipaddr;
         Variable *ident;
-        VALUE_PAIR *pair;
-        VALUE_PAIR *vector;
+        grad_avp_t *pair;
+        grad_avp_t *vector;
         struct {
-                VALUE_PAIR *head, *tail;
+                grad_avp_t *head, *tail;
         } pair_list;
         Variable variable;
-	Symtab *symtab;
+	grad_symtab_t *symtab;
+	enum grad_operator op;
 } YYSTYPE;
 #ifndef YYDEBUG
 #define YYDEBUG 1
@@ -177,9 +175,9 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    93,    94,    96,   100,   106,   107,   111,   124,   130,   158,
-   163,   169,   173,   179,   180,   191,   195,   198,   207,   217,
-   225,   226,   239,   243,   250,   256,   260,   272,   283,   290,
+    92,    93,    95,    99,   105,   106,   110,   123,   129,   157,
+   162,   168,   172,   178,   179,   190,   194,   197,   206,   216,
+   224,   225,   239,   243,   250,   256,   260,   272,   283,   290,
    297,   298,   299,   305,   309,   317,   321,   325,   329,   333,
    337,   343,   349,   355,   361,   365,   370,   375,   383,   384,
    387
@@ -817,34 +815,34 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 93 "gram.y"
+#line 92 "gram.y"
 { prompt(); ;
     break;}
 case 3:
-#line 97 "gram.y"
+#line 96 "gram.y"
 {
                         prompt();
                 ;
     break;}
 case 4:
-#line 101 "gram.y"
+#line 100 "gram.y"
 {
                         prompt();
                 ;
     break;}
 case 6:
-#line 108 "gram.y"
+#line 107 "gram.y"
 {
                         printf("\n");
                 ;
     break;}
 case 7:
-#line 112 "gram.y"
+#line 111 "gram.y"
 {
                         Variable *var;
                         
-                        if ((var = (Variable*) sym_lookup(vartab, yyvsp[-3].string)) == NULL)
-                                var = (Variable*) sym_install(vartab, yyvsp[-3].string);
+                        if ((var = (Variable*) grad_sym_lookup(vartab, yyvsp[-3].string)) == NULL)
+                                var = (Variable*) grad_sym_install(vartab, yyvsp[-3].string);
                         if (var->type == Builtin)
                                 var->datum.builtin.set(&yyvsp[-1].variable);
                         else {
@@ -854,15 +852,15 @@ case 7:
                 ;
     break;}
 case 8:
-#line 125 "gram.y"
+#line 124 "gram.y"
 {
                         radtest_send(yyvsp[-3].number, yyvsp[-2].number, yyvsp[-1].ident, yyvsp[-4].symtab);
                         tempvar_free(yyvsp[-1].ident);
-			symtab_free(&yyvsp[-4].symtab);
+			grad_symtab_free(&yyvsp[-4].symtab);
                 ;
     break;}
 case 9:
-#line 131 "gram.y"
+#line 130 "gram.y"
 {
                         int pass = 1;
                         if (verbose) {
@@ -886,38 +884,38 @@ case 9:
                                                   yyvsp[-1].ident->datum.vector))
                                         pass = 0;
                                 tempvar_free(yyvsp[-1].ident);
-                                efree(yyvsp[-1].ident);
+                                grad_free(yyvsp[-1].ident);
                         }
                         printf("%s\n", pass ? "PASS" : "FAIL");
                 ;
     break;}
 case 10:
-#line 159 "gram.y"
+#line 158 "gram.y"
 {
                         yyclearin;
                         yyerrok;
                 ;
     break;}
 case 11:
-#line 164 "gram.y"
+#line 163 "gram.y"
 {
                         YYACCEPT;
                 ;
     break;}
 case 12:
-#line 170 "gram.y"
+#line 169 "gram.y"
 {
                         yyval.number = PORT_AUTH;
                 ;
     break;}
 case 13:
-#line 174 "gram.y"
+#line 173 "gram.y"
 {
                         yyval.number = PORT_ACCT;
                 ;
     break;}
 case 15:
-#line 181 "gram.y"
+#line 180 "gram.y"
 {
                         if (yyvsp[0].ident->type != Integer) {
                                 yyerror("expected integer value");
@@ -928,34 +926,34 @@ case 15:
                 ;
     break;}
 case 16:
-#line 192 "gram.y"
+#line 191 "gram.y"
 {
 			yyval.symtab = NULL;
 		;
     break;}
 case 18:
-#line 199 "gram.y"
+#line 198 "gram.y"
 {
 			Variable *var;
 			
-			yyval.symtab = symtab_create(sizeof(Variable), var_free);
-			var = (Variable*) sym_install(yyval.symtab, yyvsp[0].variable.name);
+			yyval.symtab = grad_symtab_create(sizeof(Variable), var_free);
+			var = (Variable*) grad_sym_install(yyval.symtab, yyvsp[0].variable.name);
 			var->type = yyvsp[0].variable.type;
 			var->datum = yyvsp[0].variable.datum;
 		;
     break;}
 case 19:
-#line 208 "gram.y"
+#line 207 "gram.y"
 {
 			Variable *var;
-			var = (Variable*) sym_install(yyvsp[-1].symtab, yyvsp[0].variable.name);
+			var = (Variable*) grad_sym_install(yyvsp[-1].symtab, yyvsp[0].variable.name);
 			var->type = yyvsp[0].variable.type;
 			var->datum = yyvsp[0].variable.datum;
 			yyval.symtab = yyvsp[-1].symtab;
 		;
     break;}
 case 20:
-#line 218 "gram.y"
+#line 217 "gram.y"
 {
                         yyval.variable.name = yyvsp[-2].string;
                         yyval.variable.type = Integer;
@@ -963,14 +961,15 @@ case 20:
 		;
     break;}
 case 22:
-#line 227 "gram.y"
+#line 226 "gram.y"
 {
                         if (yyvsp[-2].variable.type != Vector) {
                                 parse_error("bad datatype of larg in +");
                         } else if (yyvsp[0].variable.type != Vector) {
                                 parse_error("bad datatype of rarg in +");
                         } else {
-                                avl_add_list(&yyvsp[-2].variable.datum.vector, yyvsp[0].variable.datum.vector);
+                                grad_avl_add_list(&yyvsp[-2].variable.datum.vector,
+						  yyvsp[0].variable.datum.vector);
                                 yyval.variable = yyvsp[-2].variable;
                         }
                 ;
@@ -984,7 +983,7 @@ case 23:
 case 24:
 #line 244 "gram.y"
 {
-			yyval.ident = emalloc(sizeof(*yyval.ident));
+			yyval.ident = grad_emalloc(sizeof(*yyval.ident));
                         *yyval.ident = yyvsp[0].variable;
                 ;
     break;}
@@ -1030,15 +1029,15 @@ case 28:
 case 29:
 #line 284 "gram.y"
 {
-                        avl_free(yyvsp[-1].pair_list.head);
+                        grad_avl_free(yyvsp[-1].pair_list.head);
                         yyval.pair_list.head = yyval.pair_list.tail = NULL;
                 ;
     break;}
 case 30:
 #line 291 "gram.y"
 {
-                        yyval.pair = install_pair(&source_locus, yyvsp[-2].string, yyvsp[-1].number, yyvsp[0].string);
-                        efree(yyvsp[0].string);
+                        yyval.pair = grad_create_pair(&source_locus, yyvsp[-2].string, yyvsp[-1].op, yyvsp[0].string);
+                        grad_free(yyvsp[0].string);
                 ;
     break;}
 case 33:
@@ -1046,7 +1045,7 @@ case 33:
 {
                         char buf[64];
                         sprintf(buf, "%d", yyvsp[0].number);
-                        yyval.string = estrdup(buf);
+                        yyval.string = grad_estrdup(buf);
                 ;
     break;}
 case 34:
@@ -1059,44 +1058,44 @@ case 35:
 #line 310 "gram.y"
 {
                         char buf[DOTTED_QUAD_LEN];
-                        ip_iptostr(yyvsp[0].ipaddr, buf);
-                        yyval.string = estrdup(buf);
+                        grad_ip_iptostr(yyvsp[0].ipaddr, buf);
+                        yyval.string = grad_estrdup(buf);
                 ;
     break;}
 case 36:
 #line 318 "gram.y"
 {
-                        yyval.number = OPERATOR_EQUAL;
+                        yyval.op = grad_operator_equal;
                 ;
     break;}
 case 37:
 #line 322 "gram.y"
 {
-                        yyval.number = OPERATOR_LESS_THAN;
+                        yyval.op = grad_operator_less_than;
                 ;
     break;}
 case 38:
 #line 326 "gram.y"
 { 
-                        yyval.number = OPERATOR_GREATER_THAN;
+                        yyval.op = grad_operator_greater_than;
                 ;
     break;}
 case 39:
 #line 330 "gram.y"
 {
-                        yyval.number = OPERATOR_NOT_EQUAL;
+                        yyval.op = grad_operator_not_equal;
                 ;
     break;}
 case 40:
 #line 334 "gram.y"
 {
-                        yyval.number = OPERATOR_LESS_EQUAL;
+                        yyval.op = grad_operator_less_equal;
                 ;
     break;}
 case 41:
 #line 338 "gram.y"
 {
-                        yyval.number = OPERATOR_GREATER_EQUAL;
+                        yyval.op = grad_operator_greater_equal;
                 ;
     break;}
 case 42:
@@ -1120,7 +1119,7 @@ case 44:
 {
                         yyval.variable.name = NULL;
                         yyval.variable.type = String;
-                        yyval.variable.datum.string = estrdup(yyvsp[0].string);
+                        yyval.variable.datum.string = grad_estrdup(yyvsp[0].string);
                 ;
     break;}
 case 45:
@@ -1415,20 +1414,20 @@ set_yydebug()
 int
 subscript(Variable *var, char *attr_name, int all, Variable *ret_var)
 {
-        DICT_ATTR *dict;
-        VALUE_PAIR *pair;
+        grad_dict_attr_t *dict;
+        grad_avp_t *pair;
 
         ret_var->type = Undefined;
         if (var->type != Vector) {
                 parse_error("subscript on non-vector");
                 return -1;
         }
-        if ((dict = attr_name_to_dict(attr_name)) == NULL) {
+        if ((dict = grad_attr_name_to_dict(attr_name)) == NULL) {
                 parse_error("unknown attribute %s", attr_name);
                 return -1;
         }
         
-        pair = avl_find(var->datum.vector, dict->value);
+        pair = grad_avl_find(var->datum.vector, dict->value);
         if (!pair) 
                 return -1;
 
@@ -1437,23 +1436,25 @@ subscript(Variable *var, char *attr_name, int all, Variable *ret_var)
                 ret_var->type = String;
                 if (all) {
                         int length = 0;
-                        VALUE_PAIR *p;
+                        grad_avp_t *p;
                         char *cp;
                         
                         /* First, count total length of all attribute
                            instances in the packet */
-                        for (p = pair; p; p = avl_find(p->next, dict->value)) 
+                        for (p = pair; p;
+			     p = grad_avl_find(p->next, dict->value)) 
                                 length += p->avp_strlength;
 
-                        cp = ret_var->datum.string = emalloc(length+1);
+                        cp = ret_var->datum.string = grad_emalloc(length+1);
                         /* Fill in the string contents */
-                        for (p = pair; p; p = avl_find(p->next, dict->value)) {
+                        for (p = pair; p;
+			     p = grad_avl_find(p->next, dict->value)) {
                                 memcpy(cp, p->avp_strvalue, p->avp_strlength);
                                 cp += p->avp_strlength;
                         }
                         *cp = 0;
                 } else
-                        ret_var->datum.string = estrdup(pair->avp_strvalue);
+                        ret_var->datum.string = grad_estrdup(pair->avp_strvalue);
                 break;
         case TYPE_INTEGER:
         case TYPE_DATE:
@@ -1465,9 +1466,9 @@ subscript(Variable *var, char *attr_name, int all, Variable *ret_var)
                 ret_var->datum.ipaddr = pair->avp_lvalue;
                 break;
         default:
-                radlog(L_CRIT,
-                       _("attribute %s has unknown type"),
-                       dict->name);
+                grad_log(L_CRIT,
+                         _("attribute %s has unknown type"),
+                         dict->name);
                 exit(1);
         }
         return 0;
