@@ -1,15 +1,16 @@
 typedef union {
         char *string;
         int number;
-        UINT4 ipaddr;
+        grad_uint32_t ipaddr;
         Variable *ident;
-        VALUE_PAIR *pair;
-        VALUE_PAIR *vector;
+        grad_avp_t *pair;
+        grad_avp_t *vector;
         struct {
-                VALUE_PAIR *head, *tail;
+                grad_avp_t *head, *tail;
         } pair_list;
         Variable variable;
-	Symtab *symtab;
+	grad_symtab_t *symtab;
+	enum grad_operator op;
 } YYSTYPE;
 #define	EOL	257
 #define	AUTH	258
