@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -31,19 +31,17 @@
 #include <sys/stat.h>
 #include <time.h>
 
-#include <radius.h>
-#include <radargp.h>
+#include <common.h>
+#include <radius/radargp.h>
 #include <radtest.h>
-#include <argcv.h>
+#include <radius/argcv.h>
 
-Symtab *vartab;
+grad_symtab_t *vartab;
 int verbose;
-extern int rad_clt_debug;
-RADIUS_SERVER_QUEUE *srv_queue;
-char *progname;
+extern int grad_client_debug;
+grad_server_queue_t *srv_queue;
 int reply_code;
-VALUE_PAIR *reply_list;
-int debug_flag = 0;
+grad_avp_t *reply_list;
 int abort_on_failure = 0;
 
 void init_symbols();
@@ -108,8 +106,6 @@ parse_opt(int key, char *arg, struct argp_state *state)
 		
         case 'f':
                 filename = optarg;
-                *(int *)state->input = state->next;
-                state->next = state->argc;
                 break;
 		
         case 't':
@@ -125,7 +121,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
 		set_module_debug_level("radpdu", 100);
 		set_module_debug_level("client", 100);
                 break;
-		
+
         default:
                 return ARGP_ERR_UNKNOWN;
         }
@@ -146,47 +142,50 @@ main(int argc, char **argv)
 {
         char *p;
         int index;
-        
-        app_setup();
-        initlog(argv[0]);
+	Variable *var;
+	
+        grad_app_setup();
         init_symbols();
 
         index = argc;
-        if (rad_argp_parse(&argp, &argc, &argv, 0, NULL, &index))
+        if (grad_argp_parse(&argp, &argc, &argv, 0, &index, NULL))
                 return 1;
 
         argv += index;
         argc -= index;
 
         set_yydebug();
-        radpath_init();
+        grad_path_init();
 	srand(time(NULL));
 	
-        if (dict_init()) {
-                radlog(L_ERR, _("error reading dictionary file"));
+        if (grad_dict_init()) {
+                grad_log(L_ERR, _("error reading dictionary file"));
                 return 1;
         }
-        srv_queue = rad_clt_create_queue(!quick, 0, 0);
+        srv_queue = grad_client_create_queue(!quick, 0, 0);
         
         if (!srv_queue)
                 return 1;
 
-        if (timeout)
+	var = (Variable*)grad_sym_lookup(vartab, "SOURCEIP");
+	var->datum.ipaddr = srv_queue->source_ip;
+		
+	if (timeout)
                 srv_queue->timeout = timeout;
         if (retry)
                 srv_queue->retries = retry;
         if (server) {
-                RADIUS_SERVER serv;
+                grad_server_t serv;
                 int i, argc;
                 char **argv;
 
                 if (argcv_get(server, ":", NULL, &argc, &argv)) {
-                        radlog(L_ERR, "can't parse server definition");
+                        grad_log(L_ERR, _("can't parse server definition"));
                         exit(1);
                 }
 
                 if (argc < 3) {
-                        radlog(L_ERR, "no shared secret for the server");
+                        grad_log(L_ERR, _("no shared secret for the server"));
                         exit(1);
                 }
 
@@ -195,11 +194,11 @@ main(int argc, char **argv)
                 for (i = 0; i < argc; i++) {
                         switch (i) {
                         case 0:
-                                serv.addr = ip_gethostaddr(argv[i]);
+                                serv.addr = grad_ip_gethostaddr(argv[i]);
                                 if (!serv.addr) {
-                                        radlog(L_ERR,
-                                             "bad IP address or host name: %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad IP address or host name: %s"),
+                                                 argv[i]);
                                         exit(1);
                                 }
                                 break;
@@ -209,26 +208,26 @@ main(int argc, char **argv)
                         case 4:
                                 serv.port[0] = strtol(argv[i], &p, 0);
                                 if (*p) {
-                                        radlog(L_ERR,
-                                               "bad port number %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad port number %s"),
+                                                 argv[i]);
                                         break;
                                 }
                                 break;
                         case 6:
                                 serv.port[1] = strtol(argv[i], &p, 0);
                                 if (*p) {
-                                        radlog(L_ERR,
-                                               "bad port number %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad port number %s"),
+                                                 argv[i]);
                                         break;
                                 }
                                 break;
                         default:
                                 if (argv[i][0] != ':') {
-                                        radlog(L_ERR,
-                                               "bad separator near %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad separator near %s"),
+                                                 argv[i]);
                                         exit(1);
                                 }
                                 break;
@@ -239,18 +238,19 @@ main(int argc, char **argv)
                         serv.port[0] = DEF_AUTH_PORT;
                 if (argc < 6)
                         serv.port[1] = DEF_ACCT_PORT;
-		rad_clt_append_server(srv_queue, rad_clt_alloc_server(&serv));
+		grad_client_append_server(srv_queue,
+					  grad_client_alloc_server(&serv));
                 argcv_free(argc, argv);
         }
 
-        if (list_count(srv_queue->servers) == 0) {
-                radlog(L_ERR,
-                       "No servers specfied. Use -s option.\n");
+        if (grad_list_count(srv_queue->servers) == 0) {
+                grad_log(L_ERR,
+                         _("No servers specfied. Use -s option.\n"));
                 exit(1);
         }
         
         x_argmax = argc + 2;
-        x_argv = emalloc(sizeof(x_argv[0]) * x_argmax);
+        x_argv = grad_emalloc(sizeof(x_argv[0]) * x_argmax);
         x_argc = 0;
         x_argv[x_argc++] = filename;
 
@@ -278,11 +278,13 @@ init_symbols()
 {
         Variable *var;
         
-        vartab = symtab_create(sizeof(Variable), NULL);
-        var = (Variable*) sym_install(vartab, "REPLY_CODE");
+        vartab = grad_symtab_create(sizeof(Variable), NULL);
+        var = (Variable*) grad_sym_install(vartab, "REPLY_CODE");
         var->type = Integer;
-        var = (Variable*) sym_install(vartab, "REPLY");
+        var = (Variable*) grad_sym_install(vartab, "REPLY");
         var->type = Vector;
+	var = (Variable*) grad_sym_install(vartab, "SOURCEIP");
+	var->type = Ipaddress;
 }
 
 void
@@ -303,7 +305,7 @@ assign(char *s)
         type = parse_datum(p, &datum);
         if (type == Undefined)
                 return;
-        var = (Variable*)sym_install(vartab, s);
+        var = (Variable*)grad_sym_install(vartab, s);
         var->type = type;
         var->datum = datum;
 }
@@ -323,7 +325,7 @@ parse_datum(char *p, union datum *dp)
                 p[length-1] = 0;
                 
                 type = String;
-                dp->string = estrdup(p);
+                dp->string = grad_estrdup(p);
         } else if (isdigit(*p)) {
                 char *endp;
                 
@@ -333,7 +335,7 @@ parse_datum(char *p, union datum *dp)
                         type = Integer;
                 } else {
                         /* IP address */
-                        if ((dp->ipaddr = ip_gethostaddr(p)) != 0)
+                        if ((dp->ipaddr = grad_ip_gethostaddr(p)) != 0)
                                 type = Ipaddress;
                         else {
                                 fprintf(stderr, _("assign: invalid IP address: %s\n"), p);
@@ -342,7 +344,7 @@ parse_datum(char *p, union datum *dp)
                 } 
         } else if (strchr(p, '.')) {
                 /* IP address */
-                if ((dp->ipaddr = ip_gethostaddr(p)) != 0)
+                if ((dp->ipaddr = grad_ip_gethostaddr(p)) != 0)
                         type = Ipaddress;
                 else {
                         fprintf(stderr, _("assign: invalid IP address: %s\n"), p);
@@ -350,7 +352,7 @@ parse_datum(char *p, union datum *dp)
                 }
         } else {
                 type = String;
-                dp->string = estrdup(p);
+                dp->string = grad_estrdup(p);
         }
         return type;
 }
@@ -361,48 +363,30 @@ print_ident(Variable *var)
         char buf[64];
         switch (var->type) {
         case Undefined:
-                return estrdup("UNDEFINED");
+                return grad_estrdup("UNDEFINED");
                 break;
         case Integer:
                 sprintf(buf, "%d", var->datum.number);
-                return estrdup(buf);
+                return grad_estrdup(buf);
         case Ipaddress:
-                ip_iptostr(var->datum.ipaddr, buf);
-                return estrdup(buf);
+                grad_ip_iptostr(var->datum.ipaddr, buf);
+                return grad_estrdup(buf);
         case String:
-                return estrdup(var->datum.string);
+                return grad_estrdup(var->datum.string);
                 break;
         case Vector:
-                return estrdup("VECTOR");
+                return grad_estrdup("VECTOR");
         }
 	return NULL;
 }
 
 void
-print_pairs(FILE *fp, VALUE_PAIR *pair)
+print_pairs(FILE *fp, grad_avp_t *pair)
 {
         for (; pair; pair = pair->next) {
-                fprintf(fp, " %s = ", pair->name);
-                switch (pair->type) {
-                case TYPE_STRING:
-                        fprintf(fp, "(STRING) %s", pair->avp_strvalue);
-                        break;
-
-                case TYPE_INTEGER:
-                        fprintf(fp, "(INTEGER) %ld", pair->avp_lvalue);
-                        break;
-
-                case TYPE_IPADDR:
-                        fprintf(fp, "(IP) %lx", pair->avp_lvalue);
-                        break;
-                
-                case TYPE_DATE:
-                        fprintf(fp, "(DATE) %ld", pair->avp_lvalue);
-                        break;
-                        
-                default:
-                        fprintf(fp, "(%d)", pair->type);
-                }
+		char *save;
+		fprintf(fp, " %s", grad_format_pair(pair, 1, &save));
+		free(save);
                 if (pair->next)
                         fprintf(fp, ",");
                 else
@@ -425,7 +409,7 @@ var_print(Variable *var)
                 printf("%d", var->datum.number);
                 break;
         case Ipaddress:
-                printf("%s", ip_iptostr(var->datum.ipaddr, buf));
+                printf("%s", grad_ip_iptostr(var->datum.ipaddr, buf));
                 break;
         case String:
                 printf("%s", var->datum.string);
@@ -446,10 +430,10 @@ var_free(Variable *var)
 {
         switch (var->type) {
         case String:
-                efree(var->datum.string);
+                grad_free(var->datum.string);
                 break;
         case Vector:
-                avl_free(var->datum.vector);
+                grad_avl_free(var->datum.vector);
                 break;
         }
 }
@@ -465,14 +449,14 @@ tempvar_free(Variable *var)
 }
 
 void
-radtest_send(int port, int code, Variable *var, Symtab *cntl)
+radtest_send(int port, int code, Variable *var, grad_symtab_t *cntl)
 {
-        RADIUS_REQ *auth;
+        grad_request_t *auth;
 	Variable *p;
-	VALUE_PAIR *pair;
+	grad_avp_t *pair;
 	
         if (reply_list)
-                avl_free(reply_list);
+                grad_avl_free(reply_list);
         reply_list = NULL;
         reply_code = 0;
         
@@ -486,94 +470,94 @@ radtest_send(int port, int code, Variable *var, Symtab *cntl)
 		pair = NULL;
 
 	if (!cntl) {
-		auth = rad_clt_send(srv_queue, port, code, pair);
+		auth = grad_client_send(srv_queue, port, code, pair);
 	} else {
 		int id;
 		u_char vector[AUTH_VECTOR_LEN];
 		int sflags = 0;
 		int retry = 1;
-		Variable *delay = (Variable*)sym_lookup(cntl, "delay");
+		Variable *delay = (Variable*)grad_sym_lookup(cntl, "delay");
 		
-		p = (Variable*)sym_lookup(cntl, "repeat");
+		p = (Variable*)grad_sym_lookup(cntl, "repeat");
 		if (p)
 			retry = p->datum.number;
-		p = (Variable*)sym_lookup(cntl, "id");
+		p = (Variable*)grad_sym_lookup(cntl, "id");
 		if (p) {
 			sflags |= RADCLT_ID;
 			id = p->datum.number;
 		}
-		p = (Variable*)sym_lookup(cntl, "keepauth");
+		p = (Variable*)grad_sym_lookup(cntl, "keepauth");
 		if (p && p->datum.number) 
 			sflags |= RADCLT_AUTHENTICATOR;
-		auth = rad_clt_send0(srv_queue,
-				     port,
-				     code,
-				     pair,
-				     0,
-				     &id,
-				     vector);
+		auth = grad_client_send0(srv_queue,
+					 port,
+					 code,
+					 pair,
+					 0,
+					 &id,
+					 vector);
 		while (--retry) {
 			if (delay)
 				sleep(delay->datum.number);
-			auth = rad_clt_send0(srv_queue,
-					     port,
-					     code,
-					     var->datum.vector,
-					     sflags,
-					     &id,
-					     vector);
+			auth = grad_client_send0(srv_queue,
+						 port,
+						 code,
+						 var->datum.vector,
+						 sflags,
+						 &id,
+						 vector);
 		}
 	}
 
 	if (!auth)
 		return;
         reply_code = auth->code;
-        var = (Variable*)sym_lookup(vartab, "REPLY_CODE");
+        var = (Variable*)grad_sym_lookup(vartab, "REPLY_CODE");
         var->type = Integer;
         var->datum.number = reply_code;
 
-        reply_list = rad_clt_decrypt_pairlist(avl_dup(auth->request),
-					      auth->vector, auth->secret);
+        reply_list = grad_client_decrypt_pairlist(grad_avl_dup(auth->request),
+						  auth->vector, auth->secret);
 
-        var = (Variable*)sym_lookup(vartab, "REPLY");
+        var = (Variable*)grad_sym_lookup(vartab, "REPLY");
         var->type = Vector;
         var->datum.vector = NULL;
         var->datum.vector = reply_list;
-        radreq_free(auth);
+        grad_request_free(auth);
 }
 
 /* FIXME: duplicated in radiusd/files.c */
 int
-comp_op(int op, int result)
+comp_op(enum grad_operator op, int result)
 {
         switch (op) {
         default:
-        case OPERATOR_EQUAL:
+        case grad_operator_equal:
                 if (result != 0)
                         return -1;
                 break;
 
-        case OPERATOR_NOT_EQUAL:
+        case grad_operator_not_equal:
                 if (result == 0)
                         return -1;
                 break;
 
-        case OPERATOR_LESS_THAN:
+        case grad_operator_less_than:
                 if (result >= 0)
                         return -1;
                 break;
 
-        case OPERATOR_GREATER_THAN:
+        case grad_operator_greater_than:
                 if (result <= 0)
                         return -1;
                 break;
                     
-        case OPERATOR_LESS_EQUAL:
+        case grad_operator_less_equal:
                 if (result > 0)
                         return -1;
                 break;
                         
-        case OPERATOR_GREATER_EQUAL:
+        case grad_operator_greater_equal:
                 if (result < 0)
                         return -1;
                 break;
@@ -582,12 +566,12 @@ comp_op(int op, int result)
 }
 
 int
-compare_lists(VALUE_PAIR *reply, VALUE_PAIR *sample)
+compare_lists(grad_avp_t *reply, grad_avp_t *sample)
 {
         int result = 0;
         
         for (; sample && result == 0; sample = sample->next) {
-                VALUE_PAIR *p;
+                grad_avp_t *p;
 
                 if (sample->attribute > 255)
                         continue;
